@@ -1,14 +1,18 @@
-import json
+from __future__ import annotations
+
 import os
 import shutil
+import textwrap
 from argparse import Namespace
+from collections import defaultdict
+from itertools import chain
 from pathlib import Path
-from typing import Dict, Iterable, List, Optional, Sequence
+from typing import Iterable, Mapping, Sequence, cast
 
+import atoml
 import click
-import tomlkit
-from pip._vendor.pkg_resources import safe_name
-from resolvelib.resolvers import ResolutionImpossible, ResolutionTooDeep
+from resolvelib.reporters import BaseReporter
+from resolvelib.resolvers import ResolutionImpossible, ResolutionTooDeep, Resolver
 
 from pdm import termui
 from pdm.cli.utils import (
@@ -16,21 +20,22 @@ from pdm.cli.utils import (
     find_importable_files,
     format_lockfile,
     format_resolution_impossible,
+    merge_dictionary,
     save_version_specifiers,
     set_env_in_reg,
+    translate_sections,
 )
 from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
 from pdm.formats import FORMATS
 from pdm.formats.base import array_of_inline_tables, make_array, make_inline_table
 from pdm.installers.installers import format_dist
-from pdm.models.builders import EnvBuilder
 from pdm.models.candidates import Candidate
-from pdm.models.in_process import get_python_version
+from pdm.models.python import PythonInfo
 from pdm.models.requirements import Requirement, parse_requirement, strip_extras
 from pdm.models.specifiers import get_specifier
 from pdm.project import Project
 from pdm.resolver import resolve
-from pdm.utils import get_python_version_string, setdefault
+from pdm.utils import normalize_name
 
 PEP582_PATH = os.path.join(os.path.dirname(os.path.dirname(__file__)), "pep582")
 
@@ -38,17 +43,11 @@ PEP582_PATH = os.path.join(os.path.dirname(os.path.dirname(__file__)), "pep582")
 def do_lock(
     project: Project,
     strategy: str = "all",
-    tracked_names: Optional[Iterable[str]] = None,
-    requirements: Optional[List[Requirement]] = None,
-) -> Dict[str, Candidate]:
-    """Performs the locking process and update lockfile.
-
-    :param project: the project instance
-    :param strategy: update strategy: reuse/eager/all
-    :param tracked_names: required when using eager strategy
-    :param requirements: An optional dictionary of requirements, read from pyproject
-        if not given.
-    """
+    tracked_names: Iterable[str] | None = None,
+    requirements: list[Requirement] | None = None,
+    dry_run: bool = False,
+) -> dict[str, Candidate]:
+    """Performs the locking process and update lockfile."""
     check_project_file(project)
     # TODO: multiple dependency definitions for the same package.
     provider = project.get_provider(strategy, tracked_names)
@@ -63,7 +62,7 @@ def do_lock(
         # any message is thrown to the output.
         with ui.open_spinner(title="Resolving dependencies", spinner="dots") as spin:
             reporter = project.get_reporter(requirements, tracked_names, spin)
-            resolver = project.core.resolver_class(provider, reporter)
+            resolver: Resolver = project.core.resolver_class(provider, reporter)
             try:
                 mapping, dependencies, summaries = resolve(
                     resolver,
@@ -88,83 +87,113 @@ def do_lock(
             else:
                 data = format_lockfile(mapping, dependencies, summaries)
                 spin.succeed(f"{termui.Emoji.LOCK} Lock successful")
-    project.write_lockfile(data)
 
+    project.write_lockfile(data, write=not dry_run)
+
+    return mapping
+
+
+def resolve_candidates_from_lockfile(
+    project: Project, requirements: Iterable[Requirement]
+) -> dict[str, Candidate]:
+    ui = project.core.ui
+    resolve_max_rounds = int(project.config["strategy.resolve_max_rounds"])
+    reqs = [
+        req
+        for req in requirements
+        if not req.marker or req.marker.evaluate(project.environment.marker_environment)
+    ]
+    with ui.logging("install-resolve"):
+        with ui.open_spinner("Resolving packages from lockfile..."):
+            reporter = BaseReporter()
+            provider = project.get_provider(for_install=True)
+            resolver: Resolver = project.core.resolver_class(provider, reporter)
+            mapping, *_ = resolve(
+                resolver,
+                reqs,
+                project.environment.python_requires,
+                resolve_max_rounds,
+            )
     return mapping
 
 
 def do_sync(
     project: Project,
+    *,
     sections: Sequence[str] = (),
-    dev: bool = False,
+    dev: bool = True,
     default: bool = True,
     dry_run: bool = False,
-    clean: Optional[bool] = None,
+    clean: bool = False,
+    requirements: list[Requirement] | None = None,
+    tracked_names: Sequence[str] | None = None,
+    no_editable: bool = False,
+    no_self: bool = False,
 ) -> None:
-    """Synchronize project
-
-    :param project: The project instance.
-    :param sections: A tuple of optional sections to be synced.
-    :param dev: whether to include dev-dependencies.
-    :param default: whether to include default dependencies.
-    :param dry_run: Print actions without actually running them.
-    :param clean: whether to remove unneeded packages.
-    """
-    if not project.lockfile_file.exists():
-        raise ProjectError("Lock file does not exist, nothing to sync")
-    clean = default if clean is None else clean
-    candidates = {}
-    for section in sections:
-        if section not in list(project.iter_sections()):
-            raise PdmUsageError(
-                f"Section {termui.green(repr(section))} doesn't exist "
-                "in the pyproject.toml"
+    """Synchronize project"""
+    if requirements is None:
+        if not project.lockfile_file.exists():
+            raise ProjectError("Lock file does not exist, nothing to sync")
+        elif not project.is_lockfile_compatible():
+            project.core.ui.echo(
+                "Lock file version is not compatible with PDM, "
+                "install may fail, please regenerate the pdm.lock",
+                err=True,
             )
-        candidates.update(project.get_locked_candidates(section))
-    if dev:
-        candidates.update(project.get_locked_candidates("dev"))
-    if default:
-        candidates.update(project.get_locked_candidates())
-    handler = project.core.synchronizer_class(candidates, project.environment)
-    handler.synchronize(clean=clean, dry_run=dry_run)
+        elif not project.is_lockfile_hash_match():
+            project.core.ui.echo(
+                "Lock file hash doesn't match pyproject.toml, packages may be outdated",
+                err=True,
+            )
+        sections = translate_sections(project, default, dev, sections or ())
+        requirements = []
+        for section in sections:
+            requirements.extend(project.get_dependencies(section).values())
+    candidates = resolve_candidates_from_lockfile(project, requirements)
+    if tracked_names and dry_run:
+        candidates = {
+            name: c for name, c in candidates.items() if name in tracked_names
+        }
+
+    handler = project.core.synchronizer_class(
+        candidates,
+        project.environment,
+        clean,
+        dry_run,
+        no_editable=no_editable,
+        install_self=not no_self and "default" in sections,
+    )
+    handler.synchronize()
 
 
 def do_add(
     project: Project,
     dev: bool = False,
-    section: Optional[str] = None,
+    section: str | None = None,
     sync: bool = True,
     save: str = "compatible",
     strategy: str = "reuse",
     editables: Iterable[str] = (),
     packages: Iterable[str] = (),
+    no_editable: bool = False,
+    no_self: bool = False,
 ) -> None:
-    """Add packages and install
-
-    :param project: the project instance
-    :param dev: add to dev dependencies section
-    :param section: specify section to be add to
-    :param sync: whether to install added packages
-    :param save: save strategy
-    :param strategy: update strategy
-    :param editables: editable requirements
-    :param packages: normal requirements
-    """
+    """Add packages and install"""
     check_project_file(project)
     if not editables and not packages:
         raise PdmUsageError("Must specify at least one package or editable package.")
-    section = "dev" if dev else section or "default"
-    tracked_names = set()
-    requirements = {}
+    if not section:
+        section = "dev" if dev else "default"
+    tracked_names: set[str] = set()
+    requirements: dict[str, Requirement] = {}
     for r in [parse_requirement(line, True) for line in editables] + [
         parse_requirement(line) for line in packages
     ]:
         key = r.identify()
-        r.from_section = section
         tracked_names.add(key)
         requirements[key] = r
     project.core.ui.echo(
-        f"Adding packages to {section} dependencies: "
+        f"Adding packages to {section} {'dev-' if dev else ''}dependencies: "
         + ", ".join(termui.green(key or "", bold=True) for key in requirements)
     )
     all_dependencies = project.all_dependencies
@@ -173,8 +202,8 @@ def do_add(
     resolved = do_lock(project, strategy, tracked_names, reqs)
 
     # Update dependency specifiers and lockfile hash.
-    save_version_specifiers(requirements, resolved, save)
-    project.add_dependencies(requirements)
+    save_version_specifiers({section: requirements}, resolved, save)
+    project.add_dependencies(requirements, section, dev)
     lockfile = project.lockfile
     project.write_lockfile(lockfile, False)
 
@@ -182,85 +211,97 @@ def do_add(
         do_sync(
             project,
             sections=(section,),
-            dev=False,
             default=False,
-            dry_run=False,
-            clean=False,
+            no_editable=no_editable,
+            no_self=no_self,
         )
 
 
 def do_update(
     project: Project,
-    dev: bool = False,
+    *,
+    dev: bool | None = None,
     sections: Sequence[str] = (),
     default: bool = True,
     strategy: str = "reuse",
     save: str = "compatible",
     unconstrained: bool = False,
+    top: bool = False,
+    dry_run: bool = False,
     packages: Sequence[str] = (),
+    no_editable: bool = False,
+    no_self: bool = False,
 ) -> None:
-    """Update specified packages or all packages
-
-    :param project: The project instance
-    :param dev: whether to update dev dependencies
-    :param sections: update specified sections
-    :param default: update default
-    :param strategy: update strategy (reuse/eager)
-    :param save: save strategy (compatible/exact/wildcard)
-    :param unconstrained: ignore version constraint
-    :param packages: specified packages to update
-    :return: None
-    """
+    """Update specified packages or all packages"""
     check_project_file(project)
-    if len(packages) > 0 and (len(sections) > 1 or not default):
+    if len(packages) > 0 and (top or len(sections) > 1 or not default):
         raise PdmUsageError(
-            "packages argument can't be used together with multiple -s or --no-default."
+            "packages argument can't be used together with multiple -s or "
+            "--no-default and --top."
         )
-    if not packages:
-        if unconstrained:
-            raise PdmUsageError(
-                "--unconstrained must be used with package names given."
-            )
-        # pdm update with no packages given, same as 'lock' + 'sync'
-        do_lock(project)
-        do_sync(project, sections, dev, default, clean=False)
-        return
-    section = sections[0] if sections else ("dev" if dev else "default")
     all_dependencies = project.all_dependencies
-    dependencies = all_dependencies[section]
-    updated_deps = {}
-    tracked_names = set()
-    for name in packages:
-        matched_name = next(
-            filter(
-                lambda k: safe_name(strip_extras(k)[0]).lower()
-                == safe_name(name).lower(),
-                dependencies.keys(),
-            ),
-            None,
-        )
-        if not matched_name:
-            raise ProjectError(
-                "{} does not exist in {} dependencies.".format(
-                    termui.green(name, bold=True), section
+    updated_deps: dict[str, dict[str, Requirement]] = defaultdict(dict)
+    install_dev = True if dev is None else dev
+    if not packages:
+        sections = translate_sections(project, default, install_dev, sections or ())
+        for section in sections:
+            updated_deps[section] = all_dependencies[section]
+    else:
+        section = sections[0] if sections else ("dev" if dev else "default")
+        dependencies = all_dependencies[section]
+        for name in packages:
+            matched_name = next(
+                filter(
+                    lambda k: normalize_name(strip_extras(k)[0])
+                    == normalize_name(name),
+                    dependencies.keys(),
+                ),
+                None,
+            )
+            if not matched_name:
+                raise ProjectError(
+                    "{} does not exist in {} {}dependencies.".format(
+                        termui.green(name, bold=True), section, "dev-" if dev else ""
+                    )
+                )
+            updated_deps[section][matched_name] = dependencies[matched_name]
+        project.core.ui.echo(
+            "Updating packages: {}.".format(
+                ", ".join(
+                    termui.green(v, bold=True)
+                    for v in chain.from_iterable(updated_deps.values())
                 )
             )
-        if unconstrained:
-            dependencies[matched_name].specifier = get_specifier("")
-        tracked_names.add(matched_name)
-        updated_deps[matched_name] = dependencies[matched_name]
-    project.core.ui.echo(
-        "Updating packages: {}.".format(
-            ", ".join(termui.green(v, bold=True) for v in tracked_names)
         )
-    )
-    reqs = [r for deps in all_dependencies.values() for r in deps.values()]
-    resolved = do_lock(project, strategy, tracked_names, reqs)
-    do_sync(project, sections=(section,), default=False, clean=False)
     if unconstrained:
+        for deps in updated_deps.values():
+            for dep in deps.values():
+                dep.specifier = get_specifier("")
+    reqs = [r for deps in all_dependencies.values() for r in deps.values()]
+    resolved = do_lock(
+        project,
+        strategy if top or packages else "all",
+        chain.from_iterable(updated_deps.values()),
+        reqs,
+        dry_run=dry_run,
+    )
+    do_sync(
+        project,
+        sections=sections,
+        dev=install_dev,
+        default=default,
+        clean=False,
+        dry_run=dry_run,
+        requirements=[r for deps in updated_deps.values() for r in deps.values()],
+        tracked_names=list(chain.from_iterable(updated_deps.values())) if top else None,
+        no_editable=no_editable,
+        no_self=no_self,
+    )
+    if unconstrained and not dry_run:
         # Need to update version constraints
         save_version_specifiers(updated_deps, resolved, save)
-        project.add_dependencies(updated_deps)
+        for section, deps in updated_deps.items():
+            project.add_dependencies(deps, section, dev or False)
         lockfile = project.lockfile
         project.write_lockfile(lockfile, False)
 
@@ -268,29 +309,24 @@ def do_update(
 def do_remove(
     project: Project,
     dev: bool = False,
-    section: Optional[str] = None,
+    section: str | None = None,
     sync: bool = True,
     packages: Sequence[str] = (),
-):
-    """Remove packages from working set and pyproject.toml
-
-    :param project: The project instance
-    :param dev: Remove package from dev-dependencies
-    :param section: Remove package from given section
-    :param sync: Whether perform syncing action
-    :param packages: Package names to be removed
-    :return: None
-    """
+    no_editable: bool = False,
+    no_self: bool = False,
+) -> None:
+    """Remove packages from working set and pyproject.toml"""
     check_project_file(project)
     if not packages:
         raise PdmUsageError("Must specify at least one package to remove.")
-    section = "dev" if dev else section or "default"
+    if not section:
+        section = "dev" if dev else "default"
     if section not in list(project.iter_sections()):
-        raise ProjectError(f"No {section} dependencies given in pyproject.toml.")
+        raise ProjectError(f"No-exist section {section}")
 
-    deps = project.get_pyproject_dependencies(section)
+    deps = project.get_pyproject_dependencies(section, dev)
     project.core.ui.echo(
-        f"Removing packages from {section} dependencies: "
+        f"Removing packages from {section} {'dev-' if dev else ''}dependencies: "
         + ", ".join(str(termui.green(name, bold=True)) for name in packages)
     )
     for name in packages:
@@ -310,7 +346,14 @@ def do_remove(
     project.write_pyproject()
     do_lock(project, "reuse")
     if sync:
-        do_sync(project, sections=(section,), default=False, clean=True)
+        do_sync(
+            project,
+            sections=(section,),
+            default=False,
+            clean=True,
+            no_editable=no_editable,
+            no_self=no_self,
+        )
 
 
 def do_list(project: Project, graph: bool = False, reverse: bool = False) -> None:
@@ -320,11 +363,7 @@ def do_list(project: Project, graph: bool = False, reverse: bool = False) -> Non
     :param graph: whether to display a graph.
     :param reverse: whether to display reverse graph.
     """
-    from pdm.cli.utils import (
-        build_dependency_graph,
-        format_dependency_graph,
-        format_reverse_dependency_graph,
-    )
+    from pdm.cli.utils import build_dependency_graph, format_dependency_graph
 
     check_project_file(project)
     working_set = project.environment.get_working_set()
@@ -333,11 +372,9 @@ def do_list(project: Project, graph: bool = False, reverse: bool = False) -> Non
     if graph:
         with project.environment.activate():
             dep_graph = build_dependency_graph(working_set)
-        if reverse:
-            graph = format_reverse_dependency_graph(project, dep_graph)
-        else:
-            graph = format_dependency_graph(project, dep_graph)
-        project.core.ui.echo(graph)
+        project.core.ui.echo(
+            format_dependency_graph(project, dep_graph, reverse=reverse)
+        )
     else:
         rows = [
             (termui.green(k, bold=True), format_dist(v))
@@ -352,11 +389,13 @@ def do_build(
     wheel: bool = True,
     dest: str = "dist",
     clean: bool = True,
-):
+    config_settings: Mapping[str, str] | None = None,
+) -> None:
     """Build artifacts for distribution."""
+    from pdm.builders import EnvSdistBuilder, EnvWheelBuilder
+
     if project.is_global:
         raise ProjectError("Not allowed to build based on the global project.")
-    check_project_file(project)
     if not wheel and not sdist:
         project.core.ui.echo("All artifacts are disabled, nothing to do.", err=True)
         return
@@ -364,16 +403,18 @@ def do_build(
         dest = project.root.joinpath(dest).as_posix()
     if clean:
         shutil.rmtree(dest, ignore_errors=True)
-    with project.core.ui.logging("build"), EnvBuilder(
-        project.root, project.environment
-    ) as builder:
+    with project.core.ui.logging("build"):
         if sdist:
             project.core.ui.echo("Building sdist...")
-            loc = builder.build_sdist(dest)
+            loc = EnvSdistBuilder(project.root, project.environment).build(
+                dest, config_settings
+            )
             project.core.ui.echo(f"Built sdist at {loc}")
         if wheel:
             project.core.ui.echo("Building wheel...")
-            loc = builder.build_wheel(dest)
+            loc = EnvWheelBuilder(project.root, project.environment).build(
+                dest, config_settings
+            )
             project.core.ui.echo(f"Built wheel at {loc}")
 
 
@@ -396,7 +437,6 @@ def do_init(
             "license": make_inline_table({"text": license}),
             "urls": {"homepage": ""},
             "dependencies": make_array([], True),
-            "dev-dependencies": make_array([], True),
             "requires-python": python_requires,
             "dynamic": ["classifiers"],
         },
@@ -407,22 +447,21 @@ def do_init(
     if not project.pyproject:
         project._pyproject = data
     else:
-        project._pyproject["project"] = data["project"]
-        project._pyproject["build-system"] = data["build-system"]
+        project._pyproject["project"] = data["project"]  # type: ignore
+        project._pyproject["build-system"] = data["build-system"]  # type: ignore
     project.write_pyproject()
 
 
-def do_use(
-    project: Project, python: Optional[str] = "", first: Optional[bool] = False
-) -> None:
+def do_use(project: Project, python: str = "", first: bool = False) -> None:
     """Use the specified python version and save in project config.
     The python can be a version string or interpreter path.
     """
 
-    def version_matcher(py_version):
+    def version_matcher(py_version: PythonInfo) -> bool:
         return project.python_requires.contains(str(py_version.version))
 
-    python = python.strip()
+    if python:
+        python = python.strip()
 
     found_interpreters = list(
         dict.fromkeys(filter(version_matcher, project.find_interpreters(python)))
@@ -434,11 +473,8 @@ def do_use(
     else:
         project.core.ui.echo("Please enter the Python interpreter to use")
         for i, py_version in enumerate(found_interpreters):
-            python_version = str(py_version.version)
-            is_64bit = py_version.get_architecture() == "64bit"
-            version_string = get_python_version_string(python_version, is_64bit)
             project.core.ui.echo(
-                f"{i}. {termui.green(py_version.executable)} ({version_string})"
+                f"{i}. {termui.green(py_version.executable)} ({py_version.identifier})"
             )
         selection = click.prompt(
             "Please select:",
@@ -450,58 +486,27 @@ def do_use(
 
     old_path = project.config.get("python.path")
     new_path = selected_python.executable
-    python_version = str(selected_python.version)
-    is_64bit = selected_python.get_architecture() == "64bit"
     project.core.ui.echo(
         "Using Python interpreter: {} ({})".format(
             termui.green(str(new_path)),
-            get_python_version_string(python_version, is_64bit),
+            selected_python.identifier,
         )
     )
-    project.python_executable = new_path
-    if old_path and Path(old_path) != Path(new_path) and not project.is_global:
+    project.python = selected_python
+    if (
+        old_path
+        and Path(old_path) != Path(new_path)
+        and not project.environment.is_global
+    ):
         project.core.ui.echo(termui.cyan("Updating executable scripts..."))
         project.environment.update_shebangs(new_path)
 
 
-def do_info(
-    project: Project,
-    python: bool = False,
-    show_project: bool = False,
-    env: bool = False,
-) -> None:
-    """Show project information."""
-    check_project_file(project)
-    python_path = project.python_executable
-    python_version, is_64bit = get_python_version(python_path, True)
-    if not python and not show_project and not env:
-        rows = [
-            (termui.cyan("PDM version:", bold=True), project.core.version),
-            (
-                termui.cyan("Python Interpreter:", bold=True),
-                python_path
-                + f" ({get_python_version_string(python_version, is_64bit)})",
-            ),
-            (termui.cyan("Project Root:", bold=True), project.root.as_posix()),
-        ]
-        project.core.ui.display_columns(rows)
-        return
-
-    if python:
-        project.core.ui.echo(python_path)
-    if show_project:
-        project.core.ui.echo(project.root.as_posix())
-    if env:
-        project.core.ui.echo(
-            json.dumps(project.environment.marker_environment, indent=2)
-        )
-
-
 def do_import(
     project: Project,
     filename: str,
-    format: Optional[str] = None,
-    options: Optional[Namespace] = None,
+    format: str | None = None,
+    options: Namespace | None = None,
 ) -> None:
     """Import project metadata from given file.
 
@@ -524,26 +529,27 @@ def do_import(
     if options is None:
         options = Namespace(dev=False, section=None)
     project_data, settings = FORMATS[key].convert(project, filename, options)
-    pyproject = project.pyproject or tomlkit.document()
+    pyproject = project.pyproject or atoml.document()
 
-    if "tool" not in pyproject or "pdm" not in pyproject["tool"]:
-        setdefault(pyproject, "tool", {})["pdm"] = tomlkit.table()
-
-    pyproject["tool"]["pdm"].update(settings)
+    if "tool" not in pyproject or "pdm" not in pyproject["tool"]:  # type: ignore
+        pyproject.setdefault("tool", {})["pdm"] = atoml.table()
 
     if "project" not in pyproject:
-        pyproject.add("project", tomlkit.table())
-        pyproject["project"].add(tomlkit.comment("PEP 621 project metadata"))
-        pyproject["project"].add(
-            tomlkit.comment("See https://www.python.org/dev/peps/pep-0621/")
+        pyproject.add("project", atoml.table())  # type: ignore
+        pyproject["project"].add(  # type: ignore
+            atoml.comment("PEP 621 project metadata")
+        )
+        pyproject["project"].add(  # type: ignore
+            atoml.comment("See https://www.python.org/dev/peps/pep-0621/")
         )
 
-    pyproject["project"].update(project_data)
+    merge_dictionary(pyproject["project"], project_data)  # type: ignore
+    merge_dictionary(pyproject["tool"]["pdm"], settings)  # type: ignore
     pyproject["build-system"] = {
         "requires": ["pdm-pep517"],
         "build-backend": "pdm.pep517.api",
     }
-    project.pyproject = pyproject
+    project.pyproject = cast(dict, pyproject)
     project.write_pyproject()
 
 
@@ -571,10 +577,10 @@ def ask_for_import(project: Project) -> None:
     if int(choice) == len(importable_files):
         return
     key, filepath = importable_files[int(choice)]
-    do_import(project, filepath, key)
+    do_import(project, str(filepath), key)
 
 
-def print_pep582_command(ui: termui.UI, shell: str = "AUTO"):
+def print_pep582_command(ui: termui.UI, shell: str = "AUTO") -> None:
     """Print the export PYTHONPATH line to be evaluated by the shell."""
     import shellingham
 
@@ -600,11 +606,39 @@ def print_pep582_command(ui: termui.UI, shell: str = "AUTO"):
         shell = shellingham.detect_shell()[0]
     shell = shell.lower()
     if shell in ("zsh", "bash"):
-        result = f"export PYTHONPATH='{lib_path}':$PYTHONPATH"
+        result = textwrap.dedent(
+            f"""
+            if [ -n "$PYTHONPATH" ]; then
+                export PYTHONPATH='{lib_path}':$PYTHONPATH
+            else
+                export PYTHONPATH='{lib_path}'
+            fi
+            """
+        ).strip()
     elif shell == "fish":
-        result = f"set -x PYTHONPATH '{lib_path}' $PYTHONPATH"
+        result = textwrap.dedent(
+            f"""
+            if test -n "$PYTHONPATH"
+                set -x PYTHONPATH '{lib_path}' $PYTHONPATH
+            else
+                set -x PYTHONPATH '{lib_path}'
+            end
+            """
+        ).strip()
     elif shell in ("tcsh", "csh"):
-        result = f"setenv PYTHONPATH '{lib_path}':$PYTHONPATH"
+        result = textwrap.dedent(
+            f"""
+            if ( $?PYTHONPATH ) then
+                if ( "$PYTHONPATH" != "" ) then
+                    setenv PYTHONPATH '{lib_path}':$PYTHONPATH
+                else
+                    setenv PYTHONPATH '{lib_path}'
+                endif
+            else
+                setenv PYTHONPATH '{lib_path}'
+            endif
+            """
+        ).strip()
     else:
         raise PdmUsageError(
             f"Unsupported shell: {shell}, please specify another shell "
@@ -613,21 +647,48 @@ def print_pep582_command(ui: termui.UI, shell: str = "AUTO"):
     ui.echo(result)
 
 
-def migrate_pyproject(project: Project):
+def migrate_pyproject(project: Project) -> None:
     """Migrate the legacy pyproject format to PEP 621"""
 
-    if (
-        not project.pyproject_file.exists()
-        or not FORMATS["legacy"].check_fingerprint(project, project.pyproject_file)
-        or "project" in project.pyproject
+    if project.pyproject and "project" in project.pyproject:
+        pyproject = project.pyproject
+        settings = {}
+        updated_fields = []
+        for field in ("includes", "excludes", "build", "package-dir"):
+            if field in pyproject["project"]:
+                updated_fields.append(field)
+                settings[field] = pyproject["project"].pop(field)
+        if "dev-dependencies" in pyproject["project"]:
+            if pyproject["project"]["dev-dependencies"]:
+                settings["dev-dependencies"] = {
+                    "dev": pyproject["project"]["dev-dependencies"]
+                }
+            del pyproject["project"]["dev-dependencies"]
+            updated_fields.append("dev-dependencies")
+        if updated_fields:
+            if "tool" not in pyproject or "pdm" not in pyproject["tool"]:
+                pyproject.setdefault("tool", {})["pdm"] = atoml.table()
+            pyproject["tool"]["pdm"].update(settings)
+            project.pyproject = pyproject
+            project.write_pyproject()
+            project.core.ui.echo(
+                f"{termui.yellow('[AUTO-MIGRATION]')} These fields are moved from "
+                f"[project] to [tool.pdm] table: {updated_fields}",
+                err=True,
+            )
+        return
+
+    if not project.pyproject_file.exists() or not FORMATS["legacy"].check_fingerprint(
+        project, project.pyproject_file
     ):
         return
 
     project.core.ui.echo(
-        termui.yellow("Legacy [tool.pdm] metadata detected, migrating to PEP 621..."),
+        f"{termui.yellow('[AUTO-MIGRATION]')} Legacy pdm 0.x metadata detected, "
+        "migrating to PEP 621...",
         err=True,
     )
-    do_import(project, project.pyproject_file, "legacy")
+    do_import(project, str(project.pyproject_file), "legacy")
     project.core.ui.echo(
         termui.green("pyproject.toml")
         + termui.yellow(
