@@ -20,13 +20,14 @@ class Command(BaseCommand):
         RemoveCommand.register_to(subparsers, "remove")
         ListCommand.register_to(subparsers, "list")
         InfoCommand.register_to(subparsers, "info")
+        parser.set_defaults(search_parent=False)
         self.parser = parser
 
     def handle(self, project: Project, options: argparse.Namespace) -> None:
         self.parser.print_help()
 
 
-def format_size(size: int) -> int:
+def format_size(size: float) -> str:
     if size > 1000 * 1000:
         return "{:.1f} MB".format(size / 1000.0 / 1000)
     elif size > 10 * 1000:
@@ -42,10 +43,10 @@ def remove_cache_files(project: Project, pattern: str) -> None:
         raise PdmUsageError("Please provide a pattern")
 
     if pattern == "*":
-        files = list(find_files(project.cache_dir, pattern))
+        files = list(find_files(project.cache_dir.as_posix(), pattern))
     else:
         # Only remove wheel files which specific pattern is given
-        files = list(find_files(project.cache("wheels"), pattern))
+        files = list(find_files(project.cache("wheels").as_posix(), pattern))
 
     if not files:
         raise PdmUsageError("No matching files found")
@@ -78,7 +79,7 @@ class ClearCommand(BaseCommand):
         with project.core.ui.open_spinner(
             f"Clearing {options.type or 'all'} caches..."
         ) as spinner:
-            files = list(find_files(cache_parent, "*"))
+            files = list(find_files(cache_parent.as_posix(), "*"))
             for file in files:
                 os.unlink(file)
             spinner.succeed(f"{len(files)} file{'s' if len(files) > 1 else ''} removed")
@@ -107,9 +108,10 @@ class ListCommand(BaseCommand):
         )
 
     def handle(self, project: Project, options: argparse.Namespace) -> None:
-        rows = []
-        for file in find_files(project.cache("wheels"), options.pattern):
-            rows.append((format_size(file_size(file)), os.path.basename(file)))
+        rows = [
+            (format_size(file_size(file)), os.path.basename(file))
+            for file in find_files(project.cache("wheels").as_posix(), options.pattern)
+        ]
         project.core.ui.display_columns(rows, [">Size", "Filename"])
 
 
@@ -119,12 +121,11 @@ class InfoCommand(BaseCommand):
     arguments = [verbose_option]
 
     def handle(self, project: Project, options: argparse.Namespace) -> None:
-        output = []
-        with project.core.ui.open_spinner("Calculating cache files") as spinner:
-            output.append(
+        with project.core.ui.open_spinner("Calculating cache files"):
+            output = [
                 f"{termui.cyan('Cache Root')}: {project.cache_dir}, "
-                f"Total size: {format_size(directory_size(project.cache_dir))}"
-            )
+                f"Total size: {format_size(directory_size(str(project.cache_dir)))}"
+            ]
             for name, description in [
                 ("hashes", "File Hashe Cache"),
                 ("http", "HTTP Cache"),
@@ -132,8 +133,9 @@ class InfoCommand(BaseCommand):
                 ("metadata", "Metadata Cache"),
             ]:
                 cache_location = project.cache(name)
-                files = list(find_files(cache_location, "*"))
-                size = directory_size(cache_location)
+                files = list(find_files(cache_location.as_posix(), "*"))
+                size = directory_size(cache_location.as_posix())
                 output.append(f"  {termui.cyan(description)}: {cache_location}")
                 output.append(f"    Files: {len(files)}, Size: {format_size(size)}")
-            spinner.stop_and_persist("\n".join(output))
+
+        project.core.ui.echo("\n".join(output))
