@@ -1,9 +1,16 @@
+from __future__ import annotations
+
 import argparse
 from pathlib import Path
+from typing import Iterable
 
+from pdm.cli.actions import resolve_candidates_from_lockfile
 from pdm.cli.commands.base import BaseCommand
 from pdm.cli.options import sections_group
+from pdm.cli.utils import translate_sections
 from pdm.formats import FORMATS
+from pdm.models.candidates import Candidate
+from pdm.models.requirements import Requirement
 from pdm.project import Project
 
 
@@ -38,28 +45,36 @@ class Command(BaseCommand):
         )
 
     def handle(self, project: Project, options: argparse.Namespace) -> None:
-        candidates = []
+        sections: list[str] = list(options.sections)
         if options.pyproject:
             options.hashes = False
-        if options.default:
-            # Don't include self candidate
-            if options.pyproject:
-                temp = project.dependencies
-            else:
-                temp = project.get_locked_candidates()
-                temp.pop(project.meta.name, None)
-            candidates.extend(temp.values())
-        if options.dev:
-            if options.pyproject:
-                candidates.extend(project.dev_dependencies.values())
-            else:
-                candidates.extend(project.get_locked_candidates("dev").values())
-        for section in options.sections:
-            if options.pyproject:
-                candidates.extend(project.get_dependencies(section).values())
-            else:
-                candidates.extend(project.get_locked_candidates(section).values())
-        content = FORMATS[options.format].export(project, candidates, options)
+        sections = translate_sections(
+            project,
+            options.default,
+            options.dev,
+            options.sections or (),
+        )
+        requirements: dict[str, Requirement] = {}
+        packages: Iterable[Requirement] | Iterable[Candidate]
+        for section in sections:
+            requirements.update(project.get_dependencies(section))
+        if options.pyproject:
+            packages = requirements.values()
+        else:
+            project.core.ui.echo(
+                "The exported requirements file is no longer cross-platform. "
+                "Using it on other platforms may cause unexpected result.",
+                fg="yellow",
+                err=True,
+            )
+            candidates = resolve_candidates_from_lockfile(
+                project, requirements.values()
+            )
+            packages = candidates.values()
+
+        content = FORMATS[options.format].export(
+            project, packages, options
+        )  # type: ignore
         if options.output:
             Path(options.output).write_text(content)
         else:
