@@ -1,6 +1,7 @@
 import argparse
+import typing as t
 
-from pip._vendor.pkg_resources import safe_name
+from packaging.version import Version
 
 from pdm import termui
 from pdm.cli.commands.base import BaseCommand
@@ -8,14 +9,11 @@ from pdm.models.candidates import Candidate
 from pdm.models.project_info import ProjectInfo
 from pdm.models.requirements import parse_requirement
 from pdm.project import Project
-
-
-def normalize_package(name):
-    return safe_name(name).lower()
+from pdm.utils import normalize_name
 
 
 def filter_stable(candidate: Candidate) -> bool:
-    return not candidate.version.is_prerelease
+    return not t.cast(Version, candidate.version).is_prerelease
 
 
 class Command(BaseCommand):
@@ -24,7 +22,7 @@ class Command(BaseCommand):
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
         parser.add_argument(
             "package",
-            type=normalize_package,
+            type=normalize_name,
             help="Specify the package name",
         )
 
@@ -47,6 +45,7 @@ class Command(BaseCommand):
         installed = project.environment.get_working_set().get(package)
 
         metadata = latest.get_metadata()
+        assert metadata
         if metadata._legacy:
             result = ProjectInfo(dict(metadata._legacy.items()), True)
         else:
