@@ -1,9 +1,7 @@
+from __future__ import annotations
+
 import argparse
-
-import click
-
-from pdm import termui
-from pdm.project import Project
+from typing import Any
 
 
 class Option:
@@ -11,46 +9,49 @@ class Option:
     to parser.add_argument().
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         self.args = args
         self.kwargs = kwargs
 
-    def add_to_parser(self, parser: argparse.ArgumentParser) -> None:
+    def add_to_parser(self, parser: argparse._ActionsContainer) -> None:
         parser.add_argument(*self.args, **self.kwargs)
 
     def add_to_group(self, group: argparse._ArgumentGroup) -> None:
         group.add_argument(*self.args, **self.kwargs)
 
 
-class ArgumentGroup:
+class ArgumentGroup(Option):
     """A reusable argument group object which can call `add_argument()`
     to add more arguments. And itself will be registered to the parser later.
     """
 
     def __init__(
-        self, name=None, parser=None, is_mutually_exclusive=False, required=None
-    ):
+        self,
+        name: str = None,
+        is_mutually_exclusive: bool = False,
+        required: bool = None,
+    ) -> None:
         self.name = name
-        self.options = []
-        self.parser = parser
+        self.options: list[Option] = []
         self.required = required
         self.is_mutually_exclusive = is_mutually_exclusive
-        self.argument_group = None
 
-    def add_argument(self, *args, **kwargs):
-        self.options.append(Option(*args, **kwargs))
+    def add_argument(self, *args: Any, **kwargs: Any) -> None:
+        if args and isinstance(args[0], Option):
+            self.options.append(args[0])
+        else:
+            self.options.append(Option(*args, **kwargs))
 
-    def add_to_parser(self, parser):
+    def add_to_parser(self, parser: argparse._ActionsContainer) -> None:
+        group: argparse._ArgumentGroup
         if self.is_mutually_exclusive:
             group = parser.add_mutually_exclusive_group(required=self.required)
         else:
             group = parser.add_argument_group(self.name)
         for option in self.options:
             option.add_to_group(group)
-        self.argument_group = group
-        self.parser = parser
 
-    def add_to_group(self, group):
+    def add_to_group(self, group: argparse._ArgumentGroup) -> None:
         self.add_to_parser(group)
 
 
@@ -67,7 +68,7 @@ dry_run_option = Option(
     "--dry-run",
     action="store_true",
     default=False,
-    help="Only prints actions without actually running them",
+    help="Show the difference only and don't perform any action",
 )
 
 
@@ -79,31 +80,57 @@ pep582_option = Option(
     help="Print the command line to be eval'd by the shell",
 )
 
-sections_group = ArgumentGroup()
+install_group = ArgumentGroup("Install options")
+install_group.add_argument(
+    "--no-editable",
+    action="store_true",
+    dest="no_editable",
+    help="Install non-editable versions for all packages",
+)
+install_group.add_argument(
+    "--no-self",
+    action="store_true",
+    dest="no_self",
+    help="Don't install the project itself",
+)
+
+sections_group = ArgumentGroup("Dependencies selection")
 sections_group.add_argument(
     "-s",
     "--section",
     dest="sections",
     metavar="SECTION",
     action="append",
-    help="Specify section(s) to include",
+    help="Select section of optional-dependencies "
+    "or dev-dependencies(with -d). Can be supplied multiple times, "
+    'use ":all" to include all groups under the same species.',
     default=[],
 )
-sections_group.add_argument(
-    "-d",
-    "--dev",
-    default=False,
-    action="store_true",
-    help="Include dev dependencies",
-)
 sections_group.add_argument(
     "--no-default",
     dest="default",
     action="store_false",
     default=True,
-    help="Don't include dependencies from default seciton",
+    help="Don't include dependencies from default section",
 )
 
+dev_group = ArgumentGroup("dev", is_mutually_exclusive=True)
+dev_group.add_argument(
+    "-d",
+    "--dev",
+    default=True,
+    dest="dev",
+    action="store_true",
+    help="Select dev dependencies",
+)
+dev_group.add_argument(
+    "--prod",
+    "--production",
+    dest="dev",
+    action="store_false",
+    help="Unselect dev dependencies",
+)
+sections_group.options.append(dev_group)
 
 save_strategy_group = ArgumentGroup("save_strategy", is_mutually_exclusive=True)
 save_strategy_group.add_argument(
@@ -153,40 +180,31 @@ project_option = Option(
 )
 
 
-def deprecate_global_option(value) -> Project:
-    if value:
-        click.echo(
-            termui.red(
-                "DEPRECATION: -g/--global with argument is deprecated and will be "
-                "removed in v1.5.0, please use '-gp <PROJECT_PATH>' instead."
-            ),
-            err=True,
-        )
-    return Project.create_global(value)
-
-
 global_option = Option(
     "-g",
     "--global",
     dest="global_project",
-    nargs="?",
-    type=deprecate_global_option,
-    const=True,
-    help="Use the global project, the project root can be supplied with "
-    "'-p/--project' option",
+    action="store_true",
+    help="Use the global project, supply the project root with `-p` option",
 )
 
 clean_group = ArgumentGroup("clean", is_mutually_exclusive=True)
-clean_group.add_argument("--clean", action="store_true", help="clean unused packages")
 clean_group.add_argument(
-    "--no-clean", action="store_false", help="don't clean unused packages"
+    "--clean", action="store_true", dest="clean", help="clean unused packages"
+)
+clean_group.add_argument(
+    "--no-clean", action="store_false", dest="clean", help="don't clean unused packages"
 )
 
 sync_group = ArgumentGroup("sync", is_mutually_exclusive=True)
-sync_group.add_argument("--sync", action="store_true", help="sync packages")
-sync_group.add_argument("--no-sync", action="store_false", help="don't sync packages")
+sync_group.add_argument(
+    "--sync", action="store_true", dest="sync", help="sync packages"
+)
+sync_group.add_argument(
+    "--no-sync", action="store_false", dest="sync", help="don't sync packages"
+)
 
-packages_group = ArgumentGroup("packages")
+packages_group = ArgumentGroup("Packages")
 packages_group.add_argument(
     "-e",
     "--editable",
