@@ -2,17 +2,19 @@ from __future__ import annotations
 
 import argparse
 import os
+from argparse import Action
 from collections import ChainMap
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Any, Iterable, Mapping, MutableMapping, cast
 
+import atoml
 import cfonts
-import tomlkit
 from packaging.specifiers import SpecifierSet
+from pip._vendor.pkg_resources import Distribution
 from resolvelib.structs import DirectedGraph
 
 from pdm import termui
-from pdm.exceptions import ProjectError
+from pdm.exceptions import PdmUsageError, ProjectError
 from pdm.formats import FORMATS
 from pdm.formats.base import make_array, make_inline_table
 from pdm.models.environment import WorkingSet
@@ -21,16 +23,13 @@ from pdm.models.specifiers import get_specifier
 from pdm.project import Project
 
 if TYPE_CHECKING:
-    from typing import Dict, Iterable, List, Optional, Tuple
-
     from resolvelib.resolvers import RequirementInformation, ResolutionImpossible
-    from tomlkit.container import Container
 
     from pdm.models.candidates import Candidate
 
 
 class PdmFormatter(argparse.HelpFormatter):
-    def _format_action(self, action):
+    def _format_action(self, action: Action) -> str:
         # determine the required width and the entry label
         help_position = min(self._action_max_length + 2, self._max_help_position)
         help_width = max(self._width - help_position, 11)
@@ -44,13 +43,13 @@ class PdmFormatter(argparse.HelpFormatter):
 
         # short action name; start on the same line and pad two spaces
         elif len(action_header) <= action_width:
-            tup = self._current_indent, "", action_width, action_header
-            action_header = "%*s%-*s  " % tup
+            tup = self._current_indent, "", action_width, action_header  # type: ignore
+            action_header = "%*s%-*s  " % tup  # type: ignore
             indent_first = 0
 
         # long action name; start on the next line
         else:
-            tup = self._current_indent, "", action_header
+            tup = self._current_indent, "", action_header  # type: ignore
             action_header = "%*s%s\n" % tup
             indent_first = help_position
 
@@ -78,7 +77,7 @@ class PdmFormatter(argparse.HelpFormatter):
 
 
 class PdmParser(argparse.ArgumentParser):
-    def format_help(self):
+    def format_help(self) -> str:
         formatter = self._get_formatter()
 
         if getattr(self, "is_root", False):
@@ -99,7 +98,7 @@ class PdmParser(argparse.ArgumentParser):
 
         # usage
         formatter.add_usage(
-            self.usage,
+            self.usage or "",
             self._actions,
             self._mutually_exclusive_groups,
             prefix=termui.yellow("Usage", bold=True) + ": ",
@@ -125,35 +124,40 @@ class PdmParser(argparse.ArgumentParser):
 class Package:
     """An internal class for the convenience of dependency graph building."""
 
-    def __init__(self, name, version, requirements):
+    def __init__(
+        self, name: str, version: str, requirements: dict[str, Requirement]
+    ) -> None:
         self.name = name
         self.version = version  # if version is None, the dist is not installed.
         self.requirements = requirements
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash(self.name)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"<Package {self.name}=={self.version}>"
 
-    def __eq__(self, value):
+    def __eq__(self, value: object) -> bool:
+        if not isinstance(value, Package):
+            return False
         return self.name == value.name
 
 
 def build_dependency_graph(working_set: WorkingSet) -> DirectedGraph:
     """Build a dependency graph from locked result."""
-    graph = DirectedGraph()
+    graph: DirectedGraph[Package | None] = DirectedGraph()
     graph.add(None)  # sentinel parent of top nodes.
     node_with_extras = set()
 
-    def add_package(key, dist):
+    def add_package(key: str, dist: Distribution) -> Package:
         name, extras = strip_extras(key)
         extras = extras or ()
-        reqs = {}
+        reqs: dict[str, Requirement] = {}
         if dist:
-            requirements = [
-                Requirement.from_pkg_requirement(r) for r in dist.requires(extras)
-            ]
+            requirements = (
+                Requirement.from_pkg_requirement(r)
+                for r in dist.requires(extras)  # type: ignore
+            )
             for req in requirements:
                 reqs[req.identify()] = req
             version = dist.version
@@ -167,14 +171,16 @@ def build_dependency_graph(working_set: WorkingSet) -> DirectedGraph:
             graph.add(node)
 
             for k in reqs:
-                child = add_package(k, working_set.get(strip_extras(k)[0]))
+                child = add_package(
+                    k, cast(Distribution, working_set.get(strip_extras(k)[0]))
+                )
                 graph.connect(node, child)
 
         return node
 
     for k, dist in working_set.items():
         add_package(k, dist)
-    for node in graph._vertices.copy():
+    for node in list(graph):
         if node is not None and not list(graph.iter_parents(node)):
             # Top requirements
             if node.name in node_with_extras:
@@ -197,7 +203,7 @@ def format_package(
     package: Package,
     required: str = "",
     prefix: str = "",
-    visited=None,
+    visited: set[str] | None = None,
 ) -> str:
     """Format one package.
 
@@ -245,15 +251,14 @@ def format_package(
 def format_reverse_package(
     graph: DirectedGraph,
     package: Package,
-    child: Optional[Package] = None,
+    child: Package | None = None,
     requires: str = "",
     prefix: str = "",
-    visited=None,
-):
+    visited: set[str] | None = None,
+) -> str:
     """Format one package for output reverse dependency graph."""
     if visited is None:
         visited = set()
-    result = []
     version = (
         termui.red("[ not installed ]")
         if not package.version
@@ -271,11 +276,13 @@ def format_reverse_package(
         if not requires
         else f"[ requires: {requires} ]"
     )
-    result.append(f"{termui.green(package.name, bold=True)} {version} {requires}\n")
+    result = [f"{termui.green(package.name, bold=True)} {version} {requires}\n"]
     if package.name in visited:
         return "".join(result)
     visited.add(package.name)
-    parents = sorted(filter(None, graph.iter_parents(package)), key=lambda p: p.name)
+    parents: list[Package] = sorted(
+        filter(None, graph.iter_parents(package)), key=lambda p: p.name
+    )
     for i, parent in enumerate(parents):
         is_last = i == len(parents) - 1
         head = LAST_CHILD if is_last else NON_LAST_CHILD
@@ -291,7 +298,7 @@ def format_reverse_package(
     return "".join(result)
 
 
-def format_dependency_graph(project: Project, graph: DirectedGraph) -> str:
+def _format_forward_dependency_graph(project: Project, graph: DirectedGraph) -> str:
     """Format dependency graph for output."""
     content = []
     all_dependencies = ChainMap(*project.all_dependencies.values())
@@ -309,56 +316,69 @@ def format_dependency_graph(project: Project, graph: DirectedGraph) -> str:
     return "".join(content).strip()
 
 
-def format_reverse_dependency_graph(project: Project, graph: DirectedGraph) -> str:
+def _format_reverse_dependency_graph(
+    project: Project, graph: DirectedGraph[Package]
+) -> str:
     """Format reverse dependency graph for output."""
     leaf_nodes = sorted(
-        (node for node in graph._vertices if not list(graph.iter_children(node))),
+        (node for node in graph if not list(graph.iter_children(node))),
         key=lambda p: p.name,
     )
-    content = []
-    for node in leaf_nodes:
-        content.append(format_reverse_package(graph, node, prefix="", visited=set()))
+    content = [
+        format_reverse_package(graph, node, prefix="", visited=set())
+        for node in leaf_nodes
+    ]
     return "".join(content).strip()
 
 
+def format_dependency_graph(
+    project: Project, graph: DirectedGraph[Package], reverse: bool = False
+) -> str:
+    if reverse:
+        return _format_reverse_dependency_graph(project, graph)
+    else:
+        return _format_forward_dependency_graph(project, graph)
+
+
 def format_lockfile(
-    mapping: Dict[str, Candidate],
-    fetched_dependencies: Dict[str, List[Requirement]],
-    summary_collection: Dict[str, str],
-) -> Container:
+    mapping: dict[str, Candidate],
+    fetched_dependencies: dict[str, list[Requirement]],
+    summary_collection: dict[str, str],
+) -> dict:
     """Format lock file from a dict of resolved candidates, a mapping of dependencies
     and a collection of package summaries.
     """
-    packages = tomlkit.aot()
-    file_hashes = tomlkit.table()
+    packages = atoml.aot()
+    file_hashes = atoml.table()
     for k, v in sorted(mapping.items()):
-        base = tomlkit.table()
-        base.update(v.as_lockfile_entry())
+        base = atoml.table()
+        base.update(v.as_lockfile_entry())  # type: ignore
         base.add("summary", summary_collection[strip_extras(k)[0]])
-        deps = make_array([r.as_line() for r in fetched_dependencies[k]], True)
+        deps = make_array(sorted(r.as_line() for r in fetched_dependencies[k]), True)
         if len(deps) > 0:
             base.add("dependencies", deps)
-        packages.append(base)
+        packages.append(base)  # type: ignore
         if v.hashes:
-            key = f"{k} {v.version}"
-            array = tomlkit.array()
-            array.multiline(True)
+            key = f"{strip_extras(k)[0]} {v.version}"
+            if key in file_hashes:
+                continue
+            array = atoml.array().multiline(True)
             for filename, hash_value in v.hashes.items():
                 inline = make_inline_table({"file": filename, "hash": hash_value})
-                array.append(inline)
+                array.append(inline)  # type: ignore
             if array:
                 file_hashes.add(key, array)
-    doc = tomlkit.document()
-    doc.add("package", packages)
-    metadata = tomlkit.table()
+    doc = atoml.document()
+    doc.add("package", packages)  # type: ignore
+    metadata = atoml.table()
     metadata.add("files", file_hashes)
-    doc.add("metadata", metadata)
-    return doc
+    doc.add("metadata", metadata)  # type: ignore
+    return cast(dict, doc)
 
 
 def save_version_specifiers(
-    requirements: Dict[str, Requirement],
-    resolved: Dict[str, Candidate],
+    requirements: dict[str, dict[str, Requirement]],
+    resolved: dict[str, Candidate],
     save_strategy: str,
 ) -> None:
     """Rewrite the version specifiers according to the resolved result and save strategy
@@ -367,14 +387,15 @@ def save_version_specifiers(
     :param resolved: the resolved mapping
     :param save_strategy: compatible/wildcard/exact
     """
-    for name, r in requirements.items():
-        if r.is_named and not r.specifier:
-            if save_strategy == "exact":
-                r.specifier = get_specifier(f"=={resolved[name].version}")
-            elif save_strategy == "compatible":
-                version = str(resolved[name].version)
-                compatible_version = ".".join((version.split(".") + ["0"])[:2])
-                r.specifier = get_specifier(f"~={compatible_version}")
+    for reqs in requirements.values():
+        for name, r in reqs.items():
+            if r.is_named and not r.specifier:
+                if save_strategy == "exact":
+                    r.specifier = get_specifier(f"=={resolved[name].version}")
+                elif save_strategy == "compatible":
+                    version = str(resolved[name].version)
+                    compatible_version = ".".join((version.split(".") + ["0"])[:2])
+                    r.specifier = get_specifier(f"~={compatible_version}")
 
 
 def check_project_file(project: Project) -> None:
@@ -386,7 +407,7 @@ def check_project_file(project: Project) -> None:
         )
 
 
-def find_importable_files(project: Project) -> Iterable[Tuple[str, Path]]:
+def find_importable_files(project: Project) -> Iterable[tuple[str, Path]]:
     """Find all possible files that can be imported"""
     for filename in (
         "Pipfile",
@@ -425,11 +446,8 @@ def set_env_in_reg(env_name: str, value: str) -> None:
 
 
 def format_resolution_impossible(err: ResolutionImpossible) -> str:
-    causes: List[RequirementInformation] = err.causes
-    result = []
-    result.append(
-        "Unable to find a resolution that satisfies the following requirements:"
-    )
+    causes: list[RequirementInformation] = err.causes
+    result = ["Unable to find a resolution that satisfies the following requirements:"]
 
     for req, parent in causes:
         result.append(
@@ -442,3 +460,50 @@ def format_resolution_impossible(err: ResolutionImpossible) -> str:
         "set a narrower `requires-python` range in the pyproject.toml."
     )
     return "\n".join(result)
+
+
+def translate_sections(
+    project: Project, default: bool, dev: bool, sections: Iterable[str]
+) -> list[str]:
+    """Translate default, dev and sections containing ":all" into a list of sections"""
+    optional_groups = set(project.meta.optional_dependencies or [])
+    dev_groups = set(project.tool_settings.get("dev-dependencies", []))
+    sections_set = set(sections)
+    if dev is None:
+        dev = True
+    if sections_set & dev_groups:
+        if not dev:
+            raise PdmUsageError(
+                "--prod is not allowed with dev sections and should be left"
+            )
+    elif dev:
+        sections_set.update(dev_groups)
+    if ":all" in sections:
+        sections_set.discard(":all")
+        sections_set.update(optional_groups)
+    if default:
+        sections_set.add("default")
+    # Sorts the result in ascending order instead of in random order
+    # to make this function pure
+    invalid_sections = sections_set - set(project.iter_sections())
+    if invalid_sections:
+        project.core.ui.echo(
+            f"Ignoring non-existing sections: {invalid_sections}", fg="yellow", err=True
+        )
+        sections_set -= invalid_sections
+    return sorted(sections_set)
+
+
+def merge_dictionary(
+    target: MutableMapping[Any, Any], input: Mapping[Any, Any]
+) -> None:
+    """Merge the input dict with the target while preserving the existing values
+    properly. This will update the target dictionary in place.
+    """
+    for key, value in input.items():
+        if key not in target:
+            target[key] = value
+        elif isinstance(value, dict):
+            target[key].update(value)
+        elif isinstance(value, list):
+            target[key].extend(value)
