@@ -5,7 +5,7 @@ import importlib
 import os
 import pkgutil
 import sys
-from typing import Optional, Type
+from typing import Any, List, Optional, Type, cast
 
 import click
 from pip._vendor import pkg_resources
@@ -22,16 +22,18 @@ from pdm.models.repositories import PyPIRepository
 from pdm.project import Project
 from pdm.project.config import Config, ConfigItem
 
-COMMANDS_MODULE_PATH = importlib.import_module("pdm.cli.commands").__path__
+COMMANDS_MODULE_PATH: str = importlib.import_module(
+    "pdm.cli.commands"
+).__path__  # type: ignore
 
 
 class Core:
     """A high level object that manages all classes and configurations"""
 
-    def __init__(self):
-        try:
+    def __init__(self) -> None:
+        if sys.version_info >= (3, 8):
             import importlib.metadata as importlib_metadata
-        except ModuleNotFoundError:
+        else:
             import importlib_metadata
         self.version = importlib_metadata.version(__name__.split(".")[0])
 
@@ -41,16 +43,16 @@ class Core:
         self.synchronizer_class = Synchronizer
 
         self.ui = termui.UI()
-        self.parser = None
-        self.subparsers = None
+        self.parser: Optional[PdmParser] = None
+        self.subparsers: Optional[argparse._SubParsersAction] = None
 
-    def init_parser(self):
+    def init_parser(self) -> None:
         self.parser = PdmParser(
             prog="pdm",
             description="PDM - Python Development Master",
             formatter_class=PdmFormatter,
         )
-        self.parser.is_root = True
+        self.parser.is_root = True  # type: ignore
         self.parser.add_argument(
             "-V",
             "--version",
@@ -68,12 +70,12 @@ class Core:
         for _, name, _ in pkgutil.iter_modules(COMMANDS_MODULE_PATH):
             module = importlib.import_module(f"pdm.cli.commands.{name}", __name__)
             try:
-                klass = module.Command
+                klass = module.Command  # type: ignore
             except AttributeError:
                 continue
             self.register_command(klass, klass.name or name)
 
-    def __call__(self, *args, **kwargs):
+    def __call__(self, *args: Any, **kwargs: Any) -> None:
         return self.main(*args, **kwargs)
 
     def ensure_project(
@@ -82,30 +84,40 @@ class Core:
         if obj is not None:
             options.project = obj
         if getattr(options, "project", None) is None:
-            project = None
             global_project = getattr(options, "global_project", None)
-            if global_project is True:
-                project_factory = self.project_class.create_global
-            elif global_project:
-                project = global_project
-            else:
-                project_factory = self.project_class
 
-            if project is None:
-                project = project_factory(getattr(options, "project_path", None))
+            default_root = (
+                None
+                if global_project or getattr(options, "search_parent", True)
+                else "."
+            )
+            project = self.create_project(
+                getattr(options, "project_path", None) or default_root,  # type: ignore
+                is_global=global_project,
+            )
             options.project = project
 
-        # Add reverse reference for core object
-        options.project.core = self
         migrate_pyproject(options.project)
 
-    def main(self, args=None, prog_name=None, obj=None, **extra):
+    def create_project(
+        self, root_path: Optional[os.PathLike] = None, is_global: bool = False
+    ) -> Project:
+        return self.project_class(self, root_path, is_global)
+
+    def main(
+        self,
+        args: List[str] = None,
+        prog_name: str = None,
+        obj: Optional[Project] = None,
+        **extra: Any,
+    ) -> None:
         """The main entry function"""
         from pdm.models.pip_shims import global_tempdir_manager
 
         self.init_parser()
         self.load_plugins()
-
+        assert self.parser
+        assert self.subparsers
         options = self.parser.parse_args(args or None)
         self.ui.set_verbosity(options.verbose)
         if options.ignore_python:
@@ -130,9 +142,10 @@ class Core:
                 etype, err, traceback = sys.exc_info()
                 should_show_tb = not isinstance(err, PdmUsageError)
                 if self.ui.verbosity > termui.NORMAL and should_show_tb:
-                    raise err.with_traceback(traceback)
+                    raise cast(Exception, err).with_traceback(traceback)
                 self.ui.echo(
-                    f"{termui.red('[' + etype.__name__ + ']')}: {err}", err=True
+                    f"{termui.red('[' + etype.__name__ + ']')}: {err}",  # type: ignore
+                    err=True,
                 )
                 if should_show_tb:
                     self.ui.echo("Add '-v' to see the detailed traceback", fg="yellow")
@@ -144,7 +157,7 @@ class Core:
         """Register a subcommand to the subparsers,
         with an optional name of the subcommand.
         """
-        command.project_class = self.project_class
+        assert self.subparsers
         command.register_to(self.subparsers, name)
 
     @staticmethod
@@ -152,7 +165,7 @@ class Core:
         """Add a config item to the configuration class"""
         Config.add_config(name, config_item)
 
-    def load_plugins(self):
+    def load_plugins(self) -> None:
         """Import and load plugins under `pdm.plugin` namespace
         A plugin is a callable that accepts the core object as the only argument.
 
@@ -162,10 +175,10 @@ class Core:
             ...
 
         """
-        for plugin in pkg_resources.iter_entry_points("pdm"):
+        for plugin in pkg_resources.iter_entry_points("pdm"):  # type: ignore
             plugin.load()(self)
 
 
-def main(args=None):
+def main(args: Optional[List[str]] = None) -> None:
     """The CLI entry function"""
     return Core().main(args)
