@@ -1,10 +1,16 @@
+from __future__ import annotations
+
 import functools
 import operator
 import os
 import re
+from argparse import Namespace
+from pathlib import Path
+from typing import TYPE_CHECKING, Any, Mapping
 
 import toml
 
+from pdm._types import RequirementDict, Source
 from pdm.formats.base import (
     MetaConverter,
     Unset,
@@ -16,10 +22,14 @@ from pdm.formats.base import (
 from pdm.models.markers import Marker
 from pdm.models.requirements import Requirement
 from pdm.models.specifiers import PySpecSet
+
+if TYPE_CHECKING:
+    from pdm.project.core import Project
+
 from pdm.utils import cd
 
 
-def check_fingerprint(project, filename):
+def check_fingerprint(project: Project | None, filename: Path | str) -> bool:
     with open(filename, encoding="utf-8") as fp:
         try:
             data = toml.load(fp)
@@ -32,7 +42,7 @@ def check_fingerprint(project, filename):
 VERSION_RE = re.compile(r"([^\d\s]*)\s*(\d.*?)\s*(?=,|$)")
 
 
-def _convert_specifier(version):
+def _convert_specifier(version: str) -> str:
     parts = []
     for op, version in VERSION_RE.findall(str(version)):
         if op == "~":
@@ -48,25 +58,27 @@ def _convert_specifier(version):
     return ",".join(parts)
 
 
-def _convert_python(python):
+def _convert_python(python: str) -> PySpecSet:
     if not python:
         return PySpecSet()
     parts = [PySpecSet(_convert_specifier(s)) for s in python.split("||")]
     return functools.reduce(operator.or_, parts)
 
 
-def _convert_req(name, req_dict):
+def _convert_req(name: str, req_dict: RequirementDict) -> str:
     if not getattr(req_dict, "items", None):
+        assert isinstance(req_dict, str)
         return Requirement.from_req_dict(name, _convert_specifier(req_dict)).as_line()
+    assert isinstance(req_dict, dict)
     req_dict = dict(req_dict)
     if "version" in req_dict:
-        req_dict["version"] = _convert_specifier(req_dict["version"])
-    markers = []
+        req_dict["version"] = _convert_specifier(str(req_dict["version"]))
+    markers: list[Marker] = []
     if "markers" in req_dict:
-        markers.append(Marker(req_dict.pop("markers")))
+        markers.append(Marker(req_dict.pop("markers")))  # type: ignore
     if "python" in req_dict:
         markers.append(
-            Marker(_convert_python(req_dict.pop("python")).as_marker_string())
+            Marker(_convert_python(str(req_dict.pop("python"))).as_marker_string())
         )
     if markers:
         req_dict["marker"] = str(functools.reduce(operator.and_, markers)).replace(
@@ -74,33 +86,33 @@ def _convert_req(name, req_dict):
         )
     if "rev" in req_dict or "branch" in req_dict or "tag" in req_dict:
         req_dict["ref"] = req_dict.pop(
-            "rev", req_dict.pop("tag", req_dict.pop("branch", None))
+            "rev", req_dict.pop("tag", req_dict.pop("branch", None))  # type: ignore
         )
     return Requirement.from_req_dict(name, req_dict).as_line()
 
 
 class PoetryMetaConverter(MetaConverter):
     @convert_from("authors")
-    def authors(self, value):
+    def authors(self, value: list[str]) -> list[str]:
         return parse_name_email(value)
 
     @convert_from("maintainers")
-    def maintainers(self, value):
+    def maintainers(self, value: list[str]) -> list[str]:
         return parse_name_email(value)
 
     @convert_from("license")
-    def license(self, value):
+    def license(self, value: str) -> dict[str, str]:
         self._data["dynamic"] = ["classifiers"]
         return make_inline_table({"text": value})
 
     @convert_from(name="requires-python")
-    def requires_python(self, source):
+    def requires_python(self, source: dict[str, Any]) -> str:
         python = source.get("dependencies", {}).pop("python", None)
         self._data["dynamic"] = ["classifiers"]
         return str(_convert_python(python))
 
     @convert_from()
-    def urls(self, source):
+    def urls(self, source: dict[str, Any]) -> dict[str, str]:
         rv = source.pop("urls", {})
         if "homepage" in source:
             rv["homepage"] = source.pop("homepage")
@@ -111,11 +123,13 @@ class PoetryMetaConverter(MetaConverter):
         return rv
 
     @convert_from("plugins", name="entry-points")
-    def entry_points(self, value):
+    def entry_points(
+        self, value: dict[str, dict[str, str]]
+    ) -> dict[str, dict[str, str]]:
         return value
 
     @convert_from()
-    def dependencies(self, source):
+    def dependencies(self, source: dict[str, Any]) -> list[str]:
         rv = []
         value, extras = dict(source["dependencies"]), source.pop("extras", {})
         for key, req_dict in value.items():
@@ -134,27 +148,40 @@ class PoetryMetaConverter(MetaConverter):
         del source["dependencies"]
         return make_array(rv, True)
 
-    @convert_from("dev-dependencies", name="dev-dependencies")
-    def dev_dependencies(self, value):
-        return make_array([_convert_req(key, req) for key, req in value.items()], True)
+    @convert_from("dev-dependencies")
+    def dev_dependencies(self, value: dict) -> None:
+        self.settings["dev-dependencies"] = {
+            "dev": make_array(
+                [_convert_req(key, req) for key, req in value.items()], True
+            ),
+        }
+        raise Unset()
 
     @convert_from()
-    def includes(self, source):
-        result = []
+    def includes(self, source: dict[str, list[str] | str]) -> list[str]:
+        result: list[str] = []
         for item in source.pop("packages", []):
+            assert isinstance(item, dict)
             include = item["include"]
             if item.get("from"):
                 include = f"{item.get('from')}/{include}"
             result.append(include)
         result.extend(source.pop("include", []))
-        return result
+        self.settings["includes"] = result
+        raise Unset()
 
     @convert_from("exclude")
-    def excludes(self, value):
-        return value
+    def excludes(self, value: list[str]) -> None:
+        self.settings["excludes"] = value
+        raise Unset()
+
+    @convert_from("build")
+    def build(self, value: str) -> None:
+        self.settings["build"] = value
+        raise Unset()
 
     @convert_from("source")
-    def source(self, value):
+    def sources(self, value: list[Source]) -> None:
         self.settings["source"] = [
             {
                 "name": item.get("name", ""),
@@ -166,13 +193,19 @@ class PoetryMetaConverter(MetaConverter):
         raise Unset()
 
 
-def convert(project, filename, options):
+def convert(
+    project: Project | None,
+    filename: str | Path,
+    options: Namespace | None,
+) -> tuple[Mapping[str, Any], Mapping[str, Any]]:
     with open(filename, encoding="utf-8") as fp, cd(
         os.path.dirname(os.path.abspath(filename))
     ):
-        converter = PoetryMetaConverter(toml.load(fp)["tool"]["poetry"])
-        return dict(converter), converter.settings
+        converter = PoetryMetaConverter(
+            toml.load(fp)["tool"]["poetry"], project.core.ui if project else None
+        )
+        return converter.convert()
 
 
-def export(project, candidates, options):
+def export(project: Project, candidates: list, options: Any) -> None:
     raise NotImplementedError()
