@@ -1,17 +1,25 @@
+from __future__ import annotations
+
+import dataclasses
 import hashlib
 import urllib.parse
+from argparse import Namespace
+from os import PathLike
+from typing import Any, Mapping, cast
 
 from distlib.wheel import Wheel
+from pip._vendor.packaging.requirements import Requirement as PRequirement
 
-from pdm.exceptions import PdmUsageError
 from pdm.formats.base import make_array
-from pdm.models.markers import Marker
-from pdm.models.pip_shims import parse_requirements
-from pdm.models.requirements import parse_requirement
+from pdm.models.candidates import Candidate
+from pdm.models.environment import Environment
+from pdm.models.pip_shims import InstallRequirement, PackageFinder, parse_requirements
+from pdm.models.requirements import Requirement, parse_requirement
+from pdm.project import Project
 from pdm.utils import get_finder
 
 
-def _requirement_to_str_lowercase_name(requirement):
+def _requirement_to_str_lowercase_name(requirement: PRequirement) -> str:
     """Formats a packaging.requirements.Requirement with a lowercase name."""
     parts = [requirement.name.lower()]
 
@@ -30,7 +38,7 @@ def _requirement_to_str_lowercase_name(requirement):
     return "".join(parts)
 
 
-def ireq_as_line(ireq, environment):
+def ireq_as_line(ireq: InstallRequirement, environment: Environment) -> str:
     """Formats an `InstallRequirement` instance as a
     PEP 508 dependency string.
 
@@ -45,40 +53,37 @@ def ireq_as_line(ireq, environment):
         line = "-e {}".format(ireq.link)
     else:
         if not ireq.req:
-            ireq.req = parse_requirement("dummy @" + ireq.link.url)
+            ireq.req = parse_requirement("dummy @" + ireq.link.url)  # type: ignore
             wheel = Wheel(environment.build(ireq))
-            ireq.req.name = wheel.name
+            ireq.req.name = wheel.name  # type: ignore
 
-        line = _requirement_to_str_lowercase_name(ireq.req)
-
-    if str(ireq.req.marker) != str(ireq.markers):
-        if not ireq.req.marker:
-            line = "{}; {}".format(line, ireq.markers)
-        else:
-            name, markers = line.split(";", 1)
-            markers = Marker(markers) & ireq.markers
-            line = "{}; {}".format(name, markers)
+        line = _requirement_to_str_lowercase_name(cast(PRequirement, ireq.req))
+    assert ireq.req
+    if not ireq.req.marker and ireq.markers:
+        line = f"{line}; {ireq.markers}"
 
     return line
 
 
-def parse_requirement_file(filename):
+def parse_requirement_file(
+    filename: str,
+) -> tuple[list[InstallRequirement], PackageFinder]:
     from pdm.models.pip_shims import install_req_from_parsed_requirement
 
     finder = get_finder([])
     ireqs = [
         install_req_from_parsed_requirement(pr)
-        for pr in parse_requirements(filename, finder.session, finder)
+        for pr in parse_requirements(filename, finder.session, finder)  # type: ignore
     ]
     return ireqs, finder
 
 
-def check_fingerprint(project, filename):
-    import tomlkit
+def check_fingerprint(project: Project, filename: PathLike) -> bool:
+    import toml
 
     with open(filename, encoding="utf-8") as fp:
         try:
-            tomlkit.parse(fp.read())
+            toml.load(fp)
         except ValueError:
             # the file should be a requirements.txt if it not a TOML document.
             return True
@@ -86,28 +91,28 @@ def check_fingerprint(project, filename):
             return False
 
 
-def convert_url_to_source(url, name=None):
+def convert_url_to_source(url: str, name: str | None = None) -> dict[str, Any]:
     if not name:
         name = hashlib.sha1(url.encode("utf-8")).hexdigest()[:6]
     return {"name": name, "url": url, "verify_ssl": url.startswith("https://")}
 
 
-def convert(project, filename, options):
+def convert(
+    project: Project, filename: PathLike, options: Namespace
+) -> tuple[Mapping[str, Any], Mapping[str, Any]]:
     ireqs, finder = parse_requirement_file(str(filename))
     with project.core.ui.logging("build"):
         reqs = [ireq_as_line(ireq, project.environment) for ireq in ireqs]
 
     deps = make_array(reqs, True)
-    data = {"dependencies": [], "dev-dependencies": []}
-    if options.dev and options.section:
-        raise PdmUsageError("Can't specify --dev and --section at the same time")
-    elif options.dev:
-        data["dev-dependencies"] = deps
+    data: dict[str, Any] = {}
+    settings: dict[str, Any] = {}
+    if options.dev:
+        settings["dev-dependencies"] = {options.section or "dev": deps}
     elif options.section:
         data["optional-dependencies"] = {options.section: deps}
     else:
         data["dependencies"] = deps
-    settings = {}
     if finder.index_urls:
         sources = [convert_url_to_source(finder.index_urls[0], "pypi")]
         sources.extend(convert_url_to_source(url) for url in finder.index_urls[1:])
@@ -116,13 +121,21 @@ def convert(project, filename, options):
     return data, settings
 
 
-def export(project, candidates, options):
+def export(
+    project: Project,
+    candidates: list[Candidate] | list[Requirement],
+    options: Namespace,
+) -> str:
     lines = []
-    for candidate in candidates:
-        req = getattr(candidate, "req", candidate).as_line()
-        lines.append(req)
-        if options.hashes and candidate.hashes:
-            for item in candidate.hashes.values():
+    for candidate in sorted(candidates, key=lambda x: x.identify()):
+        if isinstance(candidate, Candidate):
+            req = dataclasses.replace(candidate.req, specifier=f"=={candidate.version}")
+        else:
+            assert isinstance(candidate, Requirement)
+            req = candidate
+        lines.append(req.as_line())
+        if options.hashes and getattr(candidate, "hashes", None):
+            for item in candidate.hashes.values():  # type: ignore
                 lines.append(f" \\\n    --hash={item}")
         lines.append("\n")
     sources = project.tool_settings.get("source", [])
