@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-import importlib
+import pathlib
 from typing import TYPE_CHECKING
 
 import distlib.scripts
@@ -8,7 +8,6 @@ from pip._vendor.pkg_resources import EggInfoDistribution
 
 from pdm import termui
 from pdm.models import pip_shims
-from pdm.models.builders import EnvBuilder
 from pdm.models.requirements import parse_requirement
 
 if TYPE_CHECKING:
@@ -50,35 +49,36 @@ class Installer:  # pragma: no cover
     def install_wheel(self, wheel: Wheel) -> None:
         paths = self.environment.get_paths()
         maker = distlib.scripts.ScriptMaker(None, None)
-        maker.executable = self.environment.python_executable
+        maker.variants = set(("",))
+        enquoted_executable = distlib.scripts.enquote_executable(
+            self.environment.interpreter.executable
+        )
+        maker.executable = enquoted_executable
         wheel.install(paths, maker)
 
     def install_editable(self, ireq: pip_shims.InstallRequirement) -> None:
+        from pdm.builders.base import EnvBuilder
+
         setup_path = ireq.setup_py_path
         paths = self.environment.get_paths()
-        install_script = importlib.import_module(
-            "pdm.installers._editable_install"
-        ).__file__.rstrip("co")
+        install_script = pathlib.Path(__file__).with_name("_editable_install.py")
         install_args = [
-            self.environment.python_executable,
+            self.environment.interpreter.executable,
             "-u",
-            install_script,
+            str(install_script),
             setup_path,
             paths["prefix"],
             paths["purelib"],
             paths["scripts"],
         ]
-        with EnvBuilder(ireq.unpacked_source_directory, self.environment) as builder:
-            builder.install(["setuptools"])
-            extra_env = {"INJECT_SITE": "1"} if not self.environment.is_global else None
-            builder.subprocess_runner(
-                install_args, ireq.unpacked_source_directory, extra_env
-            )
+        builder = EnvBuilder(ireq.unpacked_source_directory, self.environment)
+        builder.install(["setuptools"])
+        builder.subprocess_runner(install_args, ireq.unpacked_source_directory)
 
     def uninstall(self, dist: Distribution) -> None:
         req = parse_requirement(dist.project_name)
         ireq = pip_shims.install_req_from_line(dist.project_name)
-        ireq.req = req
+        ireq.req = req  # type: ignore
 
         pathset = ireq.uninstall(auto_confirm=self.auto_confirm)
         if pathset:
