@@ -1,11 +1,13 @@
-import contextlib
+from __future__ import annotations
+
 import functools
 import multiprocessing
 import traceback
+from concurrent.futures._base import Future
 from concurrent.futures.thread import ThreadPoolExecutor
-from typing import Dict, List, Optional, Tuple
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union, cast
 
-from pip._vendor.pkg_resources import Distribution, safe_name
+from pip._vendor.pkg_resources import Distribution
 
 from pdm import termui
 from pdm.exceptions import InstallationError
@@ -18,23 +20,23 @@ from pdm.models.requirements import strip_extras
 class DummyFuture:
     _NOT_SET = object()
 
-    def __init__(self):
+    def __init__(self) -> None:
         self._result = self._NOT_SET
-        self._exc = None
+        self._exc: Optional[Exception] = None
 
-    def set_result(self, result):
+    def set_result(self, result: Any) -> None:
         self._result = result
 
-    def set_exception(self, exc):
+    def set_exception(self, exc: Exception) -> None:
         self._exc = exc
 
-    def result(self):
+    def result(self) -> Any:
         return self._result
 
-    def exception(self):
+    def exception(self) -> Optional[Exception]:
         return self._exc
 
-    def add_done_callback(self, func):
+    def add_done_callback(self, func: Callable) -> None:
         func(self)
 
 
@@ -43,7 +45,7 @@ class DummyExecutor:
     functions are called and awaited for the result
     """
 
-    def submit(self, func, *args, **kwargs):
+    def submit(self, func: Callable, *args: str, **kwargs: Any) -> DummyFuture:
         future = DummyFuture()
         try:
             future.set_result(func(*args, **kwargs))
@@ -51,10 +53,10 @@ class DummyExecutor:
             future.set_exception(exc)
         return future
 
-    def __enter__(self):
+    def __enter__(self) -> DummyExecutor:
         return self
 
-    def __exit__(self, *args, **kwargs):
+    def __exit__(self, *args: Any, **kwargs: Any) -> None:
         return
 
 
@@ -67,29 +69,37 @@ class Synchronizer:
         self,
         candidates: Dict[str, Candidate],
         environment: Environment,
+        clean: bool = False,
+        dry_run: bool = False,
         retry_times: int = 1,
+        install_self: bool = False,
+        no_editable: bool = False,
     ) -> None:
-        self.candidates = candidates
         self.environment = environment
-        self.parallel = environment.project.config["parallel_install"]
-        self.all_candidates = environment.project.get_locked_candidates("__all__")
-        self.working_set = environment.get_working_set()
+        self.clean = clean
+        self.dry_run = dry_run
         self.retry_times = retry_times
+        self.no_editable = no_editable
+        self.install_self = install_self
+
+        self.parallel = environment.project.config["parallel_install"]
+        locked_repository = environment.project.locked_repository
+        self.all_candidate_keys = list(locked_repository.all_candidates)
+        self.working_set = environment.get_working_set()
         self.ui = environment.project.core.ui
 
-    @contextlib.contextmanager
-    def create_executor(self):
+        if self.no_editable:
+            for candidate in candidates.values():
+                candidate.req.editable = None  # type: ignore
+        self.candidates = candidates
+
+    def create_executor(
+        self,
+    ) -> Union[ThreadPoolExecutor, DummyExecutor]:
         if self.parallel:
-            executor = ThreadPoolExecutor(
-                max_workers=min(multiprocessing.cpu_count(), 8)
-            )
+            return ThreadPoolExecutor(max_workers=min(multiprocessing.cpu_count(), 8))
         else:
-            executor = DummyExecutor()
-        with executor:
-            try:
-                yield executor
-            except KeyboardInterrupt:
-                pass
+            return DummyExecutor()
 
     def get_installer(self) -> Installer:
         return Installer(self.environment)
@@ -104,23 +114,24 @@ class Synchronizer:
     def compare_with_working_set(self) -> Tuple[List[str], List[str], List[str]]:
         """Compares the candidates and return (to_add, to_update, to_remove)"""
         working_set = self.working_set
-        to_update, to_remove = [], []
         candidates = self.candidates.copy()
-        environment = self.environment.marker_environment
+        to_update, to_remove = [], []
+
         for key, dist in working_set.items():
             if key == self.self_key:
                 continue
             if key in candidates:
                 can = candidates.pop(key)
-                if can.marker and not can.marker.evaluate(environment):
-                    to_remove.append(key)
-                elif (
+                if (
                     can.req.editable
                     or is_dist_editable(dist)
                     or (dist.version != can.version)
                 ):
                     to_update.append(key)
-            elif key not in self.all_candidates and key not in self.SEQUENTIAL_PACKAGES:
+            elif (
+                key not in self.all_candidate_keys
+                and key not in self.SEQUENTIAL_PACKAGES
+            ):
                 # Remove package only if it is not required by any section
                 # Packages for packaging will never be removed
                 to_remove.append(key)
@@ -128,12 +139,14 @@ class Synchronizer:
             {
                 strip_extras(name)[0]
                 for name, can in candidates.items()
-                if name != self.self_key
-                and strip_extras(name)[0] not in working_set
-                and not (can.marker and not can.marker.evaluate(environment))
+                if name != self.self_key and strip_extras(name)[0] not in working_set
             }
         )
-        return sorted(to_add), sorted(to_update), sorted(to_remove)
+        return (
+            sorted(to_add),
+            sorted(to_update),
+            sorted(to_remove) if self.clean else [],
+        )
 
     def install_candidate(self, key: str) -> Candidate:
         """Install candidate"""
@@ -153,7 +166,7 @@ class Synchronizer:
     def update_candidate(self, key: str) -> Tuple[Distribution, Candidate]:
         """Update candidate"""
         can = self.candidates[key]
-        dist = self.working_set[safe_name(can.name).lower()]
+        dist = self.working_set[strip_extras(key)[0]]
         installer = self.get_installer()
         with self.ui.open_spinner(
             f"Updating {termui.green(key, bold=True)} {termui.yellow(dist.version)} "
@@ -178,10 +191,7 @@ class Synchronizer:
         return dist, can
 
     def remove_distribution(self, key: str) -> Distribution:
-        """Remove distributions with given names.
-
-        :param distributions: a list of names to be removed.
-        """
+        """Remove distributions with given names."""
         installer = self.get_installer()
         dist = self.working_set[key]
         with self.ui.open_spinner(
@@ -229,38 +239,32 @@ class Synchronizer:
             for can in to_add:
                 lines.append(f"  - {can.format()}")
         if to_update:
-            lines.append(termui.bold("Packages to add:"))
+            lines.append(termui.bold("Packages to update:"))
             for prev, cur in to_update:
                 lines.append(
                     f"  - {termui.green(cur.name, bold=True)} "
                     f"{termui.yellow(prev.version)} -> {termui.yellow(cur.version)}"
                 )
         if to_remove:
-            lines.append(termui.bold("Packages to add:"))
+            lines.append(termui.bold("Packages to remove:"))
             for dist in to_remove:
                 lines.append(
                     f"  - {termui.green(dist.key, bold=True)} "
                     f"{termui.yellow(dist.version)}"
                 )
-        self.ui.echo("\n".join(lines))
+        if lines:
+            self.ui.echo("\n".join(lines))
 
-    def synchronize(self, clean: bool = True, dry_run: bool = False) -> None:
-        """Synchronize the working set with pinned candidates.
-
-        :param clean: Whether to remove unneeded packages, defaults to True.
-        :param dry_run: If set to True, only prints actions without actually do them.
-        """
+    def synchronize(self) -> None:
+        """Synchronize the working set with pinned candidates."""
         to_add, to_update, to_remove = self.compare_with_working_set()
-        if not clean:
-            to_remove = []
-
         to_do = {"remove": to_remove, "update": to_update, "add": to_add}
-        self._show_headline(to_do)
 
-        if dry_run:
+        if self.dry_run:
             self._show_summary(to_do)
             return
 
+        self._show_headline(to_do)
         handlers = {
             "add": self.install_candidate,
             "update": self.update_candidate,
@@ -268,11 +272,6 @@ class Synchronizer:
         }
         sequential_jobs = []
         parallel_jobs = []
-        # Self package will be installed after all other dependencies are installed.
-        self_action = None
-        self_key = self.self_key
-        if self_key in self.candidates:
-            self_action = "update" if self_key in self.working_set else "add"
 
         for kind in to_do:
             for key in to_do[kind]:
@@ -287,10 +286,12 @@ class Synchronizer:
         errors: List[str] = []
         failed_jobs: List[Tuple[str, str]] = []
 
-        def update_progress(future, kind, key):
-            if future.exception():
+        def update_progress(
+            future: Union[Future, DummyFuture], kind: str, key: str
+        ) -> None:
+            error = future.exception()
+            if error:
                 failed_jobs.append((kind, key))
-                error = future.exception()
                 errors.extend(
                     [f"{kind} {termui.green(key)} failed:\n"]
                     + traceback.format_exception(
@@ -322,9 +323,17 @@ class Synchronizer:
                 self.ui.echo("".join(errors), err=True)
                 raise InstallationError("Some package operations are not complete yet")
 
-            if self_action:
+            if self.install_self:
+                self_candidate = self.environment.project.make_self_candidate(
+                    not self.no_editable
+                )
+                self_key = cast(str, self_candidate.req.key)
+                self.candidates[self_key] = self_candidate
                 self.ui.echo("Installing the project as an editable package...")
                 with self.ui.indent("  "):
-                    handlers[self_action](self_key)
+                    if self_key in self.working_set:
+                        self.update_candidate(self_key)
+                    else:
+                        self.install_candidate(self_key)
 
             self.ui.echo(f"\n{termui.Emoji.SUCC} All complete!")
