@@ -4,6 +4,7 @@ import functools
 import os
 import warnings
 from argparse import Namespace
+from pathlib import Path
 from typing import TYPE_CHECKING, Any, Dict, List, Optional, Sequence
 
 from distlib.database import EggInfoDistribution
@@ -14,19 +15,26 @@ from pdm import termui
 from pdm.exceptions import BuildError, ExtrasError, RequirementError
 from pdm.models import pip_shims
 from pdm.models.markers import Marker
-from pdm.models.readers import SetupReader
 from pdm.models.requirements import Requirement, filter_requirements_with_extras
-from pdm.utils import cached_property, get_rev_from_url, path_replace
+from pdm.models.setup import Setup
+from pdm.models.specifiers import PySpecSet
+from pdm.utils import (
+    cached_property,
+    expand_env_vars_in_auth,
+    get_rev_from_url,
+    path_replace,
+)
 
 if TYPE_CHECKING:
     from distlib.metadata import Metadata
+    from packaging.version import Version
 
     from pdm.models.environment import Environment
 
 vcs = pip_shims.VcsSupport()
 
 
-def get_sdist(egg_info) -> Optional[EggInfoDistribution]:
+def get_sdist(egg_info: str) -> Optional[EggInfoDistribution]:
     """Get a distribution from egg_info directory."""
     return EggInfoDistribution(egg_info) if egg_info else None
 
@@ -39,7 +47,7 @@ def _patch_version_parsing():
     from packaging.requirements import InvalidRequirement
     from packaging.requirements import Requirement as PRequirement
 
-    def is_valid_matcher(self, s):
+    def is_valid_matcher(self: Any, s: str) -> bool:
         try:
             PRequirement(s)
         except InvalidRequirement:
@@ -93,7 +101,7 @@ class Candidate:
         req: Requirement,
         environment: Environment,
         name: Optional[str] = None,
-        version: Optional[str] = None,
+        version: Optional[Version] = None,
         link: Optional[pip_shims.Link] = None,
     ):
         """
@@ -111,11 +119,9 @@ class Candidate:
             link = self.ireq.link
         self.link = link
         self.hashes: Optional[Dict[str, str]] = None
-        self.marker = None
-        self.sections = []
-        self._requires_python = None
+        self._requires_python: Optional[PySpecSet] = None
 
-        self.wheel = None
+        self.wheel: Optional[Wheel] = None
         self.metadata = None
 
     def __hash__(self):
@@ -123,7 +129,21 @@ class Candidate:
 
     @cached_property
     def ireq(self) -> pip_shims.InstallRequirement:
-        return self.req.as_ireq()
+        rv = self.req.as_ireq()
+        if rv.link:
+            rv.link = pip_shims.Link(
+                expand_env_vars_in_auth(
+                    rv.link.url.replace(
+                        "${PROJECT_ROOT}",
+                        self.environment.project.root.as_posix().lstrip("/"),
+                    )
+                )
+            )
+            if rv.source_dir:
+                rv.source_dir = os.path.normpath(os.path.abspath(rv.link.file_path))
+            if rv.local_file_path:
+                rv.local_file_path = rv.link.file_path
+        return rv
 
     def identify(self) -> str:
         return self.req.identify()
@@ -166,7 +186,9 @@ class Candidate:
             if raising:
                 raise
             termui.logger.warn("Failed to build package, try parsing project files.")
-            meta_dict = SetupReader.read_from_directory(ireq.unpacked_source_directory)
+            meta_dict = Setup.from_directory(
+                Path(ireq.unpacked_source_directory)
+            ).as_dict()
             meta_dict.update(summary="UNKNOWN")
             meta_dict["requires_python"] = meta_dict.pop("python_requires", None)
             self.metadata = Namespace(**meta_dict)
@@ -192,7 +214,8 @@ class Candidate:
         return self.metadata
 
     def __repr__(self) -> str:
-        return f"<Candidate {self.name} {self.version}>"
+        source = getattr(self.link, "comes_from", "unknown")
+        return f"<Candidate {self.name} {self.version} from {source}>"
 
     @classmethod
     def from_installation_candidate(
@@ -202,14 +225,13 @@ class Candidate:
         environment: Environment,
     ) -> Candidate:
         """Build a candidate from pip's InstallationCandidate."""
-        inst = cls(
+        return cls(
             req,
             environment,
             name=candidate.name,
             version=candidate.version,
             link=candidate.link,
         )
-        return inst
 
     def get_dependencies_from_metadata(self) -> List[str]:
         """Get the dependencies of a candidate from metadata."""
@@ -263,10 +285,9 @@ class Candidate:
         """Build a lockfile entry dictionary for the candidate."""
         result = {
             "name": self.name,
-            "sections": sorted(self.sections),
             "version": str(self.version),
             "extras": sorted(self.req.extras or ()),
-            "marker": str(self.marker).replace('"', "'") if self.marker else None,
+            "requires_python": str(self.requires_python),
             "editable": self.req.editable,
         }
         project_root = self.environment.project.root.as_posix()
