@@ -4,26 +4,28 @@ import collections
 import os
 import re
 import shutil
+import subprocess
 import sys
 import sysconfig
+import tempfile
 from contextlib import contextmanager
+from os import PathLike
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Dict, Generator, Iterator, List, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Dict, Generator, Iterator, List, Optional
 
 from distlib.scripts import ScriptMaker
-from pip._internal.req import req_uninstall
-from pip._internal.utils import misc
 from pip._vendor import packaging, pkg_resources
 
 from pdm import termui
+from pdm.exceptions import BuildError
 from pdm.models import pip_shims
 from pdm.models.auth import make_basic_auth
-from pdm.models.builders import EnvBuilder
 from pdm.models.in_process import (
     get_pep508_environment,
-    get_python_version,
+    get_python_abi_tag,
     get_sys_config_paths,
 )
+from pdm.models.pip_shims import misc, patch_bin_prefix, req_uninstall
 from pdm.utils import (
     allow_all_wheels,
     cached_property,
@@ -31,13 +33,13 @@ from pdm.utils import (
     create_tracked_tempdir,
     expand_env_vars_in_auth,
     get_finder,
-    get_python_version_string,
     populate_link,
     temp_environ,
 )
 
 if TYPE_CHECKING:
     from pdm._types import Source
+    from pdm.models.python import PythonInfo
     from pdm.project import Project
 
 _egg_info_re = re.compile(r"([a-z0-9_.]+)-([a-z0-9_.!+-]+)", re.IGNORECASE)
@@ -49,7 +51,7 @@ class WorkingSet(collections.abc.Mapping):
     def __init__(
         self,
         paths: Optional[List[str]] = None,
-        python: Tuple[int, ...] = sys.version_info[:3],
+        python: str = pkg_resources.PY_MAJOR,
     ):
         self.env = pkg_resources.Environment(paths, python=python)
         self.pkg_ws = pkg_resources.WorkingSet(paths)
@@ -66,8 +68,7 @@ class WorkingSet(collections.abc.Mapping):
         return len(self.env._distmap)
 
     def __iter__(self) -> Iterator[str]:
-        for item in self.env:
-            yield item
+        return iter(self.env)
 
     def __add_editable_dists(self):
         """Editable distributions are not present in pkg_resources.WorkingSet,
@@ -89,7 +90,7 @@ class Environment:
         """
         self.python_requires = project.python_requires
         self.project = project
-        self.python_executable = project.python_executable
+        self.interpreter: PythonInfo = project.python
         self._essential_installed = False
         self.auth = make_basic_auth(
             self.project.sources, self.project.core.ui.verbosity >= termui.DETAIL
@@ -109,7 +110,7 @@ class Environment:
         return paths
 
     @contextmanager
-    def activate(self):
+    def activate(self) -> Iterator:
         """Activate the environment. Manipulate the ``PYTHONPATH`` and patches ``pip``
         to be aware of local packages. This method acts like a context manager.
 
@@ -128,45 +129,39 @@ class Environment:
             misc.is_local = req_uninstall.is_local = self.is_local
             _evaluate_marker = pkg_resources.evaluate_marker
             pkg_resources.evaluate_marker = self.evaluate_marker
-            bin_py = req_uninstall.bin_py
-            req_uninstall.bin_py = paths["scripts"]
             sys._original_executable = sys.executable
-            sys.executable = self.python_executable
-            yield
+            sys.executable = self.interpreter.executable
+            with patch_bin_prefix(paths["scripts"]):
+                yield
             sys.executable = sys._original_executable
             del sys._original_executable
             pkg_resources.evaluate_marker = _evaluate_marker
             misc.is_local = req_uninstall.is_local = _is_local
-            req_uninstall.bin_py = bin_py
             misc.site_packages = _old_sitepackages
             pkg_resources.working_set = _old_ws
 
-    def is_local(self, path) -> bool:
+    def is_local(self, path: PathLike) -> bool:
         """PEP 582 version of ``is_local()`` function."""
         return misc.normalize_path(path).startswith(
             misc.normalize_path(self.packages_path.as_posix())
         )
 
-    def evaluate_marker(self, text: str, extra=None) -> bool:
+    def evaluate_marker(self, text: str, extra: Any = None) -> bool:
         marker = packaging.markers.Marker(text)
         return marker.evaluate(self.marker_environment)
 
     @cached_property
     def packages_path(self) -> Path:
         """The local packages path."""
-        version, is_64bit = get_python_version(self.python_executable, True, 2)
-        pypackages = (
-            self.project.root
-            / "__pypackages__"
-            / get_python_version_string(version, is_64bit)
-        )
-        if not pypackages.exists() and not is_64bit:
-            compatible_packages = pypackages.parent / get_python_version_string(
-                version, True
-            )
+        pypackages = self.project.root / "__pypackages__" / self.interpreter.identifier
+        if not pypackages.exists() and "-32" in pypackages.name:
+            compatible_packages = pypackages.with_name(pypackages.name[:-3])
             if compatible_packages.exists():
                 pypackages = compatible_packages
         scripts = "Scripts" if os.name == "nt" else "bin"
+        if not pypackages.parent.exists():
+            pypackages.parent.mkdir(parents=True)
+            pypackages.parent.joinpath(".gitignore").write_text("*\n!.gitignore\n")
         for subdir in [scripts, "include", "lib"]:
             pypackages.joinpath(subdir).mkdir(exist_ok=True, parents=True)
         return pypackages
@@ -188,7 +183,7 @@ class Environment:
             src_dir = os.path.join(venv, "src")
             if os.path.exists(src_dir):
                 return src_dir
-        return create_tracked_tempdir("pdm-src")
+        return create_tracked_tempdir(prefix="pdm-src-")
 
     @contextmanager
     def get_finder(
@@ -206,11 +201,13 @@ class Environment:
         for source in sources:
             source["url"] = expand_env_vars_in_auth(source["url"])
 
-        python_version, _ = get_python_version(self.python_executable, digits=2)
+        python_version = self.interpreter.version_tuple
+        python_abi_tag = get_python_abi_tag(self.interpreter.executable)
         finder = get_finder(
             sources,
             self.project.cache_dir.as_posix(),
             python_version,
+            python_abi_tag,
             ignore_requires_python,
         )
         # Reuse the auth across sessions to avoid prompting repeatly.
@@ -232,22 +229,17 @@ class Environment:
         :param allow_all: Allow building incompatible wheels.
         :returns: The full path of the built artifact.
         """
+        from pdm.builders import EnvEggInfoBuilder, EnvWheelBuilder
+
         build_dir = self._get_build_dir(ireq)
         wheel_cache = self.project.make_wheel_cache()
-        supported_tags = pip_shims.get_supported(
-            "".join(map(str, get_python_version(self.python_executable, digits=2)[0]))
-        )
+        supported_tags = pip_shims.get_supported(self.interpreter.for_tag())
+        if hashes:
+            ireq.hash_options = convert_hashes(hashes)
         with self.get_finder(ignore_requires_python=True) as finder:
             with allow_all_wheels(allow_all):
                 # temporarily allow all wheels to get a link.
                 populate_link(finder, ireq, False)
-            ireq.link = pip_shims.Link(
-                expand_env_vars_in_auth(
-                    ireq.link.url.replace(
-                        "${PROJECT_ROOT}", self.project.root.as_posix().lstrip("/")
-                    )
-                )
-            )
             if hashes is None and not ireq.editable:
                 # If hashes are not given and cache is hit, replace the link with the
                 # cached one. This can speed up by skipping the download and build.
@@ -264,8 +256,6 @@ class Environment:
             else:
                 ireq.ensure_has_source_dir(build_dir)
 
-            if hashes:
-                ireq.hash_options = convert_hashes(hashes)
             if not (ireq.editable and ireq.req.is_local_dir):
                 downloader = pip_shims.Downloader(finder.session, "off")
                 downloaded = pip_shims.unpack_url(
@@ -291,54 +281,52 @@ class Environment:
                 return cache_entry.link.file_path
 
         # Otherwise, as all source is already prepared, build it.
-        with EnvBuilder(ireq.unpacked_source_directory, self) as builder:
-            if ireq.editable:
-                ret = builder.build_egg_info(build_dir)
-                ireq.metadata_directory = ret
-                return ret
-            should_cache = False
-            if ireq.link.is_vcs:
-                vcs = pip_shims.VcsSupport()
-                vcs_backend = vcs.get_backend_for_scheme(ireq.link.scheme)
-                if vcs_backend.is_immutable_rev_checkout(
-                    ireq.link.url, ireq.source_dir
-                ):
-                    should_cache = True
-            else:
-                base, _ = ireq.link.splitext()
-                if _egg_info_re.search(base) is not None:
-                    # Determine whether the string looks like an egg_info.
-                    should_cache = True
-            output_dir = (
-                wheel_cache.get_path_for_link(ireq.link) if should_cache else build_dir
-            )
-            if not os.path.exists(output_dir):
-                os.makedirs(output_dir, exist_ok=True)
-            return builder.build_wheel(output_dir)
+        if ireq.editable:
+            builder = EnvEggInfoBuilder(ireq.unpacked_source_directory, self)
+            ret = ireq.metadata_directory = builder.build(build_dir)
+            return ret
+        should_cache = False
+        if ireq.link.is_vcs:
+            vcs = pip_shims.VcsSupport()
+            vcs_backend = vcs.get_backend_for_scheme(ireq.link.scheme)
+            if vcs_backend.is_immutable_rev_checkout(ireq.link.url, ireq.source_dir):
+                should_cache = True
+        elif not ireq.link.is_existing_dir:
+            base, _ = ireq.link.splitext()
+            if _egg_info_re.search(base) is not None:
+                # Determine whether the string looks like an egg_info.
+                should_cache = True
+        output_dir = (
+            wheel_cache.get_path_for_link(ireq.link) if should_cache else build_dir
+        )
+        if not os.path.exists(output_dir):
+            os.makedirs(output_dir, exist_ok=True)
+        return EnvWheelBuilder(ireq.unpacked_source_directory, self).build(output_dir)
 
     def get_working_set(self) -> WorkingSet:
         """Get the working set based on local packages directory."""
         paths = self.get_paths()
         return WorkingSet(
-            [paths["platlib"]], python=get_python_version(self.python_executable)[0]
+            [paths["platlib"], paths["purelib"]],
+            python=f"{self.interpreter.major}.{self.interpreter.minor}",
         )
 
     @cached_property
     def marker_environment(self) -> Dict[str, Any]:
         """Get environment for marker evaluation"""
-        return get_pep508_environment(self.python_executable)
+        return get_pep508_environment(self.interpreter.executable)
 
     def which(self, command: str) -> str:
         """Get the full path of the given executable against this environment."""
         if not os.path.isabs(command) and command.startswith("python"):
             python = os.path.splitext(command)[0]
             version = python[6:]
-            this_version, _ = get_python_version(self.python_executable, True)
-            if not version or this_version.startswith(version):
-                return self.python_executable
+            this_version = self.interpreter.version
+            if not version or str(this_version).startswith(version):
+                return self.interpreter.executable
         # Fallback to use shutil.which to find the executable
         this_path = self.get_paths()["scripts"]
-        python_root = os.path.dirname(self.python_executable)
+        python_root = os.path.dirname(self.interpreter.executable)
         new_path = os.pathsep.join([python_root, this_path, os.getenv("PATH", "")])
         return shutil.which(command, path=new_path)
 
@@ -355,6 +343,52 @@ class Environment:
                 re.sub(rb"#!.+?python.*?$", shebang, child.read_bytes(), flags=re.M)
             )
 
+    def _download_pip_wheel(self, path: os.PathLike):
+        dirname = Path(tempfile.mkdtemp(prefix="pip-download-"))
+        try:
+            subprocess.check_call(
+                [
+                    getattr(sys, "_original_executable", sys.executable),
+                    "-m",
+                    "pip",
+                    "download",
+                    "--only-binary=:all:",
+                    "-d",
+                    str(dirname),
+                    "pip<21",  # pip>=21 drops the support of py27
+                ],
+            )
+            wheel_file = next(dirname.glob("pip-*.whl"))
+            shutil.move(str(wheel_file), path)
+        except subprocess.CalledProcessError:
+            raise BuildError("Failed to download pip for the given interpreter")
+        finally:
+            shutil.rmtree(dirname, ignore_errors=True)
+
+    @cached_property
+    def pip_command(self) -> List[str]:
+        """Get a pip command for this environment, and download one if not available.
+        Return a list of args like ['python', '-m', 'pip']
+        """
+        from pip import __file__ as pip_location
+
+        python_major = self.interpreter.major
+        executable = self.interpreter.executable
+        proc = subprocess.run(
+            [executable, "-Esm", "pip", "--version"], capture_output=True
+        )
+        if proc.returncode == 0:
+            # The pip has already been installed with the executable, just use it
+            return [executable, "-Esm", "pip"]
+        if python_major == 3:
+            # Use the host pip package.
+            return [executable, "-Es", os.path.dirname(pip_location)]
+        # For py2, only pip<21 is eligible, download a pip wheel from the Internet.
+        pip_wheel = self.project.cache_dir / "pip.whl"
+        if not pip_wheel.is_file():
+            self._download_pip_wheel(pip_wheel)
+        return [executable, str(pip_wheel / "pip")]
+
 
 class GlobalEnvironment(Environment):
     """Global environment"""
@@ -362,12 +396,12 @@ class GlobalEnvironment(Environment):
     is_global = True
 
     def get_paths(self) -> Dict[str, str]:
-        paths = get_sys_config_paths(self.python_executable)
+        paths = get_sys_config_paths(self.interpreter.executable)
         paths["prefix"] = paths["data"]
         paths["headers"] = paths["include"]
         return paths
 
-    def is_local(self, path) -> bool:
+    def is_local(self, path: PathLike) -> bool:
         return misc.normalize_path(path).startswith(
             misc.normalize_path(self.get_paths()["prefix"])
         )
