@@ -1,7 +1,7 @@
 import copy
 import operator
 from functools import reduce
-from typing import Any, Optional, Sequence, Tuple, Union
+from typing import Any, List, Optional, Sequence, Tuple, Union
 
 from pip._vendor.packaging.markers import Marker as PackageMarker
 
@@ -17,40 +17,6 @@ class Marker(PackageMarker):
         inst._markers = copy.deepcopy(self._markers)
         return inst
 
-    def __and__(self, other: Optional[PackageMarker]) -> "Marker":
-        """Intersect the two markers."""
-        if other is None or self == other:
-            return self
-        lhs = f"({self})" if "or" in self._markers else str(self)
-        rhs = f"({other})" if "or" in other._markers else str(other)
-        marker_str = f"{lhs} and {rhs}"
-        return type(self)(marker_str)
-
-    def __rand__(self, other: Optional[PackageMarker]) -> "Marker":
-        if other is None or self == other:
-            return self
-        rhs = f"({self})" if "or" in self._markers else str(self)
-        lhs = f"({other})" if "or" in other._markers else str(other)
-        marker_str = f"{lhs} and {rhs}"
-        return type(self)(marker_str)
-
-    def __or__(self, other: Optional[PackageMarker]) -> "Marker":
-        """Union the two markers."""
-        if None in (self, other):
-            return None
-        if self == other:
-            return self
-        marker_str = f"{self} or {other}"
-        return type(self)(marker_str)
-
-    def __ror__(self, other: Optional[PackageMarker]) -> "Marker":
-        if None in (self, other):
-            return None
-        if self == other:
-            return self
-        marker_str = f"{other} or {self}"
-        return type(self)(marker_str)
-
     def __eq__(self, other: Any) -> bool:
         if not isinstance(other, PackageMarker):
             return False
@@ -95,13 +61,15 @@ def split_marker_extras(
     :param marker: the input marker string
     :returns: an iterable of (op, value) pairs together with the stripped part.
     """
-    if "or" in marker._markers:
-        if "and" in marker._markers or any(
+    if "or" in marker._markers and (
+        "and" in marker._markers
+        or any(
             not isinstance(p, tuple) or p[0].value != "extra"
             for p in marker._markers
             if p != "or"
-        ):
-            return [], marker
+        )
+    ):
+        return [], marker
     result = []
     bare_markers = [m for m in marker._markers if m not in ("and", "or")]
     for m in bare_markers[:]:
@@ -120,7 +88,7 @@ def split_marker_extras(
     return result, marker
 
 
-def _only_contains_python_keys(markers):
+def _only_contains_python_keys(markers: List[Any]) -> bool:
     if isinstance(markers, tuple):
         return markers[0].value in ("python_version", "python_full_version")
 
@@ -132,8 +100,8 @@ def _only_contains_python_keys(markers):
     return True
 
 
-def _build_pyspec_from_marker(markers):
-    def split_version(version):
+def _build_pyspec_from_marker(markers: List[Any]) -> PySpecSet:
+    def split_version(version: str) -> List[str]:
         if "," in version:
             return [v.strip() for v in version.split(",")]
         return version.split()
