@@ -1,32 +1,45 @@
 from __future__ import annotations
 
+import dataclasses
 import sys
 from functools import lru_cache, wraps
-from typing import TYPE_CHECKING, Callable, Dict, Iterable, List, Optional, Tuple
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Tuple,
+)
 
 from pip._vendor.html5lib import parse
 
 from pdm import termui
 from pdm._types import CandidateInfo, Package, SearchResult, Source
-from pdm.exceptions import CandidateInfoNotFound, CorruptedCacheError
+from pdm.exceptions import CandidateInfoNotFound, CandidateNotFound, CorruptedCacheError
 from pdm.models.candidates import Candidate
 from pdm.models.requirements import (
     Requirement,
     filter_requirements_with_extras,
     parse_requirement,
 )
-from pdm.models.specifiers import PySpecSet, SpecifierSet
-from pdm.utils import allow_all_wheels
+from pdm.models.specifiers import PySpecSet, get_specifier
+from pdm.utils import allow_all_wheels, normalize_name, url_without_fragments
 
 if TYPE_CHECKING:
     from pdm.models.environment import Environment
 
+ALLOW_ALL_PYTHON = PySpecSet()
+
 
 def cache_result(
     func: Callable[["BaseRepository", Candidate], CandidateInfo]
 ) -> Callable[["BaseRepository", Candidate], CandidateInfo]:
     @wraps(func)
-    def wrapper(self, candidate: Candidate) -> CandidateInfo:
+    def wrapper(self: BaseRepository, candidate: Candidate) -> CandidateInfo:
         result = func(self, candidate)
         self._candidate_info_cache.set(candidate, result)
         return result
@@ -49,9 +62,7 @@ class BaseRepository:
 
     def get_filtered_sources(self, req: Requirement) -> List[Source]:
         """Get matching sources based on the index attribute."""
-        if not req.index:
-            return self.sources
-        return [source for source in self.sources if source["name"] == req.index]
+        return self.sources
 
     def get_dependencies(
         self, candidate: Candidate
@@ -75,8 +86,7 @@ class BaseRepository:
             # (same pinned version, no extras) as its dependency. This ensures
             # the same package with different extras (treated as distinct by
             # the resolver) have the same version.
-            self_req = candidate.req.copy()
-            self_req.extras = None
+            self_req = dataclasses.replace(candidate.req, extras=None)
             requirements.append(self_req)
         return requirements, PySpecSet(requires_python), summary
 
@@ -86,7 +96,7 @@ class BaseRepository:
     def find_candidates(
         self,
         requirement: Requirement,
-        requires_python: PySpecSet = PySpecSet(),
+        requires_python: PySpecSet = ALLOW_ALL_PYTHON,
         allow_prereleases: Optional[bool] = None,
         allow_all: bool = False,
     ) -> Iterable[Candidate]:
@@ -95,9 +105,6 @@ class BaseRepository:
         """
         # `allow_prereleases` is None means leave it to specifier to decide whether to
         # include prereleases
-        if allow_prereleases is None:
-            allow_prereleases = requirement.allow_prereleases
-
         requires_python = requires_python & requirement.requires_python
         cans = list(self._find_candidates(requirement))
 
@@ -112,6 +119,9 @@ class BaseRepository:
             reverse=True,
         )
 
+        if not sorted_cans:
+            termui.logger.debug("\tCould not find any matching candidates.")
+
         if not sorted_cans and allow_prereleases is None:
             # No non-pre-releases is found, force pre-releases now
             sorted_cans = sorted(
@@ -124,6 +134,35 @@ class BaseRepository:
                 key=lambda c: c.version,
                 reverse=True,
             )
+
+            if not sorted_cans:
+                termui.logger.debug(
+                    "\tCould not find any matching candidates even when considering "
+                    "pre-releases.",
+                )
+
+        def print_candidates(
+            title: str, candidates: List[Candidate], max_lines: int = 10
+        ) -> None:
+            termui.logger.debug("\t" + title)
+            logged_lines = set()
+            for can in candidates:
+                new_line = f"\t  {can}"
+                if new_line not in logged_lines:
+                    logged_lines.add(new_line)
+                    if len(logged_lines) > max_lines:
+                        termui.logger.debug(
+                            f"\t  ... [{len(candidates)-max_lines} more candidate(s)]"
+                        )
+                        break
+                    else:
+                        termui.logger.debug(new_line)
+
+        if sorted_cans:
+            print_candidates("Found matching candidates:", sorted_cans)
+        elif cans:
+            print_candidates("Found but non-matching candidates:", cans)
+
         return sorted_cans
 
     def _get_dependencies_from_cache(self, candidate: Candidate) -> CandidateInfo:
@@ -155,9 +194,13 @@ class BaseRepository:
             return
         if candidate.hashes:
             return candidate.hashes
-        req = candidate.req.copy()
-        req.specifier = SpecifierSet(f"=={candidate.version}")
-        matching_candidates = self.find_candidates(req, allow_all=True)
+        req = dataclasses.replace(
+            candidate.req, specifier=get_specifier(f"=={candidate.version}")
+        )
+        if candidate.req.is_file_or_url:
+            matching_candidates = [candidate]
+        else:
+            matching_candidates = self.find_candidates(req, allow_all=True)
         with self.environment.get_finder(self.sources) as finder:
             self._hash_cache.session = finder.session
             return {
@@ -231,10 +274,16 @@ class PyPIRepository(BaseRepository):
     def _find_candidates(self, requirement: Requirement) -> Iterable[Candidate]:
         sources = self.get_filtered_sources(requirement)
         with self.environment.get_finder(sources, True) as finder, allow_all_wheels():
-            return [
+            cans = [
                 Candidate.from_installation_candidate(c, requirement, self.environment)
                 for c in finder.find_all_candidates(requirement.project_name)
             ]
+        if not cans:
+            raise CandidateNotFound(
+                f"Unable to find candidates for {requirement.project_name}. There may "
+                "exist some issues with the package index or network condition."
+            )
+        return cans
 
     def search(self, query: str) -> SearchResult:
         pypi_simple = self.sources[0]["url"].rstrip("/")
@@ -278,3 +327,97 @@ class PyPIRepository(BaseRepository):
             results.append(result)
 
         return results
+
+
+class LockedRepository(BaseRepository):
+    def __init__(
+        self,
+        lockfile: Mapping[str, Any],
+        sources: List[Source],
+        environment: Environment,
+    ) -> None:
+        super().__init__(sources, environment)
+        self.packages: Dict[tuple, Candidate] = {}
+        self.file_hashes: Dict[Tuple[str, str], Dict[str, str]] = {}
+        self.candidate_info: Dict[tuple, CandidateInfo] = {}
+        self._read_lockfile(lockfile)
+
+    @property
+    def all_candidates(self) -> Dict[str, Candidate]:
+        return {can.req.identify(): can for can in self.packages.values()}
+
+    def _read_lockfile(self, lockfile: Mapping[str, Any]) -> None:
+        for package in lockfile.get("package", []):
+            version = package.get("version")
+            if version:
+                package["version"] = f"=={version}"
+            package_name = package.pop("name")
+            req_dict = {
+                k: v
+                for k, v in package.items()
+                if k not in ("dependencies", "requires_python", "summary")
+            }
+            req = Requirement.from_req_dict(package_name, req_dict)
+            can = Candidate(req, self.environment, name=package_name, version=version)
+            can_id = self._identify_candidate(can)
+            self.packages[can_id] = can
+            candidate_info: CandidateInfo = (
+                package.get("dependencies", []),
+                package.get("requires_python", ""),
+                package.get("summary", ""),
+            )
+            self.candidate_info[can_id] = candidate_info
+
+        for key, hashes in lockfile.get("metadata", {}).get("files", {}).items():
+            self.file_hashes[tuple(key.split())] = {
+                item["file"]: item["hash"] for item in hashes
+            }
+
+    def _identify_candidate(self, candidate: Candidate) -> tuple:
+        url = getattr(candidate.req, "url", None)
+        return (
+            candidate.identify(),
+            candidate.version,
+            url_without_fragments(url) if url else None,
+            candidate.req.editable,
+        )
+
+    def _get_dependencies_from_lockfile(self, candidate: Candidate) -> CandidateInfo:
+        return self.candidate_info[self._identify_candidate(candidate)]
+
+    def dependency_generators(self) -> Iterable[Callable[[Candidate], CandidateInfo]]:
+        return (self._get_dependencies_from_cache, self._get_dependencies_from_lockfile)
+
+    def get_dependencies(
+        self, candidate: Candidate
+    ) -> Tuple[List[Requirement], PySpecSet, str]:
+        reqs, python, summary = super().get_dependencies(candidate)
+        reqs = tuple(
+            req
+            for req in reqs
+            if not req.marker
+            or req.marker.evaluate(self.environment.marker_environment)
+        )
+        return reqs, python, summary
+
+    def find_candidates(
+        self,
+        requirement: Requirement,
+        requires_python: PySpecSet = ALLOW_ALL_PYTHON,
+        allow_prereleases: Optional[bool] = None,
+        allow_all: bool = False,
+    ) -> Iterable[Candidate]:
+        for key, info in self.candidate_info.items():
+            if key[0] != requirement.identify():
+                continue
+            if not (requires_python & PySpecSet(info[1])).contains(
+                str(self.environment.interpreter.version)
+            ):
+                continue
+            can = self.packages[key]
+            can.requires_python = info[1]
+            can.req = requirement
+            yield can
+
+    def get_hashes(self, candidate: Candidate) -> Optional[Dict[str, str]]:
+        return self.file_hashes.get((normalize_name(candidate.name), candidate.version))
