@@ -1,12 +1,18 @@
+from __future__ import annotations
+
+import dataclasses
+import functools
 import os
 import re
+import secrets
 import urllib.parse as urlparse
 import warnings
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
+from typing import Any, Sequence, Type, TypeVar, cast
 
 from pip._vendor.packaging.markers import InvalidMarker
 from pip._vendor.packaging.requirements import InvalidRequirement
+from pip._vendor.packaging.specifiers import SpecifierSet
 from pip._vendor.pkg_resources import Requirement as PackageRequirement
 from pip._vendor.pkg_resources import RequirementParseError, safe_name
 
@@ -21,11 +27,10 @@ from pdm.models.pip_shims import (
     path_to_url,
     url_to_path,
 )
-from pdm.models.readers import SetupReader
+from pdm.models.setup import Setup
 from pdm.models.specifiers import PySpecSet, get_specifier
 from pdm.utils import (
     add_ssh_scheme_to_git_uri,
-    is_readonly_property,
     parse_name_version_from_wheel,
     url_without_fragments,
 )
@@ -41,129 +46,125 @@ FILE_REQ = re.compile(
     r"|\"(?:[^\"]|\\\")*\"))"
     r"(?P<extras>\[[^\[\]]+\])?(?P<marker>[\t ]*;[^\n]+)?"
 )
+T = TypeVar("T", bound="Requirement")
 
 
-def strip_extras(line):
+def strip_extras(line: str) -> tuple[str, tuple[str, ...] | None]:
     match = re.match(r"^(.+?)(?:\[([^\]]+)\])?$", line)
     assert match is not None
-    name, extras = match.groups()
-    if extras:
-        extras = tuple(set(e.strip() for e in extras.split(",")))
-    else:
-        extras = None
+    name, extras_str = match.groups()
+    extras = (
+        tuple(set(e.strip() for e in extras_str.split(","))) if extras_str else None
+    )
     return name, extras
 
 
+@dataclasses.dataclass
 class Requirement:
     """Base class of a package requirement.
     A requirement is a (virtual) specification of a package which contains
     some constraints of version, python version, or other marker.
     """
 
-    attributes = (
-        "vcs",
-        "editable",
-        "name",
-        "specifier",
-        "specs",
-        "marker",
-        "extras",
-        "key",
-        "project_name",
-        "url",
-        "path",
-        "ref",
-        "revision",
-        "index",
-        "version",
-        "allow_prereleases",
-        "from_section",
-    )
+    name: str | None = None
+    marker: Marker | None = None
+    extras: Sequence[str] | None = None
+    specifier: SpecifierSet | None = None
+    editable: bool = False
 
-    def __init__(self, **kwargs):
-        self._marker = None
-        self.from_section = "default"
-        self.marker_no_python: Optional[Marker] = None
-        self.requires_python = PySpecSet()
-        for k, v in kwargs.items():
-            if k == "specifier":
-                v = get_specifier(v)
-            setattr(self, k, v)
-        if self.name and not self.project_name:
-            self.project_name = safe_name(self.name)
-            self.key = self.project_name.lower()
+    def __post_init__(self) -> None:
+        self.requires_python = (
+            self.marker.split_pyspec()[1] if self.marker else PySpecSet()
+        )
 
-    def __hash__(self):
-        hashCmp = (
+    @property
+    def project_name(self) -> str | None:
+        return safe_name(self.name) if self.name else None  # type: ignore
+
+    @property
+    def key(self) -> str | None:
+        return self.project_name.lower() if self.project_name else None
+
+    @property
+    def version(self) -> str | None:
+        if not self.specifier:
+            return None
+
+        is_pinned = len(self.specifier) == 1 and next(
+            iter(self.specifier)
+        ).operator in (
+            "==",
+            "===",
+        )
+        if is_pinned:
+            return next(iter(self.specifier)).version
+        return None
+
+    @version.setter
+    def version(self, v: str) -> None:
+        if not v or v == "*":
+            self.specifier = SpecifierSet()
+        else:
+            self.specifier = get_specifier(f"=={v}")
+
+    def _hash_key(self) -> tuple:
+        return (
             self.key,
-            self.url,
-            self.specifier,
             frozenset(self.extras) if self.extras else None,
             str(self.marker) if self.marker else None,
         )
-        return hash(hashCmp)
+
+    def __hash__(self) -> int:
+        return hash(self._hash_key())
 
     def __eq__(self, o: object) -> bool:
         return isinstance(o, Requirement) and hash(self) == hash(o)
 
-    @property
-    def marker(self) -> Optional[Marker]:
-        return self._marker
-
-    @marker.setter
-    def marker(self, value) -> None:
-        try:
-            m = self._marker = get_marker(value)
-            if not m:
-                self.marker_no_python, self.requires_python = None, PySpecSet()
-            else:
-                self.marker_no_python, self.requires_python = m.split_pyspec()
-        except InvalidMarker as e:
-            raise RequirementError("Invalid marker: %s" % str(e)) from None
-
-    def identify(self) -> Optional[str]:
+    @functools.lru_cache(maxsize=None)
+    def identify(self) -> str:
         if not self.key:
-            return None
+            return f":empty:{secrets.token_urlsafe(8)}"
         extras = "[{}]".format(",".join(sorted(self.extras))) if self.extras else ""
         return self.key + extras
 
-    def __getattr__(self, attr: str) -> Any:
-        if attr in self.attributes:
-            return None
-        raise AttributeError(attr)
-
     def __repr__(self) -> str:
         return f"<{self.__class__.__name__} {self.as_line()}>"
 
     def __str__(self) -> str:
         return self.as_line()
 
+    @classmethod
+    def create(cls: Type[T], **kwargs: Any) -> T:
+        if "marker" in kwargs:
+            try:
+                kwargs["marker"] = get_marker(kwargs["marker"])
+            except InvalidMarker as e:
+                raise RequirementError("Invalid marker: %s" % str(e)) from None
+        if "extras" in kwargs and isinstance(kwargs["extras"], str):
+            kwargs["extras"] = tuple(
+                e.strip() for e in kwargs["extras"][1:-1].split(",")
+            )
+        version = kwargs.pop("version", None)
+        if version:
+            kwargs["specifier"] = get_specifier(version)
+        return cls(**kwargs)
+
     @classmethod
     def from_req_dict(cls, name: str, req_dict: RequirementDict) -> "Requirement":
-        # TODO: validate req_dict
         if isinstance(req_dict, str):  # Version specifier only.
-            return NamedRequirement(name=name, specifier=req_dict)
+            return NamedRequirement(name=name, specifier=get_specifier(req_dict))
         for vcs in VCS_SCHEMA:
             if vcs in req_dict:
-                repo = req_dict[vcs]  # type: str
+                repo = cast(str, req_dict.pop(vcs, None))
                 url = (
                     vcs
                     + "+"
                     + VcsRequirement._build_url_from_req_dict(name, repo, req_dict)
                 )
-                return VcsRequirement(name=name, vcs=vcs, url=url, **req_dict)
+                return VcsRequirement.create(name=name, vcs=vcs, url=url, **req_dict)
         if "path" in req_dict or "url" in req_dict:
-            return FileRequirement(name=name, **req_dict)
-        specifier = req_dict.pop("version", None)
-        return NamedRequirement(name=name, specifier=specifier, **req_dict)
-
-    def copy(self) -> "Requirement":
-        kwargs = {
-            k: getattr(self, k, None)
-            for k in self.attributes
-            if not is_readonly_property(self.__class__, k)
-        }
-        return self.__class__(**kwargs)
+            return FileRequirement.create(name=name, **req_dict)
+        return NamedRequirement.create(name=name, **req_dict)
 
     @property
     def is_named(self) -> bool:
@@ -180,31 +181,6 @@ class Requirement:
     def as_line(self) -> str:
         raise NotImplementedError
 
-    def as_req_dict(self) -> Tuple[str, RequirementDict]:
-        r = {}
-        if self.editable:
-            r["editable"] = True
-        if self.extras:
-            r["extras"] = sorted(self.extras)
-        if self.is_vcs:
-            r[self.vcs] = self.repo
-        elif self.path and self.is_local_dir:
-            r["path"] = self.str_path
-        elif self.url:
-            r["url"] = self.url
-        if self.marker:
-            r["marker"] = str(self.marker).replace('"', "'")
-        if self.specifier:
-            r["version"] = str(self.specifier)
-        elif self.is_named:
-            r["version"] = "*"
-        if len(r) == 1 and next(iter(r), None) == "version":
-            r = r["version"]
-        for attr in ["index", "allow_prereleases", "ref"]:
-            if getattr(self, attr) is not None:
-                r[attr] = getattr(self, attr)
-        return self.project_name, r
-
     def matches(self, line: str, editable_match: bool = True) -> bool:
         """Return whether the passed in PEP 508 string
         is the same requirement as this one.
@@ -217,26 +193,30 @@ class Requirement:
             not editable_match or self.editable == req.editable
         )
 
-    def as_ireq(self, **kwargs) -> InstallRequirement:
+    def as_ireq(self, **kwargs: Any) -> InstallRequirement:
         line_for_req = self.as_line()
         if self.editable:
             line_for_req = line_for_req[3:].strip()
             ireq = install_req_from_editable(line_for_req, **kwargs)
         else:
             ireq = install_req_from_line(line_for_req, **kwargs)
-        ireq.req = self
+        ireq.req = self  # type: ignore
         return ireq
 
     @classmethod
     def from_pkg_requirement(cls, req: PackageRequirement) -> "Requirement":
-        klass = FileRequirement if req.url else NamedRequirement
-        return klass(
-            name=req.name,
-            extras=req.extras,
-            url=req.url,
-            specifier=req.specifier,
-            marker=req.marker,
-        )
+        kwargs = {
+            "name": req.name,
+            "extras": req.extras,
+            "specifier": req.specifier,
+            "marker": get_marker(req.marker),
+        }
+        if getattr(req, "url", None):
+            link = Link(req.url)
+            klass = VcsRequirement if link.is_vcs else FileRequirement
+            return klass(url=req.url, **kwargs)  # type: ignore
+        else:
+            return NamedRequirement(**kwargs)
 
     def _format_marker(self) -> str:
         if self.marker:
@@ -244,21 +224,43 @@ class Requirement:
         return ""
 
 
+@dataclasses.dataclass
+class NamedRequirement(Requirement):
+    def __hash__(self) -> int:
+        return hash(self._hash_key())
+
+    def as_line(self) -> str:
+        extras = f"[{','.join(sorted(self.extras))}]" if self.extras else ""
+        return f"{self.project_name}{extras}{self.specifier}{self._format_marker()}"
+
+
+@dataclasses.dataclass
 class FileRequirement(Requirement):
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-        self.path = (
-            Path(str(self.path).replace("${PROJECT_ROOT}", ".")) if self.path else None
-        )
-        self.version = None
+    url: str = ""
+    path: Path | None = None
+
+    def __post_init__(self) -> None:
+        super().__post_init__()
         self._parse_url()
         if self.path and not self.path.exists():
             raise RequirementError(f"The local path {self.path} does not exist.")
         if self.is_local_dir:
             self._check_installable()
 
+    def _hash_key(self) -> tuple:
+        return super()._hash_key() + (self.url, self.editable)
+
+    def __hash__(self) -> int:
+        return hash(self._hash_key())
+
+    @classmethod
+    def create(cls: Type[T], **kwargs: Any) -> T:
+        if kwargs.get("path"):
+            kwargs["path"] = Path(kwargs["path"].replace("${PROJECT_ROOT}", "."))
+        return super().create(**kwargs)  # type: ignore
+
     @property
-    def str_path(self) -> Optional[str]:
+    def str_path(self) -> str | None:
         if not self.path:
             return None
         result = self.path.as_posix()
@@ -270,19 +272,6 @@ class FileRequirement(Requirement):
             result = "./" + result
         return result
 
-    @classmethod
-    def parse(cls, line: str, parsed: Dict[str, str]) -> "FileRequirement":
-        extras = parsed.get("extras")
-        if extras:
-            extras = tuple(e.strip() for e in extras[1:-1].split(","))
-        r = cls(
-            url=parsed.get("url"),
-            path=parsed.get("path"),
-            marker=parsed.get("marker"),
-            extras=extras,
-        )
-        return r
-
     def _parse_url(self) -> None:
         if not self.url:
             if self.path:
@@ -305,19 +294,11 @@ class FileRequirement(Requirement):
 
     @property
     def is_local(self) -> bool:
-        return self.path and self.path.exists()
+        return self.path and self.path.exists() or False
 
     @property
     def is_local_dir(self) -> bool:
-        return self.is_local and self.path.is_dir()
-
-    @property
-    def project_name(self) -> Optional[str]:
-        return safe_name(self.name) if self.name else None
-
-    @property
-    def key(self) -> Optional[str]:
-        return self.project_name.lower() if self.project_name else None
+        return self.is_local and cast(Path, self.path).is_dir()
 
     def as_line(self) -> str:
         project_name = f"{self.project_name}" if self.project_name else ""
@@ -343,53 +324,36 @@ class FileRequirement(Requirement):
                 self.name, self.version = parse_name_version_from_wheel(filename)
 
     def _check_installable(self) -> None:
+        assert self.path
         if not (
             self.path.joinpath("setup.py").exists()
             or self.path.joinpath("pyproject.toml").exists()
         ):
             raise RequirementError(f"The local path '{self.path}' is not installable.")
-        result = SetupReader.read_from_directory(self.path.absolute().as_posix())
-        self.name = result["name"]
-
-
-class NamedRequirement(Requirement, PackageRequirement):
-    @classmethod
-    def parse(
-        cls, line: str, parsed: Optional[Dict[str, Optional[str]]] = None
-    ) -> "NamedRequirement":
-        r = cls()
-        try:
-            PackageRequirement.__init__(r, line)
-            r.marker = get_marker(r.marker)
-        except InvalidMarker as e:
-            raise RequirementError("Invalid marker: %s" % str(e)) from None
-        return r
-
-    def as_line(self) -> str:
-        extras = f"[{','.join(sorted(self.extras))}]" if self.extras else ""
-        return f"{self.project_name}{extras}{self.specifier}{self._format_marker()}"
+        result = Setup.from_directory(self.path.absolute())
+        self.name = result.name
 
 
+@dataclasses.dataclass
 class VcsRequirement(FileRequirement):
-    def __init__(self, **kwargs):
-        self.repo = None
-        if not kwargs.get("vcs"):
-            kwargs["vcs"] = kwargs["url"].split("+", 1)[0]
-        super().__init__(**kwargs)
+    vcs: str = ""
+    ref: str | None = None
+    revision: str | None = None
 
-    @classmethod
-    def parse(cls, line: str, parsed: Dict[str, str]) -> "VcsRequirement":
-        r = cls(
-            url=parsed.get("url"), vcs=parsed.get("vcs"), marker=parsed.get("marker")
-        )
-        return r
+    def __hash__(self) -> int:
+        return hash(self._hash_key())
 
-    def as_ireq(self, **kwargs) -> InstallRequirement:
+    def __post_init__(self) -> None:
+        super().__post_init__()
+        if not self.vcs:
+            self.vcs = self.url.split("+", 1)[0]
+
+    def as_ireq(self, **kwargs: Any) -> InstallRequirement:
         ireq = super().as_ireq(**kwargs)
         if not self.editable and self.revision:
             # For non-editable VCS requirements, commit-hash should be used as the
             # rev-options for InstallRequirement to consume.
-            parsed = urlparse.urlparse(ireq.link.url)
+            parsed = urlparse.urlparse(cast(Link, ireq.link).url)
             new_path = "@".join((parsed.path.split("@", 1)[0], self.revision))
             new_url = urlparse.urlunparse(parsed._replace(path=new_path))
             ireq.link = Link(new_url)
@@ -403,27 +367,31 @@ class VcsRequirement(FileRequirement):
         if not self.name:
             self._parse_name_from_url()
         repo = url_without_fragments(url_no_vcs)
-        ref = None
+        ref: str | None = None
         parsed = urlparse.urlparse(repo)
         if "@" in parsed.path:
             path, ref = parsed.path.split("@", 1)
             repo = urlparse.urlunparse(parsed._replace(path=path))
-        self.repo, self.ref = repo, ref
+        self.repo, self.ref = repo, ref  # type: ignore
 
     @staticmethod
     def _build_url_from_req_dict(name: str, url: str, req_dict: RequirementDict) -> str:
+        assert not isinstance(req_dict, str)
         ref = f"@{req_dict['ref']}" if "ref" in req_dict else ""
         fragments = f"#egg={urlparse.quote(name)}"
         if "subdirectory" in req_dict:
-            fragments += f"&subdirectory={urlparse.quote(req_dict['subdirectory'])}"
+            fragments += (
+                "&subdirectory="  # type: ignore
+                f"{urlparse.quote(req_dict.pop('subdirectory'))}"  # type: ignore
+            )
         return f"{url}{ref}{fragments}"
 
 
 def filter_requirements_with_extras(
-    requirement_lines: List[Union[str, Dict[str, Union[str, List[str]]]]],
+    requirement_lines: list[str | dict[str, str | list[str]]],
     extras: Sequence[str],
-) -> List[str]:
-    result = []
+) -> list[str]:
+    result: list[str] = []
     extras_in_meta = []
     for req in requirement_lines:
         if isinstance(req, dict):
@@ -438,7 +406,7 @@ def filter_requirements_with_extras(
                 result.append(req)
             else:
                 elements, rest = split_marker_extras(_r.marker)
-                extras_in_meta.extend(e for e in elements)
+                extras_in_meta.extend(elements)
                 _r.marker = rest
                 if not elements or set(extras) & set(elements):
                     result.append(_r.as_line())
@@ -453,31 +421,23 @@ def filter_requirements_with_extras(
 def parse_requirement(line: str, editable: bool = False) -> Requirement:
 
     m = VCS_REQ.match(line)
+    r: Requirement
     if m is not None:
-        r = VcsRequirement.parse(line, m.groupdict())
+        r = VcsRequirement.create(**m.groupdict())
     else:
         try:
-            r = NamedRequirement.parse(line)
+            package_req = PackageRequirement(line)  # type: ignore
         except (RequirementParseError, InvalidRequirement) as e:
             m = FILE_REQ.match(line)
-            if m is not None:
-                r = FileRequirement.parse(line, m.groupdict())
-            else:
+            if m is None:
                 raise RequirementError(str(e)) from None
+            r = FileRequirement.create(**m.groupdict())
         else:
-            if r.url:
-                link = Link(r.url)
-                if link.is_vcs:
-                    r = VcsRequirement(
-                        name=r.name, url=r.url, extras=r.extras, marker=r.marker
-                    )
-                else:
-                    r = FileRequirement(
-                        name=r.name, url=r.url, extras=r.extras, marker=r.marker
-                    )
+            r = Requirement.from_pkg_requirement(package_req)
 
     if editable:
-        if r.is_vcs or r.is_file_or_url and r.is_local_dir:
+        if r.is_vcs or r.is_file_or_url and r.is_local_dir:  # type: ignore
+            assert isinstance(r, FileRequirement)
             r.editable = True
         else:
             raise RequirementError(
