@@ -1,12 +1,11 @@
 import dataclasses
 import os
-from collections.abc import MutableMapping
 from pathlib import Path
-from typing import Any, Callable, Dict, Iterable, Optional, TypeVar
+from typing import Any, Callable, Dict, Iterator, MutableMapping, Optional, TypeVar
 
 import appdirs
+import atoml
 import click
-import tomlkit
 
 from pdm import termui
 from pdm.exceptions import NoConfigError
@@ -21,7 +20,7 @@ def load_config(file_path: Path) -> Dict[str, Any]:
     E.g. ["python"]["path"] will be loaded as "python.path" key.
     """
 
-    def get_item(sub_data):
+    def get_item(sub_data: Dict[str, Any]) -> Dict[str, Any]:
         result = {}
         for k, v in sub_data.items():
             if getattr(v, "items", None) is not None:
@@ -34,7 +33,7 @@ def load_config(file_path: Path) -> Dict[str, Any]:
 
     if not file_path.is_file():
         return {}
-    return get_item(dict(tomlkit.parse(file_path.read_text("utf-8"))))
+    return get_item(dict(atoml.parse(file_path.read_text("utf-8"))))
 
 
 def ensure_boolean(val: Any) -> bool:
@@ -42,7 +41,7 @@ def ensure_boolean(val: Any) -> bool:
     if not isinstance(val, str):
         return val
 
-    return val and val.lower() not in ("false", "no", "0")
+    return bool(val) and val.lower() not in ("false", "no", "0")
 
 
 @dataclasses.dataclass
@@ -61,13 +60,13 @@ class ConfigItem:
     default: Any = _NOT_SET
     global_only: bool = False
     env_var: Optional[str] = None
-    coerce: Callable[[Any], Any] = str
+    coerce: Callable = str
 
     def should_show(self) -> bool:
         return self.default is not self._NOT_SET
 
 
-class Config(MutableMapping):
+class Config(MutableMapping[str, str]):
     """A dict-like object for configuration key and values"""
 
     HOME_CONFIG = Path.home() / ".pdm" / "config.toml"
@@ -130,7 +129,7 @@ class Config(MutableMapping):
     del pypi_url, verify_ssl
 
     @classmethod
-    def get_defaults(cls):
+    def get_defaults(cls) -> Dict[str, Any]:
         return {k: v.default for k, v in cls._config_map.items() if v.should_show()}
 
     @classmethod
@@ -151,7 +150,7 @@ class Config(MutableMapping):
     def _save_config(self) -> None:
         """Save the changed to config file."""
         self._config_file.parent.mkdir(parents=True, exist_ok=True)
-        toml_data = {}
+        toml_data: Dict[str, Any] = {}
         for key, value in self._file_data.items():
             *parts, last = key.split(".")
             temp = toml_data
@@ -162,7 +161,7 @@ class Config(MutableMapping):
             temp[last] = value
 
         with self._config_file.open("w", encoding="utf-8") as fp:
-            fp.write(tomlkit.dumps(toml_data))
+            atoml.dump(toml_data, fp)
 
     def __getitem__(self, key: str) -> Any:
         env_var = self._config_map[key].env_var
@@ -199,10 +198,10 @@ class Config(MutableMapping):
     def __len__(self) -> int:
         return len(self._data)
 
-    def __iter__(self) -> Iterable[str]:
+    def __iter__(self) -> Iterator[str]:
         return iter(self._data)
 
-    def __delitem__(self, key) -> None:
+    def __delitem__(self, key: str) -> None:
         self._data.pop(key, None)
         try:
             del self._file_data[key]
