@@ -9,23 +9,21 @@ import sys
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Type, Union
 
-import tomlkit
+import atoml
 from pythonfinder import Finder
 from pythonfinder.environment import PYENV_INSTALLED, PYENV_ROOT
-from pythonfinder.models.python import PythonVersion
-from tomlkit.items import Comment, Whitespace
 
 from pdm import termui
 from pdm._types import Source
-from pdm.exceptions import NoPythonVersion, ProjectError
+from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
 from pdm.models import pip_shims
 from pdm.models.caches import CandidateInfoCache, HashCache
 from pdm.models.candidates import Candidate
 from pdm.models.environment import Environment, GlobalEnvironment
-from pdm.models.in_process import get_python_version
-from pdm.models.repositories import BaseRepository, PyPIRepository
+from pdm.models.python import PythonInfo
+from pdm.models.repositories import BaseRepository, LockedRepository, PyPIRepository
 from pdm.models.requirements import Requirement, parse_requirement
-from pdm.models.specifiers import PySpecSet
+from pdm.models.specifiers import PySpecSet, get_specifier
 from pdm.project.config import Config
 from pdm.project.metadata import MutableMetadata as Metadata
 from pdm.utils import (
@@ -36,12 +34,10 @@ from pdm.utils import (
     find_python_in_path,
     get_in_project_venv_python,
     is_venv_python,
-    setdefault,
 )
 
 if TYPE_CHECKING:
     from resolvelib.reporters import BaseReporter
-    from tomlkit.container import Container
 
     from pdm._vendor import halo
     from pdm.core import Core
@@ -53,35 +49,32 @@ class Project:
 
     PYPROJECT_FILENAME = "pyproject.toml"
     DEPENDENCIES_RE = re.compile(r"(?:(.+?)-)?dependencies")
-    PYPROJECT_VERSION = "2"
+    LOCKFILE_VERSION = "3"
     GLOBAL_PROJECT = Path.home() / ".pdm" / "global-project"
 
-    core: Core
-
-    @classmethod
-    def create_global(cls, root_path: Optional[str] = None) -> "Project":
-        if root_path is None:
-            root_path = cls.GLOBAL_PROJECT.as_posix()
-        project = cls(root_path)
-        project.is_global = True
-        project.init_global_project()
-        return project
-
-    def __init__(self, root_path: Optional[str] = None) -> None:
-        self.is_global = False
-        self._pyproject: Optional[Container] = None
-        self._lockfile: Optional[Container] = None
+    def __init__(
+        self, core: Core, root_path: Optional[os.PathLike], is_global: bool = False
+    ) -> None:
+        self._pyproject: Optional[Dict] = None
+        self._lockfile: Optional[Dict] = None
         self._environment: Optional[Environment] = None
-        self._python_executable: Optional[str] = None
+        self._python: Optional[PythonInfo] = None
+        self.core = core
 
         if root_path is None:
-            root_path = find_project_root()
-        if root_path is None and self.global_config["auto_global"]:
-            self.root = self.GLOBAL_PROJECT
-            self.is_global = True
-            self.init_global_project()
-        else:
-            self.root = Path(root_path or "").absolute()
+            root_path = find_project_root() if not is_global else self.GLOBAL_PROJECT
+        if not is_global and root_path is None and self.global_config["auto_global"]:
+            self.core.ui.echo(
+                "Project is not found, fallback to the global project",
+                fg="yellow",
+                err=True,
+            )
+            root_path = self.GLOBAL_PROJECT
+            is_global = True
+
+        self.root = Path(root_path or "").absolute()
+        self.is_global = is_global
+        self.init_global_project()
 
     def __repr__(self) -> str:
         return f"<Project '{self.root.as_posix()}'>"
@@ -95,32 +88,36 @@ class Project:
         return self.root / "pdm.lock"
 
     @property
-    def pyproject(self) -> Container:
+    def pyproject(self) -> Optional[dict]:
         if not self._pyproject and self.pyproject_file.exists():
-            data = tomlkit.parse(self.pyproject_file.read_text("utf-8"))
+            data = atoml.parse(self.pyproject_file.read_text("utf-8"))
             self._pyproject = data
         return self._pyproject
 
     @pyproject.setter
-    def pyproject(self, data):
+    def pyproject(self, data: Dict[str, Any]) -> None:
         self._pyproject = data
 
     @property
-    def tool_settings(self) -> Union[Container, Dict]:
+    def tool_settings(self) -> dict:
         data = self.pyproject
         if not data:
             return {}
-        return setdefault(setdefault(data, "tool", {}), "pdm", {})
+        return data.setdefault("tool", {}).setdefault("pdm", {})
 
     @property
-    def lockfile(self) -> Container:
-        if not self.lockfile_file.is_file():
-            raise ProjectError("Lock file does not exist.")
+    def lockfile(self) -> dict:
         if not self._lockfile:
-            data = tomlkit.parse(self.lockfile_file.read_text("utf-8"))
+            if not self.lockfile_file.is_file():
+                raise ProjectError("Lock file does not exist.")
+            data = atoml.parse(self.lockfile_file.read_text("utf-8"))
             self._lockfile = data
         return self._lockfile
 
+    @lockfile.setter
+    def lockfile(self, data: Dict[str, Any]) -> None:
+        self._lockfile = data
+
     @property
     def config(self) -> Dict[str, Any]:
         """A read-only dict configuration, any modifications won't land in the file."""
@@ -142,41 +139,49 @@ class Project:
         """Read-and-writable configuration dict for project settings"""
         return Config(self.root / ".pdm.toml")
 
+    @property
+    def python(self) -> PythonInfo:
+        if not self._python:
+            self._python = self.resolve_interpreter()
+        return self._python
+
+    @python.setter
+    def python(self, value: PythonInfo) -> None:
+        self._python = value
+        self.project_config["python.path"] = value.executable
+
     @property
     def python_executable(self) -> str:
-        if not self._python_executable:
-            self._python_executable = self.resolve_interpreter()
-        return self._python_executable
+        """For backward compatibility"""
+        return self.python.executable
 
-    @python_executable.setter
-    def python_executable(self, value: str) -> None:
-        self._python_executable = value
-        self.project_config["python.path"] = value
-
-    def resolve_interpreter(self) -> str:
+    def resolve_interpreter(self) -> PythonInfo:
         """Get the Python interpreter path."""
         config = self.config
         if self.project_config.get("python.path") and not os.getenv(
             "PDM_IGNORE_SAVED_PYTHON"
         ):
             saved_path = self.project_config["python.path"]
-            if not os.path.isfile(saved_path):
+            try:
+                return PythonInfo.from_path(saved_path)
+            except (ValueError, FileNotFoundError):
                 del self.project_config["python.path"]
-            else:
-                return saved_path
         if os.name == "nt":
             suffix = ".exe"
             scripts = "Scripts"
         else:
             suffix = ""
             scripts = "bin"
-        if config["use_venv"] and os.getenv("VIRTUAL_ENV"):
-            return os.path.join(os.getenv("VIRTUAL_ENV"), scripts, f"python{suffix}")
+        virtual_env = os.getenv("VIRTUAL_ENV")
+        if config["use_venv"] and virtual_env:
+            return PythonInfo.from_path(
+                os.path.join(virtual_env, scripts, f"python{suffix}")
+            )
 
         for py_version in self.find_interpreters():
             if self.python_requires.contains(str(py_version.version)):
-                self.python_executable = py_version.executable
-                return self.python_executable
+                self.python = py_version
+                return py_version
 
         raise NoPythonVersion(
             "No Python that satisfies {} is found on the system.".format(
@@ -190,11 +195,9 @@ class Project:
             env = GlobalEnvironment(self)
             # Rewrite global project's python requires to be
             # compatible with the exact version
-            env.python_requires = PySpecSet(
-                "==" + get_python_version(self.python_executable, True)[0]
-            )
+            env.python_requires = PySpecSet(f"=={self.python.version}")
             return env
-        if self.config["use_venv"] and is_venv_python(self.python_executable):
+        if self.config["use_venv"] and is_venv_python(self.python.executable):
             # Only recognize venv created by python -m venv and virtualenv>20
             return GlobalEnvironment(self)
         return Environment(self)
@@ -205,18 +208,34 @@ class Project:
             self._environment = self.get_environment()
         return self._environment
 
+    @environment.setter
+    def environment(self, value: Environment) -> None:
+        self._environment = value
+
     @property
     def python_requires(self) -> PySpecSet:
         return PySpecSet(self.meta.requires_python)
 
     def get_dependencies(self, section: Optional[str] = None) -> Dict[str, Requirement]:
         metadata = self.meta
+        optional_dependencies = metadata.get("optional-dependencies", {})
+        dev_dependencies = self.tool_settings.get("dev-dependencies", {})
         if section in (None, "default"):
             deps = metadata.get("dependencies", [])
-        elif section == "dev":
-            deps = metadata.get("dev-dependencies", [])
         else:
-            deps = metadata.get("optional-dependencies", {}).get(section, [])
+            if section in optional_dependencies and section in dev_dependencies:
+                self.core.ui.echo(
+                    f"The {section} section exists in both [optional-dependencies] "
+                    "and [dev-dependencies], the former is taken.",
+                    err=True,
+                    fg="yellow",
+                )
+            if section in optional_dependencies:
+                deps = optional_dependencies[section]
+            elif section in dev_dependencies:
+                deps = dev_dependencies[section]
+            else:
+                raise PdmUsageError(f"Non-exist section {section}")
         result = {}
         with cd(self.root):
             for line in deps:
@@ -224,7 +243,6 @@ class Project:
                     req = parse_requirement(line[3:].strip(), True)
                 else:
                     req = parse_requirement(line)
-                req.from_section = section or "default"
                 # make editable packages behind normal ones to override correctly.
                 result[req.identify()] = req
         return result
@@ -235,13 +253,28 @@ class Project:
 
     @property
     def dev_dependencies(self) -> Dict[str, Requirement]:
-        return self.get_dependencies("dev")
+        """All development dependencies"""
+        dev_group = self.tool_settings.get("dev-dependencies", {})
+        if not dev_group:
+            return {}
+        result = {}
+        with cd(self.root):
+            for _, deps in dev_group.items():
+                for line in deps:
+                    if line.startswith("-e "):
+                        req = parse_requirement(line[3:].strip(), True)
+                    else:
+                        req = parse_requirement(line)
+                    result[req.identify()] = req
+        return result
 
     def iter_sections(self) -> Iterable[str]:
-        yield "default"
-        yield "dev"
+        result = {"default"}
         if self.meta.optional_dependencies:
-            yield from self.meta.optional_dependencies.keys()
+            result.update(self.meta.optional_dependencies.keys())
+        if self.tool_settings.get("dev-dependencies"):
+            result.update(self.tool_settings["dev-dependencies"].keys())
+        return result
 
     @property
     def all_dependencies(self) -> Dict[str, Dict[str, Requirement]]:
@@ -256,7 +289,7 @@ class Project:
     @property
     def sources(self) -> List[Source]:
         sources = list(self.tool_settings.get("source", []))
-        if not any(source.get("name") == "pypi" for source in sources):
+        if all(source.get("name") != "pypi" for source in sources):
             sources.insert(
                 0,
                 {
@@ -276,17 +309,31 @@ class Project:
         sources = self.sources or []
         return cls(sources, self.environment)
 
+    @property
+    def locked_repository(self) -> LockedRepository:
+        import copy
+
+        try:
+            lockfile = copy.deepcopy(self.lockfile)
+        except ProjectError:
+            lockfile = {}
+
+        return LockedRepository(lockfile, self.sources, self.environment)
+
     def get_provider(
         self,
         strategy: str = "all",
         tracked_names: Optional[Iterable[str]] = None,
+        for_install: bool = False,
     ) -> BaseProvider:
         """Build a provider class for resolver.
 
         :param strategy: the resolve strategy
         :param tracked_names: the names of packages that needs to update
+        :param for_install: if the provider is for install
         :returns: The provider object
         """
+
         from pdm.resolver.providers import (
             BaseProvider,
             EagerUpdateProvider,
@@ -294,29 +341,37 @@ class Project:
         )
 
         repository = self.get_repository(cls=self.core.repository_class)
-        allow_prereleases = self.allow_prereleases
         requires_python = self.environment.python_requires
-        if strategy == "all":
-            provider = BaseProvider(repository, requires_python, allow_prereleases)
-        else:
-            provider_class = (
-                ReusePinProvider if strategy == "reuse" else EagerUpdateProvider
+        allow_prereleases = self.allow_prereleases
+        if strategy != "all" and not self.is_lockfile_compatible():
+            self.core.ui.echo(
+                "Updating the whole lock file as it is not compatible with PDM",
+                fg="yellow",
+                err=True,
             )
-            preferred_pins = self.get_locked_candidates("__all__")
-            provider = provider_class(
-                preferred_pins,
-                tracked_names or (),
-                repository,
-                requires_python,
-                allow_prereleases,
-            )
-        return provider
+            strategy = "all"
+        if not for_install and strategy == "all":
+            return BaseProvider(repository, requires_python, allow_prereleases)
+
+        locked_repository = self.locked_repository
+        if for_install:
+            return BaseProvider(locked_repository, requires_python, allow_prereleases)
+        provider_class = (
+            ReusePinProvider if strategy == "reuse" else EagerUpdateProvider
+        )
+        return provider_class(
+            locked_repository.all_candidates,
+            tracked_names or (),
+            repository,
+            requires_python,
+            allow_prereleases,
+        )
 
     def get_reporter(
         self,
         requirements: List[Requirement],
         tracked_names: Optional[Iterable[str]] = None,
-        spinner: Optional[halo.Halo] = None,
+        spinner: Union[halo.Halo, termui.DummySpinner] = None,
     ) -> BaseReporter:
         """Return the reporter object to construct a resolver.
 
@@ -330,19 +385,23 @@ class Project:
         return SpinnerReporter(spinner, requirements)
 
     def get_lock_metadata(self) -> Dict[str, Any]:
-        content_hash = tomlkit.string("sha256:" + self.get_content_hash("sha256"))
+        content_hash = atoml.string("sha256:" + self.get_content_hash("sha256"))
         content_hash.trivia.trail = "\n\n"
-        data = {"lock_version": self.PYPROJECT_VERSION, "content_hash": content_hash}
-        return data
+        return {"lock_version": self.LOCKFILE_VERSION, "content_hash": content_hash}
 
-    def write_lockfile(self, toml_data: Container, show_message: bool = True) -> None:
+    def write_lockfile(
+        self, toml_data: Dict, show_message: bool = True, write: bool = True
+    ) -> None:
         toml_data["metadata"].update(self.get_lock_metadata())
 
-        with atomic_open_for_write(self.lockfile_file) as fp:
-            fp.write(tomlkit.dumps(toml_data))
-        if show_message:
-            self.core.ui.echo(f"Changes are written to {termui.green('pdm.lock')}.")
-        self._lockfile = None
+        if write:
+            with atomic_open_for_write(self.lockfile_file) as fp:
+                atoml.dump(toml_data, fp)
+            if show_message:
+                self.core.ui.echo(f"Changes are written to {termui.green('pdm.lock')}.")
+            self._lockfile = None
+        else:
+            self._lockfile = toml_data
 
     def make_self_candidate(self, editable: bool = True) -> Candidate:
         req = parse_requirement(pip_shims.path_to_url(self.root.as_posix()), editable)
@@ -351,42 +410,13 @@ class Project:
             req, self.environment, name=self.meta.name, version=self.meta.version
         )
 
-    def get_locked_candidates(
-        self, section: Optional[str] = None
-    ) -> Dict[str, Candidate]:
-        if not self.lockfile_file.is_file():
-            return {}
-        section = section or "default"
-        result = {}
-        for package in [dict(p) for p in self.lockfile.get("package", [])]:
-            if section != "__all__" and section not in package["sections"]:
-                continue
-            version = package.get("version")
-            if version:
-                package["version"] = f"=={version}"
-            package_name = package.pop("name")
-            req = Requirement.from_req_dict(package_name, dict(package))
-            can = Candidate(req, self.environment, name=package_name, version=version)
-            can.sections = package.get("sections", [])
-            can.marker = req.marker
-            can.hashes = {
-                item["file"]: item["hash"]
-                for item in self.lockfile["metadata"]
-                .get("files", {})
-                .get(f"{req.key} {version}", [])
-            } or None
-            result[req.identify()] = can
-        if section in ("default", "__all__") and self.meta.name and self.meta.version:
-            result[self.meta.project_name.lower()] = self.make_self_candidate(True)
-        return result
-
     def get_content_hash(self, algo: str = "md5") -> str:
         # Only calculate sources and dependencies sections. Otherwise lock file is
         # considered as unchanged.
         dump_data = {
             "sources": self.tool_settings.get("source", []),
             "dependencies": self.meta.get("dependencies", []),
-            "dev-dependencies": self.meta.get("dev-dependencies", []),
+            "dev-dependencies": self.tool_settings.get("dev-dependencies", {}),
             "optional-dependencies": self.meta.get("optional-dependencies", {}),
             "requires-python": self.meta.get("requires-python", ""),
         }
@@ -407,22 +437,40 @@ class Project:
         content_hash = self.get_content_hash(algo)
         return content_hash == hash_value
 
-    def get_pyproject_dependencies(self, section: str) -> List[str]:
+    def is_lockfile_compatible(self) -> bool:
+        if not self.lockfile_file.exists():
+            return False
+        lockfile_version = str(
+            self.lockfile.get("metadata", {}).get("lock_version", "")
+        )
+        if "." not in lockfile_version:
+            lockfile_version += ".0"
+        accepted = get_specifier(f"~={lockfile_version}")
+        return accepted.contains(self.LOCKFILE_VERSION)
+
+    def get_pyproject_dependencies(self, section: str, dev: bool = False) -> List[str]:
         """Get the dependencies array in the pyproject.toml"""
         if section == "default":
-            return setdefault(self.meta, "dependencies", [])
-        elif section == "dev":
-            return setdefault(self.meta, "dev-dependencies", [])
+            return self.meta.setdefault("dependencies", [])
         else:
-            return setdefault(
-                setdefault(self.meta, "optional-dependencies", {}), section, []
-            )
+            deps_dict = {
+                False: self.meta.setdefault("optional-dependencies", {}),
+                True: self.tool_settings.setdefault("dev-dependencies", {}),
+            }
+            for deps in deps_dict.values():
+                if section in deps:
+                    return deps[section]
+            return deps_dict[dev].setdefault(section, [])
 
     def add_dependencies(
-        self, requirements: Dict[str, Requirement], show_message: bool = True
+        self,
+        requirements: Dict[str, Requirement],
+        to_section: str = "default",
+        dev: bool = False,
+        show_message: bool = True,
     ) -> None:
+        deps = self.get_pyproject_dependencies(to_section, dev)
         for _, dep in requirements.items():
-            deps = self.get_pyproject_dependencies(dep.from_section)
             matched_index = next(
                 (i for i, r in enumerate(deps) if dep.matches(r)), None
             )
@@ -431,23 +479,13 @@ class Project:
             else:
                 req = dep.as_line()
                 deps[matched_index] = req
-                # XXX: This dirty part is for tomlkit.Array.__setitem__()
-                j = 0
-                for i in range(len(deps._value)):
-                    if isinstance(deps._value[i], (Comment, Whitespace)):
-                        continue
-                    if j == matched_index:
-                        deps._value[i] = tomlkit.item(req)
-                        break
-                    j += 1
-            deps.multiline(True)
         self.write_pyproject(show_message)
 
     def write_pyproject(self, show_message: bool = True) -> None:
         with atomic_open_for_write(
             self.pyproject_file.as_posix(), encoding="utf-8"
         ) as f:
-            f.write(tomlkit.dumps(self.pyproject))
+            atoml.dump(self.pyproject, f)
         if show_message:
             self.core.ui.echo(
                 f"Changes are written to {termui.green('pyproject.toml')}."
@@ -457,8 +495,10 @@ class Project:
     @property
     def meta(self) -> Metadata:
         if not self.pyproject:
-            self.pyproject = {"project": tomlkit.table()}
-        return Metadata(self.pyproject_file, self.pyproject.get("project", {}))
+            self.pyproject = {"project": atoml.table()}
+        m = Metadata(self.pyproject_file, self.pyproject.get("project", {}))
+        m._metadata = self.pyproject.get("project", {})
+        return m
 
     def init_global_project(self) -> None:
         if not self.is_global:
@@ -500,7 +540,7 @@ dependencies = ["pip", "setuptools", "wheel"]
 
     def find_interpreters(
         self, python_spec: Optional[str] = None
-    ) -> Iterable[PythonVersion]:
+    ) -> Iterable[PythonInfo]:
         """Return an iterable of interpreter paths that matches the given specifier,
         which can be:
             1. a version specifier like 3.7
@@ -509,36 +549,36 @@ dependencies = ["pip", "setuptools", "wheel"]
             4. None that returns all possible interpreters
         """
         config = self.config
-        PythonVersion.__hash__ = lambda self: hash(self.executable)
+        python: Optional[os.PathLike] = None
 
         if not python_spec:
             if config.get("python.use_pyenv", True) and PYENV_INSTALLED:
-                yield PythonVersion.from_path(
-                    os.path.join(PYENV_ROOT, "shims", "python")
-                )
+                pyenv_shim = os.path.join(PYENV_ROOT, "shims", "python")
+                if os.path.exists(pyenv_shim):
+                    yield PythonInfo.from_path(pyenv_shim)
             if config.get("use_venv"):
                 python = get_in_project_venv_python(self.root)
                 if python:
-                    yield PythonVersion.from_path(python)
+                    yield PythonInfo.from_path(python)
             python = shutil.which("python")
             if python:
-                yield PythonVersion.from_path(python)
-            args = ()
+                yield PythonInfo.from_path(python)
+            args = []
         else:
             if not all(c.isdigit() for c in python_spec.split(".")):
                 if Path(python_spec).exists():
                     python = find_python_in_path(python_spec)
                     if python:
-                        yield PythonVersion.from_path(str(python))
+                        yield PythonInfo.from_path(python)
                 else:
                     python = shutil.which(python_spec)
                     if python:
-                        yield PythonVersion.from_path(python)
+                        yield PythonInfo.from_path(python)
                 return
             args = [int(v) for v in python_spec.split(".") if v != ""]
         finder = Finder()
         for entry in finder.find_all_python_versions(*args):
-            yield entry.py_version
+            yield PythonInfo.from_python_version(entry.py_version)
         if not python_spec:
             this_python = getattr(sys, "_base_executable", sys.executable)
-            yield PythonVersion.from_path(this_python)
+            yield PythonInfo.from_path(this_python)
