@@ -1,4 +1,6 @@
-from typing import Dict, Iterable, List, Optional, Union
+from __future__ import annotations
+
+from typing import Any, Iterable, Iterator, Mapping
 
 from resolvelib import AbstractProvider
 from resolvelib.resolvers import RequirementInformation
@@ -15,40 +17,50 @@ class BaseProvider(AbstractProvider):
         self,
         repository: BaseRepository,
         requires_python: PySpecSet,
-        allow_prereleases: Optional[bool] = None,
+        allow_prereleases: bool | None = None,
     ) -> None:
         self.repository = repository
         self.requires_python = requires_python  # Root python_requires value
         self.allow_prereleases = allow_prereleases  # Root allow_prereleases value
-        self.requires_python_collection: Dict[Optional[str], PySpecSet] = {}
-        self.summary_collection: Dict[str, str] = {}
-        self.fetched_dependencies: Dict[str, List[Requirement]] = {}
+        self.requires_python_collection: dict[str, PySpecSet] = {}
+        self.summary_collection: dict[str, str] = {}
+        self.fetched_dependencies: dict[str, list[Requirement]] = {}
 
-    def identify(self, req: Union[Requirement, Candidate]) -> Optional[str]:
-        return req.identify()
+    def identify(self, requirement_or_candidate: Requirement | Candidate) -> str:
+        return requirement_or_candidate.identify()
 
     def get_preference(
         self,
-        resolution: Candidate,
-        candidates: List[Candidate],
-        information: List[RequirementInformation],
+        identifier: str,
+        resolutions: dict[str, Candidate],
+        candidates: dict[str, Iterator[Candidate]],
+        information: dict[str, Iterator[RequirementInformation]],
     ) -> int:
-        return len(candidates)
+        return sum(1 for _ in candidates[identifier])
 
-    def find_matches(self, requirements: List[Requirement]) -> Iterable[Candidate]:
-        file_req = next((req for req in requirements if not req.is_named), None)
+    def find_matches(
+        self,
+        identifier: str,
+        requirements: Mapping[str, Iterator[Requirement]],
+        incompatibilities: Mapping[str, Iterator[Candidate]],
+    ) -> Iterable[Candidate]:
+        reqs = list(requirements[identifier])
+        file_req = next((req for req in reqs if not req.is_named), None)
+        incompat = list(incompatibilities[identifier])
         if file_req:
             can = Candidate(file_req, self.repository.environment)
             can.get_metadata()
             candidates = [can]
         else:
             candidates = self.repository.find_candidates(
-                requirements[0], self.requires_python, self.allow_prereleases
+                reqs[0],
+                self.requires_python,
+                self.allow_prereleases,
             )
         return [
             can
             for can in candidates
-            if all(self.is_satisfied_by(r, can) for r in requirements)
+            if all(self.is_satisfied_by(r, can) for r in reqs) and can not in incompat
         ]
 
     def is_satisfied_by(self, requirement: Requirement, candidate: Candidate) -> bool:
@@ -62,9 +74,7 @@ class BaseProvider(AbstractProvider):
             candidate.requires_python = str(
                 self.repository.get_dependencies(candidate)[1]
             )
-        allow_prereleases = requirement.allow_prereleases
-        if allow_prereleases is None:
-            allow_prereleases = self.allow_prereleases
+        allow_prereleases = self.allow_prereleases
         if allow_prereleases is None:
             # if not specified, should allow what `find_candidates()` returns
             allow_prereleases = True
@@ -73,15 +83,18 @@ class BaseProvider(AbstractProvider):
             candidate.version, allow_prereleases
         ) and requires_python.is_subset(candidate.requires_python)
 
-    def get_dependencies(self, candidate: Candidate) -> List[Requirement]:
+    def get_dependencies(self, candidate: Candidate) -> list[Requirement]:
         deps, requires_python, summary = self.repository.get_dependencies(candidate)
 
         # Filter out incompatible dependencies(e.g. functools32) early so that
         # we don't get errors when building wheels.
-        valid_deps: List[Requirement] = []
+        valid_deps: list[Requirement] = []
         for dep in deps:
             if (
-                dep.requires_python & requires_python & self.requires_python
+                dep.requires_python
+                & requires_python
+                & self.requires_python
+                & candidate.req.requires_python
             ).is_impossible:
                 continue
             dep.requires_python &= candidate.req.requires_python
@@ -93,7 +106,7 @@ class BaseProvider(AbstractProvider):
         self.requires_python_collection[candidate.req.key] = requires_python
         return valid_deps
 
-    def get_hashes(self, candidate) -> Optional[Dict[str, str]]:
+    def get_hashes(self, candidate: Candidate) -> dict[str, str] | None:
         return self.repository.get_hashes(candidate)
 
 
@@ -105,19 +118,28 @@ class ReusePinProvider(BaseProvider):
     """
 
     def __init__(
-        self, preferred_pins: Dict[str, Candidate], tracked_names: Iterable[str], *args
-    ):
+        self,
+        preferred_pins: dict[str, Candidate],
+        tracked_names: Iterable[str],
+        *args: Any
+    ) -> None:
         super().__init__(*args)
         self.preferred_pins = preferred_pins
         self.tracked_names = set(tracked_names)
 
-    def find_matches(self, requirements: List[Requirement]) -> Iterable[Candidate]:
-        ident = self.identify(requirements[0])
-        if ident not in self.tracked_names and ident in self.preferred_pins:
-            pin = self.preferred_pins[ident]
-            pin._preferred = True
-            yield pin
-        yield from super().find_matches(requirements)
+    def find_matches(
+        self,
+        identifier: str,
+        requirements: Mapping[str, Iterator[Requirement]],
+        incompatibilities: Mapping[str, Iterator[Candidate]],
+    ) -> Iterable[Candidate]:
+        if identifier not in self.tracked_names and identifier in self.preferred_pins:
+            pin = self.preferred_pins[identifier]
+            incompat = list(incompatibilities[identifier])
+            if pin not in incompat:
+                pin._preferred = True
+                yield pin
+        yield from super().find_matches(identifier, requirements, incompatibilities)
 
 
 class EagerUpdateProvider(ReusePinProvider):
@@ -141,7 +163,7 @@ class EagerUpdateProvider(ReusePinProvider):
             return False
         return super().is_satisfied_by(requirement, candidate)
 
-    def get_dependencies(self, candidate: Candidate) -> List[Requirement]:
+    def get_dependencies(self, candidate: Candidate) -> list[Requirement]:
         # If this package is being tracked for upgrade, remove pins of its
         # dependencies, and start tracking these new packages.
         dependencies = super().get_dependencies(candidate)
@@ -153,12 +175,12 @@ class EagerUpdateProvider(ReusePinProvider):
 
     def get_preference(
         self,
-        resolution: Candidate,
-        candidates: List[Candidate],
-        information: List[RequirementInformation],
+        identifier: str,
+        resolutions: dict[str, Candidate],
+        candidates: dict[str, Iterator[Candidate]],
+        information: dict[str, Iterator[RequirementInformation]],
     ) -> int:
         # Resolve tracking packages so we have a chance to unpin them first.
-        name = self.identify(candidates[0])
-        if name in self.tracked_names:
+        if identifier in self.tracked_names:
             return -1
-        return len(candidates)
+        return super().get_preference(identifier, resolutions, candidates, information)
