@@ -1,21 +1,38 @@
 """
 Utility functions
 """
+from __future__ import annotations
+
 import atexit
 import functools
 import os
 import re
 import shutil
 import subprocess
+import sys
 import tempfile
 import urllib.parse as parse
 from contextlib import contextmanager
+from os import PathLike
 from pathlib import Path
-from typing import Any, Callable, Dict, Generic, List, Optional, Tuple, TypeVar, Union
+from re import Match
+from typing import (
+    Any,
+    BinaryIO,
+    Callable,
+    Generic,
+    Iterable,
+    Iterator,
+    TextIO,
+    TypeVar,
+    no_type_check,
+    overload,
+)
 
 from distlib.wheel import Wheel
-from tomlkit.container import Container
-from tomlkit.items import Item
+from pip._vendor.packaging.tags import Tag
+from pip._vendor.pkg_resources import safe_name
+from pip._vendor.requests import Session
 
 from pdm._types import Source
 from pdm.models.pip_shims import (
@@ -26,11 +43,12 @@ from pdm.models.pip_shims import (
     url_to_path,
 )
 
-try:
+if sys.version_info >= (3, 8):
     from functools import cached_property
-except ImportError:
+else:
 
     _T = TypeVar("_T")
+    _C = TypeVar("_C")
 
     class cached_property(Generic[_T]):
         def __init__(self, func: Callable[[Any], _T]):
@@ -38,7 +56,15 @@ except ImportError:
             self.attr_name = func.__name__
             self.__doc__ = func.__doc__
 
-        def __get__(self, inst: Any, cls=None) -> _T:
+        @overload
+        def __get__(self: _C, inst: None, cls: Any = ...) -> _C:
+            ...
+
+        @overload
+        def __get__(self, inst: object, cls: Any = ...) -> _T:
+            ...
+
+        def __get__(self, inst, cls=None):
             if inst is None:
                 return self
             if self.attr_name not in inst.__dict__:
@@ -47,8 +73,8 @@ except ImportError:
 
 
 def prepare_pip_source_args(
-    sources: List[Source], pip_args: Optional[List[str]] = None
-) -> List[str]:
+    sources: list[Source], pip_args: list[str] | None = None
+) -> list[str]:
     if pip_args is None:
         pip_args = []
     if sources:
@@ -57,7 +83,7 @@ def prepare_pip_source_args(
         # Trust the host if it's not verified.
         if not sources[0].get("verify_ssl", True):
             pip_args.extend(
-                ["--trusted-host", parse.urlparse(sources[0]["url"]).hostname]
+                ["--trusted-host", parse.urlparse(sources[0]["url"]).hostname or ""]
             )  # type: ignore
         # Add additional sources as extra indexes.
         if len(sources) > 1:
@@ -66,12 +92,12 @@ def prepare_pip_source_args(
                 # Trust the host if it's not verified.
                 if not source.get("verify_ssl", True):
                     pip_args.extend(
-                        ["--trusted-host", parse.urlparse(source["url"]).hostname]
+                        ["--trusted-host", parse.urlparse(source["url"]).hostname or ""]
                     )  # type: ignore
     return pip_args
 
 
-def get_pypi_source():
+def get_pypi_source() -> tuple[str, bool]:
     """Get what is defined in pip.conf as the index-url."""
     install_cmd = InstallCommand()
     options, _ = install_cmd.parser.parse_args([])
@@ -84,9 +110,10 @@ def get_pypi_source():
 
 
 def get_finder(
-    sources: List[Source],
-    cache_dir: Optional[str] = None,
-    python_version: Optional[Tuple[int, int]] = None,
+    sources: list[Source],
+    cache_dir: str | None = None,
+    python_version: tuple[int, ...] | None = None,
+    python_abi_tag: str | None = None,
     ignore_requires_python: bool = False,
 ) -> PackageFinder:
     install_cmd = InstallCommand()
@@ -98,27 +125,28 @@ def get_finder(
         install_cmd=install_cmd,
         options=options,
         python_version=python_version,
+        python_abi_tag=python_abi_tag,
         ignore_requires_python=ignore_requires_python,
     )
     if not hasattr(finder, "session"):
-        finder.session = finder._link_collector.session
+        finder.session = finder._link_collector.session  # type: ignore
     return finder
 
 
 def create_tracked_tempdir(
-    suffix: Optional[str] = None, prefix: Optional[str] = "", dir: Optional[str] = None
+    suffix: str | None = None, prefix: str | None = None, dir: str | None = None
 ) -> str:
     name = tempfile.mkdtemp(suffix, prefix, dir)
     os.makedirs(name, mode=0o777, exist_ok=True)
 
-    def clean_up():
+    def clean_up() -> None:
         shutil.rmtree(name, ignore_errors=True)
 
     atexit.register(clean_up)
     return name
 
 
-def parse_name_version_from_wheel(filename: str) -> Tuple[str, str]:
+def parse_name_version_from_wheel(filename: str) -> tuple[str, str]:
     w = Wheel(filename)
     return w.name, w.version
 
@@ -127,31 +155,16 @@ def url_without_fragments(url: str) -> str:
     return parse.urlunparse(parse.urlparse(url)._replace(fragment=""))
 
 
-def is_readonly_property(cls, name):
-    """Tell whether a attribute can't be setattr'ed."""
-    attr = getattr(cls, name, None)
-    return attr and isinstance(attr, property) and not attr.fset
-
-
-def join_list_with(items: List[Any], sep: Any) -> List[Any]:
+def join_list_with(items: list[Any], sep: Any) -> list[Any]:
     new_items = []
     for item in items:
         new_items.extend([item, sep])
     return new_items[:-1]
 
 
-def _wheel_supported(self, tags=None):
-    # Ignore current platform. Support everything.
-    return True
-
-
-def _wheel_support_index_min(self, tags=None):
-    # All wheels are equal priority for sorting.
-    return 0
-
-
+@no_type_check
 @contextmanager
-def allow_all_wheels(enable: bool = True):
+def allow_all_wheels(enable: bool = True) -> Iterator:
     """Monkey patch pip.Wheel to allow all wheels
 
     The usual checks against platforms and Python versions are ignored to allow
@@ -165,17 +178,42 @@ def allow_all_wheels(enable: bool = True):
         yield
         return
 
+    def _wheel_supported(self: PipWheel, tags: Iterable[Tag]) -> bool:
+        # Ignore current platform. Support everything.
+        return True
+
+    def _wheel_support_index_min(self: PipWheel, tags: list[Tag]) -> int:
+        # All wheels are equal priority for sorting.
+        return 0
+
+    def _find_most_preferred_tag(
+        self: PipWheel, tags: list[Tag], tag_to_priority: dict[Tag, int]
+    ) -> int:
+        return 0
+
+    has_find_most_preferred_tag = (
+        getattr(PipWheel, "find_most_preferred_tag", None) is not None
+    )
+
     original_wheel_supported = PipWheel.supported
     original_support_index_min = PipWheel.support_index_min
+    if has_find_most_preferred_tag:
+        original_find = PipWheel.find_most_preferred_tag
+    else:
+        original_find = None
 
     PipWheel.supported = _wheel_supported
     PipWheel.support_index_min = _wheel_support_index_min
+    if has_find_most_preferred_tag:
+        PipWheel.find_most_preferred_tag = _find_most_preferred_tag
     yield
     PipWheel.supported = original_wheel_supported
     PipWheel.support_index_min = original_support_index_min
+    if has_find_most_preferred_tag:
+        PipWheel.find_most_preferred_tag = original_find
 
 
-def find_project_root(cwd: str = ".", max_depth: int = 5) -> Optional[str]:
+def find_project_root(cwd: str = ".", max_depth: int = 5) -> str | None:
     """Recursively find a `pyproject.toml` at given path or current working directory.
     If none if found, go to the parent directory, at most `max_depth` levels will be
     looked for.
@@ -192,23 +230,23 @@ def find_project_root(cwd: str = ".", max_depth: int = 5) -> Optional[str]:
     return None
 
 
-def convert_hashes(hashes: Dict[str, str]) -> Dict[str, List[str]]:
+def convert_hashes(hashes: dict[str, str]) -> dict[str, list[str]]:
     """Convert Pipfile.lock hash lines into InstallRequirement option format.
 
     The option format uses a str-list mapping. Keys are hash algorithms, and
     the list contains all values of that algorithm.
     """
-    result: Dict[str, List[str]] = {}
+    result: dict[str, list[str]] = {}
     for hash_value in hashes.values():
         try:
-            name, hash_value = hash_value.split(":")
+            name, hash_value = hash_value.split(":", 1)
         except ValueError:
             name = "sha256"
-        setdefault(result, name, []).append(hash_value)
+        result.setdefault(name, []).append(hash_value)
     return result
 
 
-def get_user_email_from_git() -> Tuple[str, str]:
+def get_user_email_from_git() -> tuple[str, str]:
     """Get username and email from git config.
     Return empty if not configured or git is not found.
     """
@@ -243,7 +281,7 @@ def add_ssh_scheme_to_git_uri(uri: str) -> str:
     return uri
 
 
-def get_in_project_venv_python(root: Path) -> Optional[Path]:
+def get_in_project_venv_python(root: Path) -> Path | None:
     """Get the python interpreter path of venv-in-project"""
     if os.name == "nt":
         suffix = ".exe"
@@ -259,18 +297,19 @@ def get_in_project_venv_python(root: Path) -> Optional[Path]:
 
 
 @contextmanager
-def atomic_open_for_write(filename: Union[Path, str], *, encoding: str = "utf-8"):
+def atomic_open_for_write(
+    filename: PathLike, *, encoding: str = "utf-8"
+) -> Iterator[TextIO]:
     fd, name = tempfile.mkstemp("-atomic-write", "pdm-")
-    filename = str(filename)
+    fp = open(fd, "w", encoding=encoding)
     try:
-        f = open(fd, "w", encoding=encoding)
-        yield f
+        yield fp
     except Exception:
-        f.close()
+        fp.close()
         os.unlink(name)
         raise
     else:
-        f.close()
+        fp.close()
         try:
             os.unlink(filename)
         except OSError:
@@ -279,7 +318,7 @@ def atomic_open_for_write(filename: Union[Path, str], *, encoding: str = "utf-8"
 
 
 @contextmanager
-def cd(path: str):
+def cd(path: str) -> Iterator:
     _old_cwd = os.getcwd()
     os.chdir(path)
     try:
@@ -289,7 +328,7 @@ def cd(path: str):
 
 
 @contextmanager
-def temp_environ():
+def temp_environ() -> Iterator:
     environ = os.environ.copy()
     try:
         yield
@@ -299,7 +338,7 @@ def temp_environ():
 
 
 @contextmanager
-def open_file(url, session=None):
+def open_file(url: str, session: Session | None = None) -> Iterator[BinaryIO]:
     if url.startswith("file://"):
         local_path = url_to_path(url)
         if os.path.isdir(local_path):
@@ -308,15 +347,16 @@ def open_file(url, session=None):
             with open(local_path, "rb") as local_file:
                 yield local_file
     else:
+        assert session
         headers = {"Accept-Encoding": "identity"}
         with session.get(url, headers=headers, stream=True) as resp:
             try:
                 raw = getattr(resp, "raw", None)
-                result = raw if raw else resp
+                result = raw or resp
                 yield result
             finally:
                 if raw:
-                    conn = getattr(raw, "_connection")
+                    conn = getattr(raw, "_connection", None)
                     if conn is not None:
                         conn.close()
                 result.close()
@@ -326,27 +366,17 @@ def populate_link(
     finder: PackageFinder,
     ireq: InstallRequirement,
     upgrade: bool = False,
-):
+) -> None:
     """Populate ireq's link attribute"""
     if not ireq.link:
-        link = finder.find_requirement(ireq, upgrade)
-        if not link:
+        candidate = finder.find_requirement(ireq, upgrade)
+        if not candidate:
             return
-        link = getattr(link, "link", link)
+        link = getattr(candidate, "link", candidate)
         ireq.link = link
 
 
-def setdefault(document: Container, key: str, value: Any) -> Item:
-    """A compatiable dict.setdefault() for tomlkit data structures."""
-    if key not in document:
-        document[key] = value
-    return document[key]
-
-
-def get_python_version_string(version: str, is_64bit: bool) -> str:
-    if os.name == "nt" and not is_64bit:
-        return f"{version}-32"
-    return version
+_VT = TypeVar("_VT")
 
 
 def expand_env_vars(credential: str, quote: bool = False) -> str:
@@ -358,7 +388,7 @@ def expand_env_vars(credential: str, quote: bool = False) -> str:
     Neither $ENV_VAR and %ENV_VAR is not supported.
     """
 
-    def replace_func(match):
+    def replace_func(match: Match) -> str:
         rv = os.getenv(match.group(1), match.group(0))
         return parse.quote(rv) if quote else rv
 
@@ -398,9 +428,10 @@ def is_venv_python(interpreter: os.PathLike) -> bool:
     interpreter = Path(interpreter)
     if interpreter.parent.parent.joinpath("pyvenv.cfg").exists():
         return True
-    if os.getenv("VIRTUAL_ENV"):
+    virtual_env = os.getenv("VIRTUAL_ENV")
+    if virtual_env:
         try:
-            interpreter.relative_to(os.getenv("VIRTUAL_ENV"))
+            interpreter.relative_to(virtual_env)
         except ValueError:
             pass
         else:
@@ -408,13 +439,13 @@ def is_venv_python(interpreter: os.PathLike) -> bool:
     return False
 
 
-def find_python_in_path(path: os.PathLike) -> Optional[Path]:
+def find_python_in_path(path: os.PathLike) -> Path | None:
     """Find a python interpreter from the given path, the input argument could be:
 
     - A valid path to the interpreter
     - A Python root directory that contains the interpreter
     """
-    pathlib_path = Path(path).absolute()
+    pathlib_path = Path(path).resolve()
     if pathlib_path.is_file():
         return pathlib_path
 
@@ -439,3 +470,7 @@ def get_rev_from_url(url: str) -> str:
         _, rev = path.rsplit("@", 1)
         return rev
     return ""
+
+
+def normalize_name(name: str) -> str:
+    return safe_name(name).lower()  # type: ignore
