@@ -6,9 +6,8 @@ from pathlib import Path
 import pytest
 
 from pdm.cli import actions
-from pdm.models.in_process import get_python_version
 from pdm.models.requirements import parse_requirement
-from pdm.utils import temp_environ
+from pdm.utils import cd, temp_environ
 from tests import FIXTURES
 
 
@@ -78,7 +77,7 @@ def test_info_command(project, invoke):
     assert project.root.as_posix() in result.output
 
     result = invoke(["info", "--python"], obj=project)
-    assert result.output.strip() == project.environment.python_executable
+    assert result.output.strip() == project.python.executable
 
     result = invoke(["info", "--where"], obj=project)
     assert result.output.strip() == project.root.as_posix()
@@ -87,11 +86,17 @@ def test_info_command(project, invoke):
     assert result.exit_code == 0
 
 
-def test_info_global_project(invoke):
-    result = invoke(["info", "-g", "--where"])
+def test_info_global_project(invoke, tmp_path):
+    with cd(tmp_path):
+        result = invoke(["info", "-g", "--where"])
     assert "global-project" in result.output.strip()
 
 
+def test_global_project_other_location(invoke, project):
+    result = invoke(["info", "-g", "-p", project.root.as_posix(), "--where"])
+    assert result.stdout.strip() == project.root.as_posix()
+
+
 def test_uncaught_error(invoke, mocker):
     mocker.patch.object(actions, "do_list", side_effect=RuntimeError("test error"))
     result = invoke(["list"])
@@ -129,30 +134,68 @@ def test_install_with_lockfile(project, invoke, working_set, repository):
     result = invoke(["install"], obj=project)
     assert "Lock file" not in result.output
 
-    project.add_dependencies({"pytz": parse_requirement("pytz")})
+    project.add_dependencies({"pytz": parse_requirement("pytz")}, "default")
     result = invoke(["install"], obj=project)
     assert "Lock file hash doesn't match" in result.output
-    assert "pytz" in project.get_locked_candidates()
+    assert "pytz" in project.locked_repository.all_candidates
     assert project.is_lockfile_hash_match()
 
 
 def test_init_command(project_no_init, invoke, mocker):
+    mocker.patch(
+        "pdm.cli.commands.init.get_user_email_from_git",
+        return_value=("Testing", "me@example.org"),
+    )
+    do_init = mocker.patch.object(actions, "do_init")
+    result = invoke(["init"], input="\n\n\n\n\n\n", obj=project_no_init)
+    assert result.exit_code == 0
+    python_version = f"{project_no_init.python.major}.{project_no_init.python.minor}"
+    do_init.assert_called_with(
+        project_no_init,
+        "",
+        "",
+        "MIT",
+        "Testing",
+        "me@example.org",
+        f">={python_version}",
+    )
+
+
+def test_init_command_library(project_no_init, invoke, mocker):
     mocker.patch(
         "pdm.cli.commands.init.get_user_email_from_git",
         return_value=("Testing", "me@example.org"),
     )
     do_init = mocker.patch.object(actions, "do_init")
     result = invoke(
-        ["init"], input="python\ntest-project\n\n\n\n\n\n", obj=project_no_init
+        ["init"], input="\ny\ntest-project\n\n\n\n\n\n", obj=project_no_init
     )
     assert result.exit_code == 0
-    python_version, _ = get_python_version(
-        project_no_init.environment.python_executable, True, 2
-    )
+    python_version = f"{project_no_init.python.major}.{project_no_init.python.minor}"
     do_init.assert_called_with(
         project_no_init,
         "test-project",
-        "0.0.0",
+        "0.1.0",
+        "MIT",
+        "Testing",
+        "me@example.org",
+        f">={python_version}",
+    )
+
+
+def test_init_non_interactive(project_no_init, invoke, mocker):
+    mocker.patch(
+        "pdm.cli.commands.init.get_user_email_from_git",
+        return_value=("Testing", "me@example.org"),
+    )
+    do_init = mocker.patch.object(actions, "do_init")
+    result = invoke(["init", "-n"], obj=project_no_init)
+    assert result.exit_code == 0
+    python_version = f"{project_no_init.python.major}.{project_no_init.python.minor}"
+    do_init.assert_called_with(
+        project_no_init,
+        "",
+        "",
         "MIT",
         "Testing",
         "me@example.org",
@@ -192,6 +235,21 @@ def test_config_set_command(project, invoke):
     assert result.exit_code != 0
 
 
+def test_config_del_command(project, invoke):
+
+    result = invoke(["config", "-l", "python.use_pyenv", "false"], obj=project)
+    assert result.exit_code == 0
+
+    result = invoke(["config", "python.use_pyenv"], obj=project)
+    assert result.output.strip() == "False"
+
+    result = invoke(["config", "-ld", "python.use_pyenv"], obj=project)
+    assert result.exit_code == 0
+
+    result = invoke(["config", "python.use_pyenv"], obj=project)
+    assert result.output.strip() == "True"
+
+
 def test_config_env_var_shadowing(project, invoke):
     with temp_environ():
         os.environ["PDM_PYPI_URL"] = "https://example.org/simple"
@@ -218,13 +276,6 @@ def test_config_project_global_precedence(project, invoke):
     assert result.output.strip() == "/path/to/bar"
 
 
-def test_cache_clear_command(project, invoke, mocker):
-    m = mocker.patch("shutil.rmtree")
-    result = invoke(["cache", "clear"], obj=project)
-    assert result.exit_code == 0
-    m.assert_called_once()
-
-
 @pytest.mark.parametrize(
     "filename",
     [
@@ -237,10 +288,20 @@ def test_cache_clear_command(project, invoke, mocker):
 def test_import_other_format_file(project, invoke, filename):
     requirements_file = FIXTURES / filename
     result = invoke(["import", str(requirements_file)], obj=project)
-    print(result.stderr)
     assert result.exit_code == 0
 
 
+def test_import_requirement_no_overwrite(project, invoke, tmp_path):
+    project.add_dependencies({"requests": parse_requirement("requests")})
+    tmp_path.joinpath("reqs.txt").write_text("flask\nflask-login\n")
+    result = invoke(
+        ["import", "-dsweb", str(tmp_path.joinpath("reqs.txt"))], obj=project
+    )
+    assert result.exit_code == 0, result.stderr
+    assert list(project.get_dependencies()) == ["requests"]
+    assert list(project.get_dependencies("web")) == ["flask", "flask-login"]
+
+
 @pytest.mark.pypi
 def test_search_package(project, invoke):
     result = invoke(["search", "requests"], obj=project)
@@ -266,6 +327,7 @@ def test_export_to_requirements_txt(invoke, fixture_project):
     requirements_pyproject = project.root / "requirements.ini"
 
     result = invoke(["export"], obj=project)
+    print("==========OUTPUT=============", result.output.strip(), result.stderr.strip())
     assert result.exit_code == 0
     assert result.output.strip() == requirements_txt.read_text().strip()
 
@@ -296,4 +358,4 @@ def test_lock_legacy_project(invoke, fixture_project, repository):
     project = fixture_project("demo-legacy")
     result = invoke(["lock"], obj=project)
     assert result.exit_code == 0
-    assert "urllib3" in project.get_locked_candidates()
+    assert "urllib3" in project.locked_repository.all_candidates
