@@ -1,6 +1,9 @@
+import functools
 import os
 import subprocess
 import textwrap
+from pathlib import Path
+from tempfile import TemporaryDirectory
 
 import pytest
 
@@ -19,7 +22,7 @@ def test_pep582_launcher_for_python_interpreter(project, invoke):
     env = os.environ.copy()
     env.update({"PYTHONPATH": PEP582_PATH})
     output = subprocess.check_output(
-        [project.environment.python_executable, str(project.root.joinpath("main.py"))],
+        [project.python.executable, str(project.root.joinpath("main.py"))],
         env=env,
     )
     assert output.decode().strip() == "2.24.0"
@@ -43,9 +46,37 @@ def test_run_cmd_script(project, invoke):
     assert result.exit_code == 0
 
 
+def test_run_cmd_script_with_array(project, invoke):
+    project.tool_settings["scripts"] = {
+        "test_script": ["python", "-c", "import sys; sys.exit(22)"]
+    }
+    project.write_pyproject()
+    result = invoke(["run", "test_script"], obj=project)
+    assert result.exit_code == 22
+
+
+def test_run_script_pass_project_root(project, invoke, capfd):
+    project.tool_settings["scripts"] = {
+        "test_script": [
+            "python",
+            "-c",
+            "import os;print(os.getenv('PDM_PROJECT_ROOT'))",
+        ]
+    }
+    project.write_pyproject()
+    capfd.readouterr()
+    result = invoke(["run", "test_script"], obj=project)
+    assert result.exit_code == 0
+    out, _ = capfd.readouterr()
+    assert Path(out.strip()) == project.root
+
+
 def test_run_shell_script(project, invoke):
     project.tool_settings["scripts"] = {
-        "test_script": {"shell": "echo hello > output.txt"}
+        "test_script": {
+            "shell": "echo hello > output.txt",
+            "help": "test it won't fail",
+        }
     }
     project.write_pyproject()
     with cd(project.root):
@@ -105,6 +136,7 @@ def test_run_expand_env_vars(project, invoke, capfd):
         "test_cmd": 'python -c "foo, bar = 0, 1;print($FOO)"',
         "test_cmd_no_expand": "python -c 'print($FOO)'",
         "test_script": "python test_script.py",
+        "test_cmd_array": ["python", "test_script.py"],
         "test_shell": {"shell": "echo $FOO"},
     }
     project.write_pyproject()
@@ -120,6 +152,9 @@ def test_run_expand_env_vars(project, invoke, capfd):
         invoke(["run", "test_script"], obj=project)
         assert capfd.readouterr()[0].strip() == "bar"
 
+        invoke(["run", "test_cmd_array"], obj=project)
+        assert capfd.readouterr()[0].strip() == "bar"
+
         invoke(["run", "test_shell"], obj=project)
         assert capfd.readouterr()[0].strip() == "bar"
 
@@ -180,3 +215,56 @@ def test_run_show_list_of_scripts(project, invoke):
         == "test_script call  test_script:main call a python function"
     )
     assert result_lines[2].strip() == "test_shell  shell echo $FOO        shell command"
+
+
+def test_run_with_another_project_root(project, invoke, capfd):
+    project.meta["requires-python"] = ">=3.6"
+    project.write_pyproject()
+    invoke(["add", "requests==2.24.0"], obj=project)
+    with TemporaryDirectory(prefix="pytest-run-") as tmp_dir:
+        Path(tmp_dir).joinpath("main.py").write_text(
+            "import requests\nprint(requests.__version__)\n"
+        )
+        capfd.readouterr()
+        with cd(tmp_dir):
+            ret = invoke(["run", "-p", str(project.root), "python", "main.py"])
+            assert ret.exit_code == 0
+            out, _ = capfd.readouterr()
+            assert out.strip() == "2.24.0"
+
+
+def test_import_another_sitecustomize(project, invoke, capfd):
+    project.meta["requires-python"] = ">=2.7"
+    project.write_pyproject()
+    # a script for checking another sitecustomize is imported
+    project.root.joinpath("foo.py").write_text(
+        textwrap.dedent(
+            """
+            import sys
+            module = sys.modules.get('another_sitecustomize')
+            if module:
+                print(module.__file__)
+            """
+        )
+    )
+    # ensure there have at least one sitecustomize can be imported
+    # there may have more than one sitecustomize.py in sys.path
+    project.root.joinpath("sitecustomize.py").write_text("# do nothing")
+    env = os.environ.copy()
+    paths = [str(project.root)]
+    original_paths = env.get("PYTHONPATH", "")
+    if original_paths:
+        paths.insert(0, original_paths)
+    env["PYTHONPATH"] = os.pathsep.join(paths)
+    # invoke pdm commands
+    invoke = functools.partial(invoke, env=env, obj=project)
+    project._environment = None
+    capfd.readouterr()
+    with cd(project.root):
+        invoke(["run", "python", "foo.py"])
+    # only the first and second sitecustomize module will be imported
+    # as sitecustomize and another_sitecustomize
+    # the first one is pdm.pep582.sitecustomize for sure
+    # the second one maybe not the dummy module injected here
+    out, _ = capfd.readouterr()
+    assert out.strip()
