@@ -14,11 +14,11 @@ import pytest
 from click.testing import CliRunner
 from pip._internal.vcs import versioncontrol
 from pip._vendor import requests
-from pip._vendor.pkg_resources import safe_name
+from pip._vendor.pkg_resources import WorkingSet
 
 from pdm._types import CandidateInfo
 from pdm.cli.actions import do_init, do_use
-from pdm.core import main
+from pdm.core import Core
 from pdm.exceptions import CandidateInfoNotFound
 from pdm.models.candidates import Candidate
 from pdm.models.environment import Environment
@@ -26,10 +26,11 @@ from pdm.models.repositories import BaseRepository
 from pdm.models.requirements import Requirement, filter_requirements_with_extras
 from pdm.project import Project
 from pdm.project.config import Config
-from pdm.utils import cached_property, get_finder, temp_environ
+from pdm.utils import cached_property, get_finder, normalize_name, temp_environ
 from tests import FIXTURES
 
 os.environ["CI"] = "1"
+main = Core()
 
 
 class LocalFileAdapter(requests.adapters.BaseAdapter):
@@ -41,9 +42,7 @@ class LocalFileAdapter(requests.adapters.BaseAdapter):
     def send(
         self, request, stream=False, timeout=None, verify=True, cert=None, proxies=None
     ):
-        file_path = self.base_path / urlparse(request.url).path.lstrip(
-            "/"
-        )  # type: Path
+        file_path = self.base_path / urlparse(request.url).path.lstrip("/")
         response = requests.models.Response()
         response.request = request
         if not file_path.exists():
@@ -86,13 +85,13 @@ class TestRepository(BaseRepository):
         self.load_fixtures()
 
     def add_candidate(self, name, version, requires_python=""):
-        pypi_data = self._pypi_data.setdefault(safe_name(name), {}).setdefault(
+        pypi_data = self._pypi_data.setdefault(normalize_name(name), {}).setdefault(
             version, {}
         )
         pypi_data["requires_python"] = requires_python
 
     def add_dependencies(self, name, version, requirements):
-        pypi_data = self._pypi_data[safe_name(name)][version]
+        pypi_data = self._pypi_data[normalize_name(name)][version]
         pypi_data.setdefault("dependencies", []).extend(requirements)
 
     def _get_dependencies_from_fixture(
@@ -136,19 +135,24 @@ class TestRepository(BaseRepository):
 
 
 class TestProject(Project):
-    def __init__(self, root_path):
-        self.GLOBAL_PROJECT = Path(root_path) / ".pdm-home" / "global-project"
-        super().__init__(root_path)
+    def __init__(self, core, root_path, is_global):
+        self.root_path = Path(root_path or ".")
+        self.GLOBAL_PROJECT = self.root_path / ".pdm-home" / "global-project"
+        super().__init__(core, root_path, is_global)
 
     @cached_property
     def global_config(self):
-        return Config(self.root / ".pdm-home" / "config.toml", is_global=True)
+        return Config(self.root_path / ".pdm-home" / "config.toml", is_global=True)
+
+
+main.project_class = TestProject
 
 
 class Distribution:
-    def __init__(self, key, version):
+    def __init__(self, key, version, editable=False):
         self.key = key
         self.version = version
+        self.editable = editable
         self.dependencies = []
 
     def requires(self, extras=()):
@@ -157,7 +161,7 @@ class Distribution:
 
 class MockWorkingSet(collections.abc.MutableMapping):
     def __init__(self, *args, **kwargs):
-        self.pkg_ws = None
+        self.pkg_ws = WorkingSet([])
         self._data = {}
 
     def add_distribution(self, dist):
@@ -189,8 +193,8 @@ def working_set(mocker, repository):
     def install(candidate):
         pip_logging._log_state.indentation = 0
         dependencies = repository.get_dependencies(candidate)[0]
-        key = safe_name(candidate.name).lower()
-        dist = Distribution(key, candidate.version)
+        key = normalize_name(candidate.name)
+        dist = Distribution(key, candidate.version, candidate.req.editable)
         dist.dependencies = dependencies
         rv.add_distribution(dist)
 
@@ -220,10 +224,16 @@ def pip_global_tempdir_manager():
         yield
 
 
+def remove_pep582_path_from_pythonpath(pythonpath):
+    """Remove all pep582 paths of PDM from PYTHONPATH"""
+    paths = pythonpath.split(os.pathsep)
+    paths = [path for path in paths if "pdm/pep582" not in path]
+    return os.pathsep.join(paths)
+
+
 @pytest.fixture()
 def project_no_init(tmp_path, mocker):
-    p = TestProject(tmp_path.as_posix())
-    p.core = main
+    p = main.create_project(tmp_path)
     mocker.patch("pdm.utils.get_finder", get_local_finder)
     mocker.patch("pdm.models.environment.get_finder", get_local_finder)
     mocker.patch("pdm.project.core.Config.HOME_CONFIG", tmp_path)
@@ -232,7 +242,11 @@ def project_no_init(tmp_path, mocker):
     do_use(p, getattr(sys, "_base_executable", sys.executable))
     with temp_environ():
         os.environ.pop("VIRTUAL_ENV", None)
-        os.environ.pop("PYTHONPATH", None)
+        os.environ.pop("PEP582_PACKAGES", None)
+        pythonpath = os.environ.pop("PYTHONPATH", "")
+        pythonpath = remove_pep582_path_from_pythonpath(pythonpath)
+        if pythonpath:
+            os.environ["PYTHONPATH"] = pythonpath
         yield p
     # Restore the config items
     Config._config_map = old_config_map
@@ -242,7 +256,7 @@ def project_no_init(tmp_path, mocker):
 def project(project_no_init):
     do_init(project_no_init, "test_project", "0.0.0")
     # Clean the cached property
-    project_no_init.__dict__.pop("environment", None)
+    project_no_init._environment = None
     return project_no_init
 
 
@@ -293,3 +307,8 @@ def is_dev(request):
 def invoke():
     runner = CliRunner(mix_stderr=False)
     return functools.partial(runner.invoke, main, prog_name="pdm")
+
+
+@pytest.fixture()
+def core():
+    return main
