@@ -1,7 +1,8 @@
 import pytest
 from resolvelib.resolvers import ResolutionImpossible, Resolver
 
-from pdm.iostream import stream
+from pdm import termui
+from pdm.cli.actions import resolve_candidates_from_lockfile
 from pdm.models.requirements import parse_requirement
 from pdm.models.specifiers import PySpecSet
 from pdm.resolver import resolve
@@ -39,8 +40,8 @@ def resolve_requirements(
             requires_python,
             allow_prereleases,
         )
-
-    with stream.open_spinner("Resolving dependencies") as spin, stream.logging("lock"):
+    ui = termui.UI()
+    with ui.open_spinner("Resolving dependencies") as spin, ui.logging("lock"):
         reporter = SpinnerReporter(spin, requirements)
         resolver = Resolver(provider, reporter)
         mapping, *_ = resolve(resolver, requirements, requires_python)
@@ -128,11 +129,11 @@ def test_resolve_vcs_and_local_requirements(
 def test_resolve_local_and_named_requirement(project, repository, vcs):
     requirements = ["demo", "git+https://github.com/test-root/demo.git#egg=demo"]
     result = resolve_requirements(repository, requirements, ">=3.6")
-    assert result["demo"].req.repo == "https://github.com/test-root/demo.git"
+    assert result["demo"].req.is_vcs
 
     requirements = ["git+https://github.com/test-root/demo.git#egg=demo", "demo"]
     result = resolve_requirements(repository, requirements, ">=3.6")
-    assert result["demo"].req.repo == "https://github.com/test-root/demo.git"
+    assert result["demo"].req.is_vcs
 
 
 def test_resolving_auto_avoid_conflicts(project, repository):
@@ -215,17 +216,6 @@ def test_resolve_dependency_with_extra_marker(project, repository):
     assert "pytz" in result
 
 
-def test_resolve_parent_from_multiple_sources(project, repository):
-    repository.add_candidate("foo", "0.1.0")
-    repository.add_dependencies("foo", "0.1.0", ["django"])
-    repository.add_candidate("bar", "0.1.0")
-    repository.add_dependencies("bar", "0.1.0", ["django"])
-    result = resolve_requirements(
-        repository, ["foo; python_version ~= '3.8'", "bar"], ">=3.6"
-    )
-    assert not result["pytz"].marker
-
-
 def test_resolve_circular_dependencies(project, repository):
     repository.add_candidate("foo", "0.1.0")
     repository.add_dependencies("foo", "0.1.0", ["foobar"])
@@ -234,3 +224,46 @@ def test_resolve_circular_dependencies(project, repository):
     result = resolve_requirements(repository, ["foo"])
     assert result["foo"].version == "0.1.0"
     assert result["foobar"].version == "0.2.0"
+
+
+def test_resolve_candidates_to_install(project):
+    project.lockfile = {
+        "package": [
+            {
+                "name": "pytest",
+                "version": "4.6.0",
+                "summary": "pytest module",
+                "dependencies": ["py>=3.0", "configparser; sys_platform=='win32'"],
+            },
+            {
+                "name": "configparser",
+                "version": "1.2.0",
+                "summary": "configparser module",
+                "dependencies": ["backports"],
+            },
+            {
+                "name": "py",
+                "version": "3.6.0",
+                "summary": "py module",
+            },
+            {
+                "name": "backports",
+                "version": "2.2.0",
+                "summary": "backports module",
+            },
+        ]
+    }
+    project.environment.marker_environment["sys_platform"] = "linux"
+    reqs = [parse_requirement("pytest")]
+    result = resolve_candidates_from_lockfile(project, reqs)
+    assert result["pytest"].version == "4.6.0"
+    assert result["py"].version == "3.6.0"
+    assert "configparser" not in result
+    assert "backports" not in result
+
+    project.environment.marker_environment["sys_platform"] = "win32"
+    result = resolve_candidates_from_lockfile(project, reqs)
+    assert result["pytest"].version == "4.6.0"
+    assert result["py"].version == "3.6.0"
+    assert result["configparser"].version == "1.2.0"
+    assert result["backports"].version == "2.2.0"
