@@ -8,24 +8,31 @@ import pytest
 
 from pdm.models.requirements import filter_requirements_with_extras
 from pdm.pep517.api import build_wheel
-from pdm.project import Project
-from pdm.utils import cd
+from pdm.utils import cd, temp_environ
 
 
 def test_project_python_with_pyenv_support(project, mocker):
-    from pythonfinder.environment import PYENV_ROOT
 
     del project.project_config["python.path"]
-    pyenv_python = os.path.join(PYENV_ROOT, "shims", "python")
+    project._python = None
+    with temp_environ():
+        os.environ["PDM_IGNORE_SAVED_PYTHON"] = "1"
+        mocker.patch("pdm.project.core.PYENV_INSTALLED", True)
+        mocker.patch("pdm.project.core.PYENV_ROOT", str(project.root))
+        pyenv_python = project.root / "shims/python"
+        pyenv_python.parent.mkdir()
+        pyenv_python.touch()
+        mocker.patch(
+            "pythonfinder.models.python.get_python_version",
+            return_value="3.8.0",
+        )
+        assert Path(project.python.executable) == pyenv_python
 
-    mocker.patch("pdm.models.environment.PYENV_INSTALLED", True)
-    assert project.environment.python_executable == pyenv_python
+        # Clean cache
+        project._python = None
 
-    # Clean cache
-    del project.environment.__dict__["python_executable"]
-
-    project.project_config["python.use_pyenv"] = False
-    assert project.environment.python_executable != pyenv_python
+        project.project_config["python.use_pyenv"] = False
+        assert Path(project.python.executable) != pyenv_python
 
 
 def test_project_config_items(project):
@@ -52,14 +59,22 @@ def test_project_sources_overriding(project):
     assert project.sources[0]["url"] == "https://example.org/simple"
 
 
-def test_global_project(tmp_path):
-    project = Project.create_global(tmp_path.as_posix())
-    project.init_global_project()
+def test_global_project(tmp_path, core):
+    project = core.create_project(tmp_path, True)
     assert project.environment.is_global
 
 
+def test_auto_global_project(tmp_path, core):
+    tmp_path.joinpath(".pdm-home").mkdir()
+    (tmp_path / ".pdm-home/config.toml").write_text("auto_global = true\n")
+    with cd(tmp_path):
+        project = core.create_project()
+    assert project.is_global
+
+
 def test_project_use_venv(project):
     del project.project_config["python.path"]
+    project._python = None
     scripts = "Scripts" if os.name == "nt" else "bin"
     suffix = ".exe" if os.name == "nt" else ""
     venv.create(project.root / "venv")
@@ -67,7 +82,7 @@ def test_project_use_venv(project):
     project.project_config["use_venv"] = True
     env = project.environment
     assert (
-        Path(env.python_executable)
+        Path(env.interpreter.executable)
         == project.root / "venv" / scripts / f"python{suffix}"
     )
     assert env.is_global
@@ -104,8 +119,47 @@ def test_project_auto_detect_venv(project):
     suffix = ".exe" if os.name == "nt" else ""
 
     project.project_config["use_venv"] = True
+    project._python = None
     project.project_config["python.path"] = (
         project.root / "test_venv" / scripts / f"python{suffix}"
     ).as_posix()
 
     assert project.environment.is_global
+
+
+def test_ignore_saved_python(project):
+    project.project_config["use_venv"] = True
+    project._python = None
+    scripts = "Scripts" if os.name == "nt" else "bin"
+    suffix = ".exe" if os.name == "nt" else ""
+    venv.create(project.root / "venv")
+    with temp_environ():
+        os.environ["PDM_IGNORE_SAVED_PYTHON"] = "1"
+        assert Path(project.python.executable) != project.project_config["python.path"]
+        assert (
+            Path(project.python.executable)
+            == project.root / "venv" / scripts / f"python{suffix}"
+        )
+
+
+def test_select_dependencies(project):
+    project.meta["dependencies"] = ["requests"]
+    project.meta["optional-dependencies"] = {
+        "security": ["cryptography"],
+        "venv": ["virtualenv"],
+    }
+    project.tool_settings["dev-dependencies"] = {"test": ["pytest"], "doc": ["mkdocs"]}
+    assert sorted(project.get_dependencies()) == ["requests"]
+    assert sorted(project.dependencies) == ["requests"]
+
+    assert sorted(project.get_dependencies("security")) == ["cryptography"]
+    assert sorted(project.get_dependencies("test")) == ["pytest"]
+    assert sorted(project.dev_dependencies) == ["mkdocs", "pytest"]
+
+    assert sorted(project.iter_sections()) == [
+        "default",
+        "doc",
+        "security",
+        "test",
+        "venv",
+    ]
